<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class DashboardFacilitiesFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getDadosLimpezaFinalizada($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3'
                       AND FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1'
                       $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                  $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
    
    public function getDadosTarefaAtrasada($parametros, $listaOperadores, $filtroTipo)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
               CASE
                WHEN SYSDATE < FCM_CHECKLISTAGEND.DTHORAAGEND THEN ' '  
                WHEN TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) < 1 THEN 
                    GREATEST(FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24), 0) || ' horas ' || 
                    GREATEST(FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24 * 60) - (FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24) * 60), 0) || ' minutos'
                ELSE 
                    TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) || ' dias'
                END AS TEMPO, 
                FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1')
                       AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE
                       $filtroTipo";
    
        // Se houver filtro
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
            $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaConcluida($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' 
                       AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaEmAndamento($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaPendente($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' 
                       AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaCancelado($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDadosTarefaPendenteAprovacao($parametros, $listaOperadores, $filtroTipo)
    {

        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.CDOPERADOR, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, CLIENTE.CDCLIENTE,
              TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTEPAI AS LOCALPAI, FCM_CHECKLIST.NMCHECKLIST, CLIENTE.NMFANTCLIE, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
              FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, OPERADOR.NMOPERADOR, FCM_EQUIPE.NMEQUIPE
                FROM 
                    FCM_CHECKLISTAGEND
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                JOIN 
                    FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                LEFT JOIN 
                    FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ON FCM_CHECKLISTAGEND.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN 
                    FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    FCM_EQUIPE ON FCM_EQUIPE.NREQUIPE = FCM_EQUIPETAREFA.NREQUIPE
                    WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                        OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                        OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                            (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                            FROM FCM_EQUIPE
                            JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                            AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                            WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                        OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                       AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
                       AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6'
                       $filtroTipo";

              // Se houver filtro
              if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
              }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }


    public function getNaoConformidades($parametros, $listaOperadores) 
    {
        $sql = "SELECT CTGC.NRCRITERIOTAREFACHK, CTGC.NMCRITERIOTARGERCHK, COUNT(CTGC.NRCRITERIOTAREFACHK) AS QTDADE
              FROM FCM_CRITERIOTARGERCHK CTGC
              WHERE IDMARCADO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND CTGC.DTULTATU BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY CTGC.NRCRITERIOTAREFACHK, CTGC.NMCRITERIOTARGERCHK";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getNaoConformidadesPorLocal($parametros, $listaOperadores) 
    {
        $sql = "SELECT TP.NMTIPOLOCALCLI, 
                   COUNT(CRIT.NRCRITERIOTAREFACHK) AS QTD
              FROM FCM_GERENCIAMENTOCHK GECHK
              JOIN FCM_TAREFAGERCHK TA
                ON GECHK.NRGERENCIAMENTOCHK = TA.NRGERENCIAMENTOCHK
              JOIN FCM_CRITERIOTARGERCHK CRIT
                ON CRIT.NRTAREFAGERCHK = TA.NRTAREFAGERCHK
              JOIN FCM_LOCALCLI LOC
                ON LOC.NRLOCALCLIENTE = GECHK.NRLOCALCLIENTE
              JOIN FCM_TIPOLOCALCLI TP
                ON TP.NRTIPOLOCALCLI = LOC.NRTIPOLOCALCLI
              WHERE CRIT.IDMARCADO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND CRIT.DTULTATU BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY TP.NMTIPOLOCALCLI";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTarefasPorCliente($parametros, $listaOperadores) 
    {
        $sql = "SELECT 
            CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3' THEN 1 END) AS total_concluidas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1') AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '1' THEN 1 END) AS total_finalizadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 1 END) AS total_andamento,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 1 END) AS total_cancelados,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 1 END) AS total_pendentesaprovacao
        FROM 
            FCM_CHECKLISTAGEND
        JOIN 
            FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
        LEFT JOIN
            FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        JOIN 
            CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
        LEFT JOIN 
            FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                    OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                    OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                        (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                        FROM FCM_EQUIPE
                        JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                        AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                        WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                    OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                    AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY 
        CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTarefasPorLocal($parametros, $listaOperadores) 
    {
        $sql = "SELECT 
            FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3' THEN 1 END) AS total_concluidas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1') AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '1' THEN 1 END) AS total_finalizadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 1 END) AS total_andamento,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 1 END) AS total_cancelados,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 1 END) AS total_pendentesaprovacao
        FROM 
            FCM_CHECKLISTAGEND
        JOIN 
            FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
        LEFT JOIN
            FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        LEFT JOIN 
            FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                    OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                    OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                        (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                        FROM FCM_EQUIPE
                        JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                        AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                        WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                    OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                    AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY 
        FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
    
    public function getTarefasPorChecklist($parametros, $listaOperadores) 
    {
        $sql = "SELECT 
            FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '3' THEN 1 END) AS total_concluidas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('5', '1') AND FCM_CHECKLISTAGEND.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '3' AND COALESCE(FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, '1') = '1' THEN 1 END) AS total_finalizadas,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '2' THEN 1 END) AS total_andamento,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '1' AND FCM_CHECKLISTAGEND.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '4' THEN 1 END) AS total_cancelados,
            COUNT(CASE WHEN FCM_CHECKLISTAGEND.IDSITUACAOAGEND = '6' THEN 1 END) AS total_pendentesaprovacao
        FROM 
            FCM_CHECKLISTAGEND
        JOIN
            FCM_CHECKLIST ON FCM_CHECKLISTAGEND.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
        LEFT JOIN
            FCM_EQUIPETAREFA ON FCM_EQUIPETAREFA.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        LEFT JOIN 
            FCM_GERENCIAMENTOCHK ON FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
        WHERE ((FCM_CHECKLISTAGEND.CDOPERADOR IN ($listaOperadores)) -- Tarefas criadas para operador específico
                    OR ((FCM_CHECKLISTAGEND.CDOPERADOR IS NULL) AND (FCM_CHECKLISTAGEND.IDTAREFAEQUIPE = 'N')) -- Tarefas sem operador
                    OR (FCM_EQUIPETAREFA.NREQUIPE IN 
                        (SELECT DISTINCT FCM_EQUIPE.NREQUIPE 
                        FROM FCM_EQUIPE
                        JOIN FCM_RELACEQUIPEOPER ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                        AND FCM_RELACEQUIPEOPER.NRORG = FCM_EQUIPE.NRORG
                        WHERE FCM_RELACEQUIPEOPER.CDOPERADOR IN ($listaOperadores))) -- Tarefas de equipe
                    OR FCM_CHECKLISTAGEND.CDOPERINCLUSAO = :CDOPERADOR) -- Operador de inclusão
                    AND FCM_CHECKLISTAGEND.IDATIVO = 'S'";

        // Se houver filtro por data
        if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
          $sql .= " AND FCM_CHECKLISTAGEND.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
        }
        
        $sql .= " GROUP BY 
        FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPendentes($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' 
                AND 
                    FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
    
    public function getVistoriasAtrasadas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, 
                    CASE
                        WHEN SYSDATE < FCM_GERENCIAMENTOCHK.DTHORAAGEND THEN ' '  
                        WHEN TRUNC(SYSDATE) - TRUNC(FCM_GERENCIAMENTOCHK.DTHORAAGEND) < 1 THEN 
                            GREATEST(FLOOR((SYSDATE - FCM_GERENCIAMENTOCHK.DTHORAAGEND) * 24), 0) || ' horas ' || 
                            GREATEST(FLOOR((SYSDATE - FCM_GERENCIAMENTOCHK.DTHORAAGEND) * 24 * 60) - (FLOOR((SYSDATE - FCM_GERENCIAMENTOCHK.DTHORAAGEND) * 24) * 60), 0) || ' minutos'
                        ELSE 
                            TRUNC(SYSDATE) - TRUNC(FCM_GERENCIAMENTOCHK.DTHORAAGEND) || ' dias'
                        END AS TEMPO, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' 
                AND 
                    FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasAvaliadas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3'
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasEmAndamento($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2')
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasCancelado($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE,
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_GERENCIAMENTOCHK.DTHORAAGEND, FCM_CHECKLIST.NMCHECKLIST, OPERADOR.NMOPERADOR
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_GERENCIAMENTOCHK.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE 
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ({$hierarchyParam}) OR FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5'
                $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPorCliente($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 1 END) AS total_avaliadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2') THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 1 END) AS total_cancelado
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                WHERE (
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR AND FCM_GERENCIAMENTOCHK.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPorChecklist($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 1 END) AS total_avaliadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2') THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 1 END) AS total_cancelado
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN
                    FCM_CHECKLIST ON FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
                WHERE (
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR AND FCM_GERENCIAMENTOCHK.IDATIVO = 'S')
                    )";

                    
                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_CHECKLIST.NMCHECKLIST, FCM_CHECKLIST.NRCHECKLIST";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getVistoriasPorLocal($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND >= SYSDATE THEN 1 END) AS total_pendentes,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '1' AND FCM_GERENCIAMENTOCHK.DTHORAAGEND <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '3' THEN 1 END) AS total_avaliadas,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK IN ('4', '2') THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK = '5' THEN 1 END) AS total_cancelado
                FROM 
                    FCM_GERENCIAMENTOCHK
                JOIN 
                    FCM_LOCALCLI ON FCM_GERENCIAMENTOCHK.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                WHERE (
                    (FCM_GERENCIAMENTOCHK.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_GERENCIAMENTOCHK.CDOPERINCLUSAO = :CDOPERADOR AND FCM_GERENCIAMENTOCHK.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_GERENCIAMENTOCHK.DTHORAAGEND BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasAtrasadas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '1' 
                AND 
                    FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasEmAndamento($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '2'
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasConcluidas($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '3'
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPendentes($parametros, $hierarchyParam, $filtroTipo)
    {
        $sql = "SELECT 
                    FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE, FCM_MODELOAUDIT.NMMODELOAUDIT,
                    FCM_AUDITORIA.IDSITUACAOAUDIT, FCM_AUDITORIA.DTHORAAUDIT, FCM_AUDITORIA.PERCENTUALCONCLUSAO, FCM_AUDITORIA.NOTATOTAL, OPERADOR.NMOPERADOR
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN
                    OPERADOR ON FCM_AUDITORIA.CDOPERADOR =  OPERADOR.CDOPERADOR
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE 
                    (FCM_AUDITORIA.CDOPERADOR IN ({$hierarchyParam}) OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR)
                AND
                    FCM_AUDITORIA.IDSITUACAOAUDIT = '1'
                AND 
                    FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE
                    $filtroTipo";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPorLocal($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '3' THEN 1 END) AS total_concluidas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE THEN 1 END) AS total_pendentes
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                WHERE (
                    (FCM_AUDITORIA.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR AND FCM_AUDITORIA.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPorCliente($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE, 
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '3' THEN 1 END) AS total_concluidas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE THEN 1 END) AS total_pendentes
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_LOCALCLI ON FCM_AUDITORIA.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                WHERE (
                    (FCM_AUDITORIA.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR AND FCM_AUDITORIA.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                CLIENTE.NMFANTCLIE, CLIENTE.CDCLIENTE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAuditoriasPorModelo($parametros, $hierarchyParam)
    {
        $sql = "SELECT 
                    FCM_MODELOAUDIT.NMMODELOAUDIT, FCM_MODELOAUDIT.NRMODELOAUDIT,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '2' THEN 1 END) AS total_andamento,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT <= SYSDATE THEN 1 END) AS total_atrasadas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '3' THEN 1 END) AS total_concluidas,
                    COUNT(CASE WHEN FCM_AUDITORIA.IDSITUACAOAUDIT = '1' AND FCM_AUDITORIA.DTHORAAUDIT >= SYSDATE THEN 1 END) AS total_pendentes
                FROM 
                    FCM_AUDITORIA
                JOIN 
                    FCM_MODELOAUDIT ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_MODELOAUDIT.NRMODELOAUDIT
                WHERE (
                    (FCM_AUDITORIA.CDOPERADOR IN ($hierarchyParam)) 
                OR (FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR AND FCM_AUDITORIA.IDATIVO = 'S')
                    )";

                // Se houver filtro por data
                if (isset($parametros['DTRANGEINICIO']) && isset($parametros['DTRANGEFINAL'])) {
                    $sql .= " AND FCM_AUDITORIA.DTHORAAUDIT BETWEEN TO_DATE(:DTRANGEINICIO, 'DD/MM/YY') AND TO_DATE(:DTRANGEFINAL, 'DD/MM/YY')";
                }
                
                $sql .= " GROUP BY 
                FCM_MODELOAUDIT.NMMODELOAUDIT, FCM_MODELOAUDIT.NRMODELOAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
}