<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class EPIsFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getEPIs()
    {
        $sql = "SELECT NREPI, NMEPI, IDATIVO, NRORG FROM FCM_EPI";
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirEPIs(array $params)
    {
        $sql = "INSERT INTO FCM_EPI (
           NREPI, NMEPI, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO
        ) VALUES (
            :NREPI, :NMEPI, :IDATIVO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO
        )";

        $this->connection->executeStatement($sql, $params);
    }



    public function getTarefasCheck($params)
    {
        $sql = "SELECT c.NMCHECKLISTAGEND, c.NRCHECKLISTAGEND, c.DTHORAAGEND, c.IDSITUACAOAGEND, l.NMLOCALCLIENTE, cl.NMFANTCLIE
            FROM FCM_RELCHKAGENDEPI r
            JOIN FCM_CHECKLISTAGEND c ON r.NRCHECKLISTAGEND = c.NRCHECKLISTAGEND
            JOIN FCM_LOCALCLI l ON c.NRLOCALCLIENTE = l.NRLOCALCLIENTE
            JOIN CLIENTE cl ON cl.CDCLIENTE = l.CDCLIENTE
            WHERE r.NREPI = :NREPI
            AND c.IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
        return $retorno;
    }

    public function atualizarEPI(array $params)
    {
        $sql = "UPDATE FCM_EPI
            SET NMEPI            = :NMEPI,
                CDOPERULTATU    = :CDOPERULTATU,
                IDATIVO = :IDATIVO,
                NRORGULTATU    = :NRORGULTATU,
                DTULTATU         = SYSDATE
            WHERE NREPI = :NREPI";

        $this->connection->executeStatement($sql, $params);
    }

    public function excluirEPI(array $parametros)
    {
        $listaNREPI = implode(',', array_map(function ($n) {
            return $this->connection->quote($n);
        }, $parametros));

        $sql = "DELETE FROM FCM_EPI WHERE NREPI IN ($listaNREPI)";

        $this->connection->executeStatement($sql);
    }
}
