<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Fm00000000HabilidadesFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    // QUERY FRAME PRINCIPAL HABILIDADES
    public function getHabilidades()
    {
        $sql = "SELECT NRHABILIDADE, NMHABILIDADE, IDATIVO FROM FCM_HABILIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirHabilidade($params)
    {
        $sql = "INSERT INTO FCM_HABILIDADE (
            NRHABILIDADE, NMHABILIDADE, IDATIVO, NRORG, CDOPERINCLUSAO,NRORGINCLUSAO, DTINCLUSAO
        ) VALUES (
            :NRHABILIDADE, :NMHABILIDADE, :IDATIVO, :NRORG, :CDOPERINCLUSAO, :NRORGINCLUSAO, SYSDATE
        )";

        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarHabilidade($params)
    {
        $sql = "UPDATE FCM_HABILIDADE 
                SET NMHABILIDADE    = :NMHABILIDADE,
                    CDOPERULTATU    = :CDOPERULTATU,
                    IDATIVO = :IDATIVO,
                     NRORGULTATU    = :NRORGULTATU,
                        DTULTATU    = SYSDATE
              WHERE NRHABILIDADE    = :NRHABILIDADE";

        $this->connection->executeStatement($sql, $params);
    }

    public function excluirHabilidades($parametro)
    {
        $sql = "DELETE FROM FCM_HABILIDADE 
                 WHERE NRHABILIDADE = :NRHABILIDADE";

        $this->connection->executeStatement($sql, ['NRHABILIDADE' => $parametro]);
    }

    public function tarefasHabilidade($params) {
    
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, FCM_CHECKLISTAGEND.DTHORAAGEND,
                    FCM_CHECKLISTAGEND.IDSITUACAOAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE
                FROM
                    FCM_HABILIDADE
                JOIN 
                    FCM_CHECKLIST ON FCM_HABILIDADE.NRHABILIDADE = FCM_CHECKLIST.NRHABILIDADE
                JOIN 
                    FCM_CHECKLISTAGEND ON FCM_CHECKLIST.NRCHECKLIST = FCM_CHECKLISTAGEND.NRCHECKLIST
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                WHERE
                    FCM_HABILIDADE.NRHABILIDADE = :NRHABILIDADE";
    
    return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }  
    
    public function verificarHabilidadesEmUso($params) {
        $nrHabilidade = isset($params['NRHABILIDADE']) ? $params['NRHABILIDADE'] : null;
    
        $sql = "SELECT c.NMTAREFA, c.NRTAREFA, c.DTINCLUSAO
                FROM FCM_TAREFA c
                JOIN FCM_RELACHABATIV r ON r.NRTAREFA = c.NRTAREFA
                WHERE r.NRHABILIDADE = :NRHABILIDADE
                AND c.IDATIVO = 'S'";
    
        return $this->connection->executeQuery($sql, ['NRHABILIDADE' => $nrHabilidade])->fetchAllAssociative();
    }    

    // QUERY TAB OEPRADORES
    public function getOperadorHabilidade($params)
    {
        $sql = "SELECT NRRELACHABOPER, NRHABILIDADE, CDOPERADOR 
                   FROM FCM_RELACHABOPER
                  WHERE NRHABILIDADE = :NRHABILIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function relacionarOperadorHabilidade($params)
    {
        $sql = "INSERT INTO FCM_RELACHABOPER (
           NRRELACHABOPER, NRHABILIDADE, CDOPERADOR, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, IDATIVO, DTINCLUSAO
        ) VALUES (
            :NRRELACHABOPER, :NRHABILIDADE, :CDOPERADOR, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO, :IDATIVO, SYSDATE
        )";

        $this->connection->executeStatement($sql, $params);
    }

    public function excluirOperadorHabilidades($filtro)
    {
        $sql = "DELETE FROM FCM_RELACHABOPER
                $filtro";

        $this->connection->executeStatement($sql);
    }

    // QUERY TAB ATIVIDADES
    public function getAtividadeHabilidade($params)
    {
        $sql = "SELECT FCM_TAREFA.NMTAREFA, FCM_RELACHABATIV .NRRELACHABATIV, FCM_TAREFA.NRTAREFA 
                FROM FCM_RELACHABATIV 
                 LEFT JOIN  FCM_TAREFA
                    ON FCM_RELACHABATIV.NRTAREFA = FCM_TAREFA.NRTAREFA
                 WHERE FCM_RELACHABATIV.NRHABILIDADE IN (" . $params . ")";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function relacionarAtividadeHabilidade($params)
    {
        $sql = "INSERT INTO FCM_RELACHABATIV (
           NRRELACHABATIV, NRTAREFA, NRHABILIDADE, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, IDATIVO, DTINCLUSAO
        ) VALUES (
            :NRRELACHABATIV, :NRTAREFA, :NRHABILIDADE, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO, :IDATIVO, SYSDATE
        )";

        $this->connection->executeStatement($sql, $params);
    }

    public function excluirAtividadeHabilidades($filtro)
    {
        $sql = "DELETE FROM FCM_RELACHABATIV
                $filtro";

        $this->connection->executeStatement($sql);
    }

    public function getAtividadesEmUso($parametros)
{
    $sql = "SELECT COUNT(DISTINCT FCM_TAREFAAGEND.NRTAREFAAGEND) AS ATIVIDADES
            FROM FCM_TAREFAAGEND
            JOIN FCM_TAREFACHK ON FCM_TAREFAAGEND.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
            JOIN FCM_TAREFA ON FCM_TAREFA.NRTAREFA = FCM_TAREFACHK.NRTAREFA
            JOIN FCM_RELACHABATIV ON FCM_TAREFA.NRTAREFA = FCM_RELACHABATIV.NRTAREFA
            WHERE FCM_RELACHABATIV.NRRELACHABATIV = :NRRELACHABATIV";

    // Executa a consulta com o parâmetro
    $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();

    // Retorna o número de atividades associadas ou 0 se não houver
    return $retorno[0]['ATIVIDADES'] ?? 0;
}

}
