<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Fm00003000ChecklistFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    // FUNÇÕES DA TAB DE CHECKLIST

    public function getAtividadesChecklist($parametros)
    {

        $sql = " SELECT FCM_ITEMCHECKLIST.NRTAREFA, FCM_TAREFA.NMTAREFA 
                    FROM FCM_ITEMCHECKLIST
                    JOIN FCM_TAREFA
                    ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
                    WHERE FCM_ITEMCHECKLIST.NRATIVIDADE IN ({$parametros})
                    AND FCM_ITEMCHECKLIST.IDATIVO = 'S'
                    ORDER BY FCM_TAREFA.NMTAREFA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosChecklist($parametros)
    {
        $sql = "SELECT DISTINCT TO_CHAR(FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA) AS CODIGO, 
                                FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA AS CODIGOAUX, 
                                FCM_ESCOPOSTAREFA.NMESCOPOSTAREFA AS NOME, 
                                'Escopo' AS TIPO, 
                                NULL AS CODIGOPAI, 
                                NULL AS CODIGOPAIAUX, 
                                NULL AS IDATIVO
                FROM FCM_ITEMCHECKLIST
                JOIN FCM_ITEMCHECKLISTESC 
                  ON FCM_ITEMCHECKLISTESC.NRITEMCHECKLIST = FCM_ITEMCHECKLIST.NRITEMCHECKLIST
                JOIN FCM_ESCOPOSTAREFA
                  ON FCM_ESCOPOSTAREFA.NRESCOPOSTAREFA = FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA
                WHERE FCM_ITEMCHECKLIST.IDATIVO = 'S'
                  AND FCM_ITEMCHECKLIST.NRTAREFA IN ({$parametros})
    
                UNION ALL 
    
                SELECT DISTINCT FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA || '_' || FCM_ITEMCHECKLIST.NRTAREFA AS CODIGO, 
                                FCM_ITEMCHECKLIST.NRTAREFA AS CODIGOAUX,  
                                FCM_TAREFA.NMTAREFA AS NOME, 
                                'Atividade' AS TIPO, 
                                TO_CHAR(FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA) AS CODIGOPAI, 
                                FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA AS CODIGOPAIAUX, 
                                FCM_ITEMCHECKLIST.IDATIVO
                FROM FCM_ITEMCHECKLIST
                JOIN FCM_TAREFA
                  ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
                JOIN FCM_ITEMCHECKLISTESC 
                  ON FCM_ITEMCHECKLISTESC.NRITEMCHECKLIST = FCM_ITEMCHECKLIST.NRITEMCHECKLIST
                WHERE FCM_ITEMCHECKLIST.IDATIVO = 'S'
                  AND FCM_ITEMCHECKLIST.NRTAREFA IN ({$parametros})
    
                UNION ALL
    
                SELECT DISTINCT TO_CHAR(FCM_ITEMCHECKLISTCRIT.NRCRITERIOTAREFA) AS CODIGO, 
                                FCM_ITEMCHECKLISTCRIT.NRCRITERIOTAREFA AS CODIGOAUX, 
                                FCM_CRITERIOTAREFA.DSCRITERIOTAREFA AS NOME, 
                                'Critério' AS TIPO,  
                                FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA || '_' || FCM_ITEMCHECKLIST.NRTAREFA AS CODIGOPAI, 
                                FCM_ITEMCHECKLIST.NRTAREFA AS CODIGOPAIAUX, 
                                FCM_ITEMCHECKLIST.IDATIVO
                FROM FCM_ITEMCHECKLIST
                JOIN FCM_ITEMCHECKLISTCRIT 
                  ON FCM_ITEMCHECKLISTCRIT.NRITEMCHECKLIST = FCM_ITEMCHECKLIST.NRITEMCHECKLIST
                JOIN FCM_ITEMCHECKLISTESC 
                  ON FCM_ITEMCHECKLISTESC.NRITEMCHECKLIST = FCM_ITEMCHECKLIST.NRITEMCHECKLIST
                JOIN FCM_CRITERIOTAREFA 
                  ON FCM_CRITERIOTAREFA.NRCRITERIOTAREFA = FCM_ITEMCHECKLISTCRIT.NRCRITERIOTAREFA
                WHERE FCM_ITEMCHECKLIST.IDATIVO = 'S'
                  AND FCM_ITEMCHECKLIST.NRTAREFA IN ({$parametros})";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function salvarChecklist($parametros)
    {

        $sql = "INSERT INTO FCM_CHECKLIST (NRCHECKLIST, NMCHECKLIST, NRTIPOLOCALCLI, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRCHECKLIST, :NMCHECKLIST, :NRTIPOLOCALCLI, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function salvarItens($parametros)
    {

        $sql = "INSERT INTO FCM_ATIVIDADECHK (NRATIVIDADECHK, NRATIVIDADE, NRCHECKLIST, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRATIVIDADECHK, :NRATIVIDADE, :NRCHECKLIST, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function verificaExistenciaAtividade($parametros)
    {

        $sql = "SELECT COUNT(*) CONT
                FROM FCM_ITEMCHECKLIST
                WHERE NRATIVIDADE = :NRATIVIDADE
                AND NRTAREFA = :NRTAREFA";


        $st = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $st[0]['CONT'];
    }

    public function salvarAtividade($parametros)
    {
        $sql = "INSERT INTO FCM_TAREFACHK (NRTAREFACHK, NRTAREFA, NRCHECKLIST, NRATIVIDADECHK, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRTAREFACHK, :NRTAREFA, :NRCHECKLIST, :NRATIVIDADECHK, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }


    public function salvarCriterio($parametros)
    {
        $sql = "INSERT INTO FCM_CRITERIOTAREFACHK (NRCRITERIOTAREFACHK, NRCRITERIOTAREFA, NRTAREFACHK, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRCRITERIOTAREFACHK, :NRCRITERIOTAREFA, :NRTAREFACHK, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function salvarEscopo($parametros)
    {
        $sql = "INSERT INTO FCM_ESCOPOSTAREFACHK (NRESCOPOSTAREFACHK, NRCHECKLIST, NRTAREFACHK, NRESCOPOSTAREFA, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRESCOPOSTAREFACHK, :NRCHECKLIST, :NRTAREFACHK, :NRESCOPOSTAREFA, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getChecklist()
    {
        $sql = "SELECT FCM_CHECKLIST.NRCHECKLIST, 
                       FCM_CHECKLIST.NMCHECKLIST, 
                       FCM_TIPOLOCALCLI.NMTIPOLOCALCLI, 
                       CASE FCM_CHECKLIST.IDATIVO 
                           WHEN 'S' THEN 'Sim' 
                           WHEN 'N' THEN 'Não' 
                           ELSE FCM_CHECKLIST.IDATIVO 
                       END AS IDATIVO 
                FROM FCM_CHECKLIST
                JOIN FCM_TIPOLOCALCLI
                  ON FCM_TIPOLOCALCLI.NRTIPOLOCALCLI = FCM_CHECKLIST.NRTIPOLOCALCLI";  

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    // FUNÇÕES DA TAB DE ITENS E CRITÉRIOS

    public function getItensChecklist()
    {
        $sql = "SELECT NRATIVIDADE, NMATIVIDADE, DSCORIDATIVIDADE, IDATIVO 
                FROM FCM_ATIVIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirItemChecklist($parametros)
    {

        $sql = "INSERT INTO FCM_ATIVIDADE (NRATIVIDADE, NMATIVIDADE, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DSCORIDATIVIDADE)
        VALUES (:NRATIVIDADE, :NMATIVIDADE, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :DSCORIDATIVIDADE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getChecklistsItem($parametros)
    {
        $sql = "SELECT CHK.NRCHECKLIST, CHK.NMCHECKLIST, LOC.NMTIPOLOCALCLI
                FROM FCM_ATIVIDADECHK ATVCHK
                JOIN FCM_CHECKLIST CHK ON ATVCHK.NRCHECKLIST = CHK.NRCHECKLIST
                JOIN FCM_TIPOLOCALCLI LOC ON LOC.NRTIPOLOCALCLI = CHK.NRTIPOLOCALCLI
                WHERE ATVCHK.NRATIVIDADE = :NRATIVIDADE
                AND CHK.IDATIVO = 'S'";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getNumTarefasEscopo($parametros)
    {
        $sql = "SELECT COUNT(*) AS NUMTAREFAS
            FROM FCM_ESCOPOSTAREFACHK ESCOPOTAREFA
            JOIN FCM_TAREFACHK TAREFA ON ESCOPOTAREFA.NRTAREFACHK = TAREFA.NRTAREFACHK
            WHERE ESCOPOTAREFA.NRESCOPOSTAREFA = :NRESCOPOSTAREFA
            AND TAREFA.IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return (int) $retorno[0]['NUMTAREFAS'];
    }

    public function getNumTarefasAtividade($parametros)
    {
        $sql = "SELECT COUNT(*) AS NUMTAREFAS
            FROM FCM_TAREFACHK TAREFA
            JOIN FCM_ATIVIDADECHK ATIVIDADE ON TAREFA.NRATIVIDADECHK = ATIVIDADE.NRATIVIDADECHK
            WHERE ATIVIDADE.NRATIVIDADE = :NRATIVIDADE
            AND TAREFA.IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();

        return (int) $retorno[0]['NUMTAREFAS'];
    }

    public function getNumOutrosRegistrosFilhos($parametros)
    {
        $sql = "SELECT COUNT(*) AS NUMTAREFAS
            FROM FCM_TAREFACHK TAREFA
            JOIN FCM_ATIVIDADECHK ATIVIDADE ON TAREFA.NRATIVIDADECHK = ATIVIDADE.NRATIVIDADECHK
            WHERE ATIVIDADE.NRATIVIDADE = :NRATIVIDADE
            AND TAREFA.IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();

        return !empty($retorno) ? (int) $retorno[0]['NUMTAREFAS'] : 0;
    }


    public function getAtividadesItem($parametros)
    {
        $sql = "SELECT NRITEMCHECKLIST
                FROM FCM_ITEMCHECKLIST
                WHERE NRATIVIDADE = :NRATIVIDADE
                AND IDATIVO = 'S'";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getOutrasDependencias($parametros)
    {
        $sql = "SELECT COLUNA
            FROM OUTRA_TABELA
            WHERE NRATIVIDADE = :NRATIVIDADE
            AND IDATIVO = 'S'";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }


    public function inativarItem($parametros)
    {
        $sql = "UPDATE FCM_ATIVIDADE
                SET IDATIVO      = 'N',
                    DTULTATU     = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU  = :NRORGULTATU
                WHERE NRATIVIDADE = :NRATIVIDADE";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function ativarItem($parametros)
    {
        $sql = "UPDATE FCM_ATIVIDADE
                SET IDATIVO      = 'S',
                    DTULTATU     = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU  = :NRORGULTATU
                WHERE NRATIVIDADE = :NRATIVIDADE";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getNumRegistrosFilhos($parametros)
    {
        $sql = "SELECT COUNT(*) AS total FROM FCM_ATIVIDADE WHERE NRATIVIDADE = :NRATIVIDADE";
        $resultado = $this->connection->executeQuery($sql, $parametros)->fetchOne();
        return $resultado;
    }

    public function excluirItem($params)
    {
        $sql = "DELETE FROM FCM_ATIVIDADE
        WHERE NRATIVIDADE = :NRATIVIDADE";
        
        $this->connection->executeStatement($sql, $params);
    }


    public function getCriteriosChecklist($filtro)
    {

        $sql = "SELECT * FROM FCM_CRITERIOTAREFA
                $filtro";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTarefasChecklist($parametros)
    {

        $sql = "SELECT 
                FCM_ITEMCHECKLIST.NRITEMCHECKLIST, FCM_ITEMCHECKLIST.NRTAREFA, FCM_TAREFA.NMTAREFA, FCM_ITEMCHECKLIST.IDATIVO,
                (SELECT RTRIM(XMLAGG(XMLELEMENT(E, CT.NRCRITERIOTAREFA, ',')
                                    .EXTRACT('//text()')
                                    ORDER BY CT.NRCRITERIOTAREFA
                                ).GETCLOBVAL(), ','
                            ) AS NMCRITERIO
                    FROM FCM_ITEMCHECKLISTCRIT CT
                    WHERE CT.NRITEMCHECKLIST = FCM_ITEMCHECKLIST.NRITEMCHECKLIST) AS NRCRITERIOTAREFA, 
                    
                    (SELECT RTRIM(XMLAGG(XMLELEMENT(E, EI.NRESCOPOSTAREFA, ',')
                                    .EXTRACT('//text()')
                                    ORDER BY EI.NRESCOPOSTAREFA
                                ).GETCLOBVAL(), ','
                            ) AS NMESCOPO
                    FROM FCM_ITEMCHECKLISTESC EI
                    WHERE EI.NRITEMCHECKLIST = FCM_ITEMCHECKLIST.NRITEMCHECKLIST) AS NRESCOPOSTAREFA
                                            FROM FCM_ITEMCHECKLIST
                                    LEFT JOIN FCM_TAREFA
                                            ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
                                    LEFT JOIN FCM_CRITERIOTAREFA
                                            ON FCM_CRITERIOTAREFA.NRCRITERIOTAREFA = FCM_ITEMCHECKLIST.NRCRITERIOTAREFA
                                    LEFT JOIN FCM_ESCOPOSTAREFA
                                            ON FCM_ESCOPOSTAREFA.NRESCOPOSTAREFA = FCM_ITEMCHECKLIST.NRESCOPOSTAREFA
                                        WHERE FCM_ITEMCHECKLIST.NRATIVIDADE = :NRATIVIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros, [], null, null, null, array('NRESCOPOSTAREFA', 'NRCRITERIOTAREFA'));
    }

    public function inserirTarefaChecklist($parametros)
    {
        $sql = "INSERT INTO FCM_ITEMCHECKLIST (NRITEMCHECKLIST, NRATIVIDADE, NRTAREFA, NRCRITERIOTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, NRESCOPOSTAREFA)
        VALUES (:NRITEMCHECKLIST, :NRATIVIDADE, :NRTAREFA, :NRCRITERIOTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :NRESCOPOSTAREFA)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateTarefaChecklist($parametros)
    {
        $sql = "UPDATE FCM_ITEMCHECKLIST
        SET NRATIVIDADE     = :NRATIVIDADE,
            NRTAREFA        = :NRTAREFA,
            NRCRITERIOTAREFA= :NRCRITERIOTAREFA,
            IDATIVO         = :IDATIVO,
            NRESCOPOSTAREFA = :NRESCOPOSTAREFA,
            DTULTATU        =  SYSDATE,
            CDOPERULTATU    = :CDOPERULTATU,
            NRORGULTATU     = :NRORGULTATU
        WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirCriterioItemChecklist($parametros)
    {
        $sql = " INSERT INTO FCM_ITEMCHECKLISTCRIT(NRITEMCHECKLISTCRIT, NRITEMCHECKLIST, NRCRITERIOTAREFA, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRITEMCHECKLISTCRIT, :NRITEMCHECKLIST, :NRCRITERIOTAREFA, :NRORG, 'S', SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirEscopoItemChecklist($parametros)
    {
        $sql = "INSERT INTO FCM_ITEMCHECKLISTESC(NRITEMCHECKLISTESC, NRITEMCHECKLIST, NRESCOPOSTAREFA, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRITEMCHECKLISTESC, :NRITEMCHECKLIST, :NRESCOPOSTAREFA, :NRORG, 'S', SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getNumChecklistsTarefa($parametros)
    {
        $sql = "SELECT COUNT(*) TOTAL
                FROM FCM_TAREFACHK TRFCHK
                JOIN FCM_ATIVIDADECHK ATVCHK ON TRFCHK.NRATIVIDADECHK = ATVCHK.NRATIVIDADECHK
                JOIN FCM_ITEMCHECKLIST ITEMCHK ON TRFCHK.NRTAREFA = ITEMCHK.NRTAREFA AND ATVCHK.NRATIVIDADE = ITEMCHK.NRATIVIDADE
                JOIN FCM_CHECKLIST CHK ON TRFCHK.NRCHECKLIST = CHK.NRCHECKLIST
                WHERE ITEMCHK.NRITEMCHECKLIST = :NRITEMCHECKLIST
                AND CHK.IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0]['TOTAL'];
    }

    public function inativarTarefasChecklist($parametros)
    {

        $sql = "UPDATE FCM_ITEMCHECKLIST 
                SET IDATIVO = 'N',
                    NRORGULTATU = :NRORGULTATU,
                    DTULTATU = SYSDATE, 
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function excluirTarefasChecklist($parametros)
    {

        $sql = "DELETE FROM FCM_ITEMCHECKLIST 
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";



        $this->connection->executeStatement($sql, $parametros);
    }


    public function inativarEscoposTarefasChecklist($parametros)
    {

        $sql = "UPDATE FCM_ITEMCHECKLISTESC 
                SET IDATIVO = 'N',
                    NRORGULTATU = :NRORGULTATU,
                    DTULTATU = SYSDATE, 
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function excluirEscoposTarefasChecklist($parametros)
    {

        $sql = "DELETE FROM FCM_ITEMCHECKLISTESC 
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inativarCriteriosTarefasChecklist($parametros)
    {

        $sql = "UPDATE FCM_ITEMCHECKLISTCRIT 
                SET IDATIVO = 'N',
                    NRORGULTATU = :NRORGULTATU,
                    DTULTATU = SYSDATE, 
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function excluirCriteriosTarefasChecklist($parametros)
    {

        $sql = "DELETE FROM FCM_ITEMCHECKLISTCRIT 
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";

        $this->connection->executeStatement($sql, $parametros);
    }


    public function updateCorItemChecklist($parametros)
    {

        $sql = "UPDATE FCM_ATIVIDADE
                    SET DSCORIDATIVIDADE = :DSCORIDATIVIDADE,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU, 
                        NRORGULTATU = :NRORGULTATU
                WHERE NRATIVIDADE = :NRATIVIDADE";

        $this->connection->executeStatement($sql, $parametros);
    }

    // FUNÇÕES DA TAB ATIVIDADE E CRITÉRIOS

    public function getAtividadesCriterios()
    {
        $sql = "SELECT 'T'||TAR.NRTAREFA AS CODIGO, 
                       TAR.NMTAREFA AS NOME, 
                       NULL AS CODIGOPAI, 
                       'Atividade' AS TIPO, 
                       TAR.NRTAREFA AS NR, 
                       NRPRIORIDADETAREFA AS PRIORIDADE,
                       TAR.IDATIVO AS IDATIVO,
                       (SELECT RTRIM(XMLAGG(XMLELEMENT(E, ATIV.NRHABILIDADE, ',')
                               .EXTRACT('//text()')
                               ORDER BY ATIV.NRHABILIDADE
                           ).GETCLOBVAL(), ',') AS HABILIDADE
                        FROM FCM_RELACHABATIV ATIV
                        WHERE TAR.NRTAREFA = ATIV.NRTAREFA
                       ) AS HABILIDADE
                FROM FCM_TAREFA TAR 
    
                UNION ALL
    
                SELECT 'C'||CRI.NRCRITERIOTAREFA AS CODIGO, 
                       CRI.DSCRITERIOTAREFA AS NOME, 
                       'T'||CRI.NRTAREFA AS CODIGOPAI, 
                       'Critério' AS TIPO, 
                       CRI.NRCRITERIOTAREFA AS NR, 
                       NULL AS PRIORIDADE, 
                       CRI.IDATIVO AS IDATIVO,
                       (SELECT RTRIM(XMLAGG(XMLELEMENT(E, ATIV.NRHABILIDADE, ',')
                               .EXTRACT('//text()')
                               ORDER BY ATIV.NRHABILIDADE
                           ).GETCLOBVAL(), ',') AS HABILIDADE
                        FROM FCM_RELACHABATIV ATIV
                        WHERE CRI.NRTAREFA = ATIV.NRTAREFA
                       ) AS HABILIDADE
                FROM FCM_CRITERIOTAREFA CRI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function inserirCriterio($parametros)
    {

        $sql = "INSERT INTO FCM_CRITERIOTAREFA (NRCRITERIOTAREFA, DSCRITERIOTAREFA, NRTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRCRITERIOTAREFA, :DSCRITERIOTAREFA, :NRTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getNumItensCriterio($parametros)
    {
        $sql = "SELECT COUNT(*) ITENS
                FROM FCM_ITEMCHECKLISTCRIT
                WHERE 'C'||NRCRITERIOTAREFA = :NRCRITERIOTAREFA 
                AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0]['ITENS'];
    }

    public function inativarCriterio($parametros)
    {
        $sql = "UPDATE FCM_CRITERIOTAREFA 
        SET     IDATIVO = 'N'
        WHERE 'C'||NRCRITERIOTAREFA = :CODIGO";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateCriterio($parametros)
    {

        $sql = "UPDATE FCM_CRITERIOTAREFA
                        SET DSCRITERIOTAREFA = :DSCRITERIOTAREFA,
                            DTULTATU = SYSDATE,
                            CDOPERULTATU = :CDOPERULTATU, 
                            NRORGULTATU = :NRORGULTATU,
                            IDATIVO = :IDATIVO
                    WHERE 'C'||NRCRITERIOTAREFA = :NRCRITERIOTAREFA";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirAtividade($parametros)
    {

        $sql = "INSERT INTO FCM_TAREFA(NRTAREFA, NMTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, NRPRIORIDADETAREFA)
        VALUES (:NRTAREFA, :NMTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :NRPRIORIDADETAREFA)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getNumItensAtividade($parametros)
    {
        $sql = "SELECT COUNT(*) ITENS
                FROM FCM_ITEMCHECKLIST
                WHERE 'T'||NRTAREFA = :NRTAREFA 
                AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $retorno[0]['ITENS'];
    }


    public function inativarAtividade($parametros)
    {
        $sql = "UPDATE FCM_TAREFA  
        SET     IDATIVO = 'N'
        WHERE 'T'||NRTAREFA = :CODIGO";

        $this->connection->executeStatement($sql, $parametros);
    }


    public function updateAtividade($parametros)
    {

        $sql = "UPDATE FCM_TAREFA
                        SET NMTAREFA = :NMTAREFA,
                            DTULTATU = SYSDATE,
                            CDOPERULTATU = :CDOPERULTATU, 
                            NRORGULTATU = :NRORGULTATU,
                            IDATIVO = :IDATIVO,
                            NRPRIORIDADETAREFA  = :NRPRIORIDADETAREFA
                    WHERE 'T'||NRTAREFA = :NRTAREFA";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function verificarExistenciaFilhos($parametros)
    {

        $sql = "SELECT COUNT(*) FILHOS
                FROM FCM_CRITERIOTAREFA
                WHERE 'T'||NRTAREFA = :CODIGO
                AND IDATIVO = 'S'";

        $st = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $st[0]['FILHOS'];
    }

    // FUNÇÕES DA TAB DE ESCOPO
    public function getEscoposServicos()
    {

        $sql = "SELECT * FROM FCM_ESCOPOSTAREFA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirEscopoServico($parametros)
    {

        $sql = "INSERT INTO FCM_ESCOPOSTAREFA (NRESCOPOSTAREFA, NMESCOPOSTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRESCOPOSTAREFA, :NMESCOPOSTAREFA, :IDATIVO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateEscopoServico($parametros){

        $sql = "UPDATE FCM_ESCOPOSTAREFA
           SET NMESCOPOSTAREFA = :NMESCOPOSTAREFA,
        IDATIVO = :IDATIVO,
        NRORG = :NRORG,
        DTULTATU = SYSDATE,
        CDOPERULTATU = :CDOPERINCLUSAO, 
        NRORGULTATU = :NRORGINCLUSAO
    WHERE NRESCOPOSTAREFA = :NRESCOPOSTAREFA";

    $this->connection->executeStatement($sql, $parametros);}
    
        public function inativarEscopos($parametros)
    {

        $sql = "DELETE FROM FCM_ESCOPOSTAREFA 
                WHERE NRESCOPOSTAREFA IN (:NRESCOPOSTAREFA)";

        $this->connection->executeStatement($sql, $parametros);
    }

    // SELECTS PARA MODAL DE EDICAO

    public function getTipoLocalModalEdicao($params)
    {
        $sql = "SELECT NRTIPOLOCALCLI
            FROM FCM_TIPOLOCALCLI
            WHERE IDTIPOLOCALCLI = 3
            AND NMTIPOLOCALCLI = '$params'
            AND IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function getItensModalEdicao($params)
    {
        $sql = "SELECT AT.NRATIVIDADE
                FROM FCM_CHECKLIST CHK
                JOIN FCM_ATIVIDADECHK ATV
                ON CHK.NRCHECKLIST = ATV.NRCHECKLIST
                JOIN FCM_ATIVIDADE AT
                ON ATV.NRATIVIDADE = AT.NRATIVIDADE
                WHERE CHK.NRCHECKLIST = '$params'
                AND CHK.IDATIVO = 'S'
                AND ATV.IDATIVO = 'S' ";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getAtividadesModalEdicao($params)
    {
        $sql = "SELECT DISTINCT TAR.NRTAREFA
                FROM FCM_CHECKLIST CHK
                JOIN FCM_TAREFACHK TARCHK
                ON TARCHK.NRCHECKLIST = CHK.NRCHECKLIST
                JOIN FCM_TAREFA TAR
                ON TAR.NRTAREFA = TARCHK.NRTAREFA
                WHERE CHK.NRCHECKLIST = $params
                AND CHK.IDATIVO = 'S'  
                AND TARCHK.IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function verificarNomeItens($paramsQuery)
    {
        try {

            $sql = "SELECT NRATIVIDADE, NMATIVIDADE, DSCORIDATIVIDADE
            FROM FCM_ATIVIDADE
            WHERE UPPER(NMATIVIDADE) LIKE UPPER(:NOME)
            AND NRORG = :NRORG";

            return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $paramsQuery);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public function validaUsoChecklist()
    {
        $sql = "SELECT DISTINCT CHK.NRCHECKLIST, GERCHK.NRCHECKLIST AS VALIDADOR 
                            FROM FCM_CHECKLIST CHK
                            LEFT JOIN FCM_GERENCIAMENTOCHK  GERCHK
                            ON CHK.NRCHECKLIST = GERCHK.NRCHECKLIST
                            
                            UNION ALL 
                            
                            SELECT DISTINCT CHK.NRCHECKLIST, GERCHK.NRCHECKLIST AS VALIDADOR 
                            FROM FCM_CHECKLIST CHK
                            LEFT JOIN FCM_CHECKLISTAGEND  GERCHK
                            ON CHK.NRCHECKLIST = GERCHK.NRCHECKLIST";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
    // EDIÇÔES ===============================
    public function editarChecklist($params)
    {

        $sql = "UPDATE FCM_CHECKLIST
                   SET NMCHECKLIST    = :NMCHECKLIST,
                       NRTIPOLOCALCLI = :NRTIPOLOCALCLI,
                       NRORGULTATU    = :NRORGULTATU,
                       IDATIVO        = :IDATIVO,
                       CDOPERULTATU   = :CDOPERULTATU,
                       DTULTATU       = SYSDATE
                 WHERE NRCHECKLIST    = :NRCHECKLIST";

        $this->connection->executeStatement($sql, $params);
    }

    public function editarItens($params)
    {

        $sql = "UPDATE FCM_ATIVIDADECHK
                   SET NRATIVIDADE    = :NRATIVIDADE,
                       NRCHECKLIST    = :NRCHECKLIST,
                       NRORGULTATU    = :NRORGULTATU,
                       CDOPERULTATU   = :CDOPERULTATU,
                       DTULTATU       = SYSDATE
                 WHERE NRATIVIDADECHK = :NRATIVIDADECHK";
        $this->connection->executeStatement($sql, $params);
    }

    public function editarAtividades($params)
    {
        $sql = "UPDATE FCM_TAREFACHK
                   SET NRTAREFA    = :NRTAREFA,
                       NRCHECKLIST    = :NRCHECKLIST,
                       NRATIVIDADECHK    = :NRATIVIDADECHK,
                       CDOPERULTATU   = :CDOPERULTATU,
                       DTULTATU       = SYSDATE,
                       NRORGULTATU = :NRORGULTATU
                 WHERE NRTAREFACHK = :NRTAREFACHK";
        $this->connection->executeStatement($sql, $params);
    }

    public function getNratividadechk($nrcheck, $nrativ)
    {

        $sql = "SELECT NRATIVIDADECHK 
        FROM FCM_ATIVIDADECHK
        WHERE NRCHECKLIST = $nrcheck
        AND NRATIVIDADE = $nrativ";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getNrtarefachk($nrcheck, $nrtarefa)
    {

        $sql = "SELECT NRTAREFACHK
        FROM FCM_TAREFACHK
        WHERE NRCHECKLIST = $nrcheck
        AND NRTAREFA = $nrtarefa";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function verificaExistencia($querry)
    {

        $sql = "SELECT COUNT (*) CONTADOR " . $querry;

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function deleteChecklist($nrchecklist, $nrtarefachk)
    {

        $sql = "DELETE FCM_ESCOPOSTAREFACHK 
                WHERE NRCHECKLIST = $nrchecklist";

        $this->connection->executeStatement($sql);

        $sql1 = "DELETE FCM_CRITERIOTAREFACHK 
                WHERE NRTAREFACHK IN  ($nrtarefachk)";

        $this->connection->executeStatement($sql1);

        $sql2 = "DELETE FCM_TAREFACHK
                WHERE NRCHECKLIST = $nrchecklist";

        $this->connection->executeStatement($sql2);

        $sql3 = "DELETE FCM_ATIVIDADECHK
                WHERE NRCHECKLIST = $nrchecklist";

        $this->connection->executeStatement($sql3);

        $sql4 = "DELETE FCM_CHECKLIST 
                WHERE NRCHECKLIST = $nrchecklist";
        $this->connection->executeStatement($sql4);
    }

    public function getNumTarefasChecklist($params)
    {
        $sql = "SELECT COUNT(*) NUMTAREFAS
                FROM FCM_CHECKLISTAGEND
                WHERE NRCHECKLIST = :NRCHECKLIST
                AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
        return $retorno[0]['NUMTAREFAS'];
    }

    public function getTarefasAssociadasChecklist($params)
    {
        $sql = "SELECT NRTAREFA, DSTAREFA
            FROM FCM_CHECKLISTAGEND
            WHERE NRCHECKLIST = :NRCHECKLIST
            AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $params)->fetchAllAssociative();

        $tarefas = array_map(function ($row) {
            return $row['NRTAREFA'] . ' - ' . $row['DSTAREFA'];
        }, $retorno);

        return $tarefas;
    }

    public function getTarefasCheck($params)
    {
        $sql = "SELECT NMCHECKLISTAGEND
            FROM FCM_CHECKLISTAGEND
            WHERE NRCHECKLIST = :NRCHECKLIST
            AND IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
        return $retorno;
    }


    public function inativarCheckt($nrchecklist)
    {
        $sql4 = "DELETE FROM FCM_CHECKLIST 
                WHERE NRCHECKLIST = $nrchecklist";

        $this->connection->executeStatement($sql4);
    }

    public function verificarExistenciaFilhosChech($parametros)
    {
        $sql = "SELECT COUNT(*) AS FILHOS
                FROM FCM_CHECKLIST
                WHERE NRCHECKLIST = :CODIGO
                AND IDATIVO = 'S'";

        $st = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $st[0]['FILHOS'];
    }
    public function inativarChecklist($params)
    {
        $sql = "UPDATE  FCM_CHECKLIST
                SET     IDATIVO = 'N'
                WHERE   NRCHECKLIST = :NRCHECKLIST";

        $this->connection->executeStatement($sql, $params);
    }

    public function getEscoposUtil($params)
    {
        $sql = "SELECT FCM_TAREFA.NMTAREFA, FCM_TAREFA.NRTAREFA
                FROM FCM_TAREFA 
                JOIN FCM_ITEMCHECKLIST ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
                JOIN FCM_ITEMCHECKLISTESC ON FCM_ITEMCHECKLIST.NRITEMCHECKLIST = FCM_ITEMCHECKLISTESC.NRITEMCHECKLIST
                WHERE FCM_ITEMCHECKLISTESC.NRESCOPOSTAREFA = :NRESCOPOSTAREFA
                AND FCM_TAREFA.IDATIVO = 'S'";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function getCriteriosUtil($params)
    {
        $sql = "SELECT FCM_ATIVIDADE.NMATIVIDADE, FCM_ATIVIDADE.NRATIVIDADE
                FROM FCM_ATIVIDADE 
                JOIN FCM_ITEMCHECKLIST ON FCM_ATIVIDADE.NRATIVIDADE = FCM_ITEMCHECKLIST.NRATIVIDADE
                JOIN FCM_ITEMCHECKLISTCRIT ON FCM_ITEMCHECKLIST.NRITEMCHECKLIST = FCM_ITEMCHECKLISTCRIT.NRITEMCHECKLIST
                WHERE 'C'||FCM_ITEMCHECKLISTCRIT.NRCRITERIOTAREFA = :NRCRITERIOTAREFA
                AND FCM_ATIVIDADE.IDATIVO = 'S'";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function getAtividadesUtil($params)
    {
        $sql = "SELECT FCM_ATIVIDADE.NMATIVIDADE, FCM_ATIVIDADE.NRATIVIDADE
                FROM FCM_ATIVIDADE 
                JOIN FCM_ITEMCHECKLIST ON FCM_ATIVIDADE.NRATIVIDADE = FCM_ITEMCHECKLIST.NRATIVIDADE
                WHERE 'T'||FCM_ITEMCHECKLIST.NRITEMCHECKLIST = :NRITEMCHECKLIST
                AND FCM_ATIVIDADE.IDATIVO = 'S'";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function getChecklistsUtil($params)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, FCM_CHECKLISTAGEND.DTHORAAGEND,
                    FCM_CHECKLISTAGEND.IDSITUACAOAGEND, FCM_LOCALCLI.NMLOCALCLIENTE, CLIENTE.NMFANTCLIE
                FROM 
                    FCM_CHECKLIST
                JOIN 
                    FCM_CHECKLISTAGEND ON FCM_CHECKLIST.NRCHECKLIST = FCM_CHECKLISTAGEND.NRCHECKLIST
                JOIN 
                    FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
                JOIN 
                    CLIENTE ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                WHERE
                    FCM_CHECKLIST.NRCHECKLIST = :NRCHECKLIST
                AND 
                    FCM_CHECKLISTAGEND.IDSITUACAOAGEND IN ('1', '2', '6', '5')";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }
}
