<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Fm00005000VistoriasFactory
{

    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    // Busca as atividades das vistorias

    public function getAtividades($parametros)
    {

        $sql = "SELECT DISTINCT FCM_ATIVIDADECHK.NRATIVIDADECHK, FCM_ATIVIDADE.NMATIVIDADE
                  FROM FCM_ATIVIDADECHK
                  JOIN FCM_ATIVIDADE
                    ON FCM_ATIVIDADE.NRATIVIDADE = FCM_ATIVIDADECHK.NRATIVIDADE
                  JOIN FCM_TAREFACHK
                    ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
                    AND NRTAREFACHK IN (SELECT NRTAREFACHK
                                          FROM FCM_TAREFAGERCHK
                                         WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getCriterios($parametros)
    {

        $sql = "SELECT FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK, FCM_CRITERIOTARGERCHK.NMCRITERIOTARGERCHK, FCM_CRITERIOTARGERCHK.IDMARCADO
                    FROM FCM_CRITERIOTARGERCHK
                    JOIN FCM_TAREFAGERCHK
                    ON FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK = FCM_TAREFAGERCHK.NRTAREFAGERCHK
                    JOIN FCM_TAREFACHK
                    ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
                    JOIN FCM_TAREFA 
                    ON FCM_TAREFA.NRTAREFA = FCM_TAREFACHK.NRTAREFA
                WHERE FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK
                    AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getVistorias($hierarchyParam, $cdoperador)
    {
        $sql = "SELECT DISTINCT 
    GCHK.NRGERENCIAMENTOCHK, 
    CA.NRCHECKLISTAGEND, 
    GCHK.NMGERENCIAMENTOCHK,
    CLIENTE.CDCLIENTE, 
    CLIENTE.NMRAZSOCCLIE, 
    CA.DSJUSTCANCELAMENTO, 
    TA.DSOBSTAREFAAGEND, 
    GCHK.IDOBRIGAOBSERVACAO, 
    GCHK.NRLOCALCLIENTE, 
    GCHK.CDOPERADOR,
    GCHK.DTHORAAGEND, 
    FUNC.NMOPERADOR, 
    LC.NMLOCALCLIENTE, 
    LC.NRLOCALCLIENTEPAI AS LOCALPAI, 
    LCP.NMLOCALCLIENTE AS NMLOCALPAI, 
    CA.IDSITUACAOAGEND, 
    FCT_CONTRATO.NRCONTRATOFCT,
    DECODE(CA.IDSITUACAOAGEND,
        '1', CASE GCHK.IDSITUACAOGERCHK
            WHEN '4' THEN 'Em andamento (Tarefa em correção)'
            ELSE 'Tarefa Pendente' END,
        '4', CASE GCHK.IDSITUACAOGERCHK 
            WHEN '1' THEN 'Aguardando Vistoria'
            WHEN '2' THEN 'Em andamento'
            WHEN '3' THEN 'Avaliado'
            WHEN '5' THEN 'Cancelado'
            ELSE NULL END,
        NULL, CASE GCHK.IDSITUACAOGERCHK 
            WHEN '1' THEN 'Aguardando Vistoria'
            WHEN '2' THEN 'Em andamento'
            WHEN '3' THEN 'Avaliado'
            ELSE NULL END,
        'Tarefa Pendente'
            ) AS IDSITUACAOGERCHK,
            GCHK.IDATIVO,
            MAX(FCM_TAREFAGERCHK.DSOBSTAREFAGERCHK) AS OBS,
                CASE
                    WHEN SYSDATE < GCHK.DTHORAAGEND THEN ' '  
                    WHEN TRUNC(SYSDATE) - TRUNC(GCHK.DTHORAAGEND) < 1 THEN 
                        GREATEST(FLOOR((SYSDATE - GCHK.DTHORAAGEND) * 24), 0) || ' horas ' || 
                        GREATEST(FLOOR((SYSDATE - GCHK.DTHORAAGEND) * 24 * 60) - (FLOOR((SYSDATE - GCHK.DTHORAAGEND) * 24) * 60), 0) || ' minutos'
                    ELSE 
                        TRUNC(SYSDATE) - TRUNC(GCHK.DTHORAAGEND) || ' dias'
                END AS ATRASO
            FROM FCM_GERENCIAMENTOCHK GCHK
            INNER JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
            INNER JOIN OPERADOR FUNC
                ON FUNC.CDOPERADOR = GCHK.CDOPERADOR
            INNER JOIN FCM_LOCALCLI LCP
                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
            LEFT JOIN FCM_CHECKLISTAGEND CA
                ON CA.NRCHECKLISTAGEND = GCHK.NRCHECKLISTAGEND 
            INNER JOIN CLIENTE
                ON LC.CDCLIENTE = CLIENTE.CDCLIENTE
            LEFT JOIN FCM_TAREFAAGEND TA
                ON CA.NRCHECKLISTAGEND = TA.NRCHECKLISTAGEND
            LEFT JOIN FCT_CONTRATO
                ON FCT_CONTRATO.NRSEQCONTRATO = GCHK.NRSEQCONTRATO
            LEFT JOIN FCM_TAREFAGERCHK
                ON FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = GCHK.NRGERENCIAMENTOCHK
            WHERE (GCHK.CDOPERADOR IN ({$hierarchyParam}) OR GCHK.CDOPERINCLUSAO = {$cdoperador})
            GROUP BY GCHK.NRGERENCIAMENTOCHK, 
                CA.NRCHECKLISTAGEND, 
                GCHK.NMGERENCIAMENTOCHK,
                CLIENTE.CDCLIENTE, 
                CLIENTE.NMRAZSOCCLIE, 
                CA.DSJUSTCANCELAMENTO, 
                TA.DSOBSTAREFAAGEND, 
                GCHK.IDOBRIGAOBSERVACAO, 
                GCHK.NRLOCALCLIENTE, 
                GCHK.CDOPERADOR,
                GCHK.DTHORAAGEND, 
                FUNC.NMOPERADOR, 
                LC.NMLOCALCLIENTE, 
                LC.NRLOCALCLIENTEPAI, 
                LCP.NMLOCALCLIENTE, 
                CA.IDSITUACAOAGEND, 
                FCT_CONTRATO.NRCONTRATOFCT,
                DECODE(CA.IDSITUACAOAGEND,
                    '1', CASE GCHK.IDSITUACAOGERCHK
                        WHEN '4' THEN 'Em andamento (Tarefa em correção)'
                        ELSE 'Tarefa Pendente' END,
                    '4', CASE GCHK.IDSITUACAOGERCHK 
                        WHEN '1' THEN 'Aguardando Vistoria'
                        WHEN '2' THEN 'Em andamento'
                        WHEN '3' THEN 'Avaliado'
                        WHEN '5' THEN 'Cancelado'
                        ELSE NULL END,
                    NULL, CASE GCHK.IDSITUACAOGERCHK 
                        WHEN '1' THEN 'Aguardando Vistoria'
                        WHEN '2' THEN 'Em andamento'
                        WHEN '3' THEN 'Avaliado'
                        ELSE NULL END,
                    'Tarefa Pendente'
                ),
                GCHK.IDATIVO
            ORDER BY GCHK.DTHORAAGEND DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
    
    public function getCriteriosGrid($params)
    {
        $sql = "SELECT DISTINCT 'I'||FCM_ATIVIDADECHK.NRATIVIDADECHK AS CODIGO, NMATIVIDADE AS NOME, 
        NULL AS CODIGOPAI, 'Item' AS TIPO, NULL AS IDMARCADO,
         FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_ATIVIDADECHK.NRATIVIDADECHK AS NR
          FROM FCM_ATIVIDADECHK
          JOIN FCM_TAREFACHK
           ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
          JOIN FCM_ATIVIDADE
            ON FCM_ATIVIDADECHK.NRATIVIDADE = FCM_ATIVIDADE.NRATIVIDADE
          JOIN FCM_TAREFAGERCHK
            ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
          JOIN FCM_GERENCIAMENTOCHK 
            ON FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
           AND FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_TAREFACHK.NRCHECKLIST
          WHERE FCM_TAREFAGERCHK.IDATIVO = 'S'
            AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
        UNION ALL
        SELECT 'A'||FCM_TAREFAGERCHK.NRTAREFAGERCHK AS CODIGO, FCM_TAREFA.NMTAREFA AS NOME, 
              'I'||FCM_ATIVIDADECHK.NRATIVIDADECHK AS CODIGOPAI, 'Atividade' AS TIPO, NULL AS IDMARCADO, 
              FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_TAREFAGERCHK.NRTAREFAGERCHK AS NR
              FROM FCM_TAREFAGERCHK
              JOIN FCM_TAREFACHK
                ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
              JOIN FCM_ATIVIDADECHK
                ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
              JOIN FCM_TAREFA
                ON FCM_TAREFACHK.NRTAREFA = FCM_TAREFA.NRTAREFA
              WHERE FCM_TAREFAGERCHK.IDATIVO = 'S'
                AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
          UNION ALL
              SELECT 'C'||FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK AS CODIGO, FCM_CRITERIOTARGERCHK.NMCRITERIOTARGERCHK AS NOME, 
                     'A'||FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK AS CODIGOPAI, 'Critério' AS TIPO, FCM_CRITERIOTARGERCHK.IDMARCADO, 
                     FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK AS NR
              FROM FCM_CRITERIOTARGERCHK
              JOIN FCM_TAREFAGERCHK
                ON FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK = FCM_TAREFAGERCHK.NRTAREFAGERCHK
              WHERE FCM_CRITERIOTARGERCHK.IDATIVO = 'S'
                AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    // Atualiza o campo IDMARCADO do critério para S ou N
    public function salvarCriterios($param)
    {
        $sql = "UPDATE FCM_CRITERIOTARGERCHK
                   SET IDMARCADO    = '$param[IDMARCADO]',
                       DTULTATU     = SYSDATE,
                       NRORGULTATU  = $param[NRORGULTATU], 
                       CDOPERULTATU = $param[CDOPERULTATU]
                 WHERE NRCRITERIOTARGERCHK = $param[NRCRITERIOTARGERCHK]";

        $this->connection->executeStatement($sql);

        $sql1 = "UPDATE FCM_GERENCIAMENTOCHK
                    SET IDSITUACAOGERCHK = '2',
                        DTULTATU         = SYSDATE,
                        NRORGULTATU      = $param[NRORGULTATU],
                        CDOPERULTATU     = $param[CDOPERULTATU],
                        OBS              = $param[OBS]
                  WHERE NRGERENCIAMENTOCHK = $param[NRGERENCIAMENTOCHK]";

        $this->connection->executeStatement($sql1);
    }

    public function updateCriterios($parametros, $paramQuery)
    {

        $sql = "UPDATE FCM_CRITERIOTARGERCHK
                    SET IDMARCADO = :IDMARCADO,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU
                  WHERE NRCRITERIOTARGERCHK IN (SELECT NRCRITERIOTARGERCHK
                                                    FROM FCM_CRITERIOTARGERCHK
                                                   WHERE NRTAREFAGERCHK IN (SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK
                                                    FROM FCM_TAREFAGERCHK
                                                    JOIN FCM_TAREFACHK
                                                      ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                                                   WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                                                     AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK))" . $paramQuery;

        $this->connection->executeStatement($sql, $parametros);
    }

    public function atualizaHistoricoVistoria($parametros)
    {
        $sql = "INSERT INTO FCM_GERCHKHISTORICO (
                    NRGERCHKHISTORICO,      NRGERENCIAMENTOCHK, NMGERENCIAMENTOCHK, DSGERENCIAMENTOCHK,
                    DTHISTGERENCIAMENTOCHK, IDTIPOHISTORICO,    IDATIVO,            NRORG,
                    DTINCLUSAO,             CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,
                    CDOPERULTATU,           NRORGULTATU
                )
                VALUES (
                    :NRGERCHKHISTORICO,     :NRGERENCIAMENTOCHK, :NMGERENCIAMENTOCHK, :DSGERENCIAMENTOCHK,
                    SYSDATE,                :IDTIPOHISTORICO,    'S',                 :NRORG,
                    SYSDATE,                :CDOPERINCLUSAO,     :NRORGINCLUSAO,      SYSDATE,
                    :CDOPERULTATU,          :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function atualizaHistoricoTarefa($parametros)
    {
        $sql = "INSERT INTO FCM_CHKAGENDHISTORICO (
                    NRCHKAGENDHISTORICO,    NRCHECKLISTAGEND,   NMCHECKLISTAGEND,   DSCHKAGENDHISTORICO,
                    DTCHKAGENDHISTORICO,    IDTIPOHISTORICO,    IDATIVO,            NRORG,
                    DTINCLUSAO,             CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,
                    CDOPERULTATU,           NRORGULTATU
                )
                VALUES (
                    :NRCHKAGENDHISTORICO,   :NRCHECKLISTAGEND,  :NMCHECKLISTAGEND,  :DSCHKAGENDHISTORICO,
                    SYSDATE,                :IDTIPOHISTORICO,   'S',                :NRORG,
                    SYSDATE,                :CDOPERINCLUSAO,    :NRORGINCLUSAO,     SYSDATE,
                    :CDOPERULTATU,          :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateStatusVistoria($parametros)
    {

        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = :IDSITUACAOGERCHK,
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU
                  WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";


        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateObservacao($parametros)
    {

        $sql = "UPDATE FCM_TAREFAGERCHK
                SET DSOBSTAREFAGERCHK = :DSOBSTAREFAGERCHK,
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU
                    WHERE NRTAREFAGERCHK IN (SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK
                                  FROM FCM_TAREFAGERCHK
                                  JOIN FCM_TAREFACHK
                                    ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                                 WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                                   AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK)";

        $this->connection->executeStatement($sql, $parametros);
    }

    // Atualiza o campo de observação da vistoria
    public function salvarObservacao($params)
    {
        $sql = "UPDATE FCM_TAREFAGERCHK
                   SET DSOBSTAREFAGERCHK = :DSOBSTAREFAGERCHK,
                       DTULTATU          = SYSDATE,
                       NRORGULTATU       = :NRORGULTATU, 
                       CDOPERULTATU      = :CDOPERULTATU
                 WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        $this->connection->executeStatement($sql, $params);
    }

    // Atualiza campo IDSITUACAOGERCHK como Avaliado
    public function finalizarVistoria($parametros)
    {
        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = '3',
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU, 
                CDOPERULTATU = :CDOPERULTATU
                  WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertChk($params)
    {
        $sql = "INSERT INTO FCM_GERENCIAMENTOCHK (
                                NRGERENCIAMENTOCHK, NMGERENCIAMENTOCHK, NRLOCALCLIENTE, 
                                NRCHECKLIST, CDOPERADOR, IDSITUACAOGERCHK, 
                                DTHORAAGEND, IDATIVO, DTINCLUSAO, 
                                CDOPERINCLUSAO, NRCHECKLISTAGEND,DTHRFINALAGEND, IDOBRIGAOBSERVACAO, NRSEQCONTRATO)
                VALUES (
                    :NRGERENCIAMENTOCHK, :NMGERENCIAMENTOCHK, :NRLOCALCLIENTE, 
                    :NRCHECKLIST, :CDOPERADOR, '1',  
                    :DTHORAAGEND, 'S', SYSDATE, 
                    :OPERADOR, :NRCHECKLISTAGEND,TO_DATE(:DTHRFINALAGEND), :IDOBRIGAOBSERVACAO, :NRSEQCONTRATO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function insertChkTarefa($parametros)
    {
        $sql = "INSERT INTO FCM_TAREFAGERCHK (
                    NRTAREFAGERCHK, NRGERENCIAMENTOCHK, NRTAREFACHK, 
                    CDOPERADOR,     DTHORATAREFAGERCHK, DSOBSTAREFAGERCHK, 
                    IDATIVO,        DTINCLUSAO,         CDOPERINCLUSAO
                )
                VALUES (
                    :NRTAREFAGERCHK, :NRGERENCIAMENTOCHK, :NRTAREFACHK, 
                    :CDOPERADOR,     :DTHORATAREFAGERCHK, :DSOBSTAREFAGERCHK, 
                    'S',             SYSDATE,             :OPERADOR
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getChkTarefa($parametros) 
    {
        $sql = "SELECT NRTAREFAGERCHK FROM FCM_TAREFAGERCHK WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function deleteCriterios($parametros) {
        $sql = "DELETE FROM FCM_CRITERIOTARGERCHK WHERE NRTAREFAGERCHK = :NRTAREFAGERCHK";

        return $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteChkTarefa($parametros) 
    {
        $sql = "DELETE FROM FCM_TAREFAGERCHK WHERE NRTAREFAGERCHK = :NRTAREFAGERCHK";

        return $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteGerenciamentoChk($parametros) 
    {
        $sql = "DELETE FROM FCM_GERENCIAMENTOCHK WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        return $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteImagensVistoria($param) {
        $sql = "DELETE FROM FCM_FOTOTAREFAGERCHK
                 WHERE NRTAREFAGERCHK = (SELECT MIN(NRTAREFAGERCHK) AS NRTAREFAGERCHK FROM FCM_TAREFAGERCHK WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

        $this->connection->executeStatement($sql, $param);
    }

    public function validaCriterio($parametros)
    {

        $sql = "SELECT COUNT(*) AS VALIDA
                FROM FCM_CRITERIOTAREFACHK 
                WHERE NRTAREFACHK = :NRTAREFACHK
                AND NRCRITERIOTAREFA = :NRCRITERIOTAREFA";

        $resultado = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $resultado[0]['VALIDA'];
    }

    public function insertChkCriterio($parametros)
    {
        $sql = "INSERT INTO FCM_CRITERIOTARGERCHK (
                    NRCRITERIOTARGERCHK,    NMCRITERIOTARGERCHK,    NRTAREFAGERCHK, 
                    NRCRITERIOTAREFACHK,    IDATIVO,                DTINCLUSAO, 
                    CDOPERINCLUSAO
                )
                VALUES (
                    :NRCRITERIOTARGERCHK,   :NMCRITERIOTARGERCHK,   :NRTAREFAGERCHK, 
                    :NRCRITERIOTAREFACHK,   'S',                    SYSDATE, 
                    :OPERADOR
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getResponsaveisVistoria($locais)
    {
        $sql = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR
                            FROM OPERADOR
                            JOIN FCM_FUNCIONARIOSHIERARQ ON FCM_FUNCIONARIOSHIERARQ.CDOPERADOR = OPERADOR.CDOPERADOR
                            JOIN FCM_HIERARQUIAFUNC ON FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC = FCM_FUNCIONARIOSHIERARQ.NRHIERARQUIAFUNC
                            WHERE FCM_FUNCIONARIOSHIERARQ.CDOPERADOR IN (
                                SELECT DISTINCT CDOPERADORPAI
                                FROM FCM_FUNCIONARIOSHIERARQ
                                WHERE CDOPERADORPAI IS NOT NULL
                                CONNECT BY PRIOR CDOPERADORPAI = CDOPERADOR
                            )
                            AND FCM_HIERARQUIAFUNC.NRLOCALCLIENTE IN (
                                SELECT NRLOCALCLIENTE
                                FROM FCM_LOCALCLI
                                START WITH NRLOCALCLIENTE IN ($locais)
                                CONNECT BY PRIOR NRLOCALCLIENTEPAI = NRLOCALCLIENTE
                            )";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function excluirImagensVistoria($param) {
        $sql = "DELETE FROM FCM_FOTOTAREFAGERCHK
                 WHERE NRFOTOTAREFAGERCHK = :NRFOTOTAREFAGERCHK
                   AND NRTAREFAGERCHK = (SELECT MIN(NRTAREFAGERCHK) AS NRTAREFAGERCHK FROM FCM_TAREFAGERCHK WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

        $this->connection->executeStatement($sql, $param);
    }

    public function atualizaStatusTarefa($parametros)
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND
                SET IDSITUACAOAGEND = :IDSITUACAOAGEND,
                    DTHORAAGEND     = SYSDATE,
                    DTULTATU        = SYSDATE,
                    NRORGULTATU     = :NRORGULTATU,
                    CDOPERULTATU    = :CDOPERULTATU,
                    DTHRFINALAGEND  = SYSDATE
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function atualizaStatusVistoria($parametros)
    {
        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = :IDSITUACAOGERCHK,
                    DTULTATU         = SYSDATE,
                    NRORGULTATU      = :NRORGULTATU,
                    CDOPERULTATU     = :CDOPERULTATU
                WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function atualizaStatusTarefaAgend($parametros)
    {
        $sql = "UPDATE FCM_TAREFAAGEND
                SET IDREALIZADO  = 'N',
                    DTULTATU     = SYSDATE,
                    NRORGULTATU  = :NRORGULTATU,
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRTAREFACHK IN (SELECT NRTAREFACHK FROM FCM_TAREFACHK WHERE NRATIVIDADECHK = :NRATIVIDADECHK)
                AND NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getHistoricoVistoria($parametros)
    {
        $sql = "SELECT HIST.NRGERCHKHISTORICO, HIST.NRGERENCIAMENTOCHK, HIST.NMGERENCIAMENTOCHK, HIST.DSGERENCIAMENTOCHK, 
                TO_CHAR(HIST.DTHISTGERENCIAMENTOCHK, 'DD/MM/YYYY HH24:MI') AS DTHISTGERENCIAMENTOCHK, HIST.IDTIPOHISTORICO, HIST.DTINCLUSAO,
                TO_CHAR(HIST.DTULTATU, 'DD/MM/YYYY HH24:MI') AS DTULTATU, HIST.CDOPERULTATU, OPER.NMOPERADOR AS NMOPERULTATU
                FROM FCM_GERCHKHISTORICO HIST
                JOIN OPERADOR OPER ON OPER.CDOPERADOR = HIST.CDOPERULTATU
                WHERE HIST.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                ORDER BY HIST.DTHISTGERENCIAMENTOCHK DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAprovacoes($parametros)
    {
        $sql = "SELECT CHK.NRCHECKLISTAGEND, GER.NRGERENCIAMENTOCHK, GER.NMGERENCIAMENTOCHK, CHK.NMCHECKLISTAGEND, CHK.NRSEQCONTRATO,
                CHK.NRLOCALCLIENTE, LOC.NMLOCALCLIENTE, LOC.CDCLIENTE, CLI.NMRAZSOCCLIE, CHK.DTHORAAGEND, CHK.DSJUSTCANCELAMENTO,
                CHK.CDOPERADOR AS OPERRESPONSAVEL, OPERCHK.NMOPERADOR AS NMOPERRESPONSAVEL, GER.CDOPERADOR, OPERGER.NMOPERADOR
                FROM FCM_CHECKLISTAGEND CHK
                JOIN FCM_GERENCIAMENTOCHK GER ON CHK.NRCHECKLISTAGEND = GER.NRCHECKLISTAGEND
                JOIN FCM_LOCALCLI LOC ON CHK.NRLOCALCLIENTE = LOC.NRLOCALCLIENTE
                JOIN CLIENTE CLI ON LOC.CDCLIENTE = CLI.CDCLIENTE
                JOIN OPERADOR OPERCHK ON CHK.CDOPERADOR = OPERCHK.CDOPERADOR
                JOIN OPERADOR OPERGER ON GER.CDOPERADOR = OPERGER.CDOPERADOR
                WHERE CHK.IDSITUACAOAGEND = 6
                AND GER.CDOPERADOR = :CDOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getOperadoresFiltrados($horarioInicial, $horarioFinal, $local)
    {
        $sql = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR
                FROM OPERADOR
                JOIN FCM_FUNCIONARIOSHIERARQ
                ON FCM_FUNCIONARIOSHIERARQ.CDOPERADOR = OPERADOR.CDOPERADOR
                JOIN FCM_HIERARQUIAFUNC
                ON FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC = FCM_FUNCIONARIOSHIERARQ.NRHIERARQUIAFUNC
                LEFT JOIN FCM_RELACHABOPER 
                ON FCM_RELACHABOPER.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN  FCM_RELACHABATIV
                ON  FCM_RELACHABATIV.NRHABILIDADE = FCM_RELACHABOPER.NRHABILIDADE
                LEFT JOIN FCM_TAREFACHK
                ON FCM_TAREFACHK.NRTAREFA = FCM_RELACHABATIV.NRTAREFA
                WHERE FCM_HIERARQUIAFUNC.NRLOCALCLIENTE IN (SELECT NRLOCALCLIENTE
                                                                FROM FCM_LOCALCLI
                                                            START WITH NRLOCALCLIENTE = $local
                                                            CONNECT BY PRIOR NRLOCALCLIENTEPAI = NRLOCALCLIENTE)
                AND OPERADOR.CDOPERADOR NOT IN (SELECT DISTINCT(CDOPERADOR)
                    FROM FCM_CHECKLISTAGEND
                    WHERE IDATIVO = 'S' AND
                    CDOPERADOR IS NOT NULL AND
                    (TO_DATE('".$horarioInicial."', 'DD-MM-YYYY HH24:MI') >= DTHORAAGEND AND TO_DATE('".$horarioInicial."', 'DD-MM-YYYY HH24:MI') <= DTHRFINALAGEND)
                    OR (TO_DATE('".$horarioFinal."', 'DD-MM-YYYY HH24:MI') >= DTHORAAGEND AND TO_DATE('".$horarioFinal."', 'DD-MM-YYYY HH24:MI') <= DTHRFINALAGEND)
                    OR (TO_DATE('".$horarioInicial."', 'DD-MM-YYYY HH24:MI') >= DTHORAAGEND AND TO_DATE('".$horarioFinal."', 'DD-MM-YYYY HH24:MI') <= DTHRFINALAGEND))";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function defineNovoHorarioOperadorTarefa($parametros)
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND
                SET DTHORAAGEND = :DTHORAAGEND,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU,
                    DTHRFINALAGEND = :DTHRFINALAGEND,
                    CDOPERADOR = :CDOPERADOR
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getOperadoresAprovacoes($parametros)
    {

        $sql = "SELECT CDOPERADOR, NMOPERADOR
                FROM OPERADOR
                WHERE CDOPERADOR IN ($parametros)
                ORDER BY NMOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
