<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class FmModeloPopFactory
{
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getModeloPop()
    {
        $sql = "SELECT 
            MP.NRMODELOPOP, 
            MP.NMMODELOPOP, 
            MP.DESCMODELOPOP, 
            MP.LINKMODELOPOP, 
            MP.IDATIVO, 
            MP.NRORG,
            AN.NMARQUIVO
        FROM 
            FCM_MODELOPOP MP
        LEFT JOIN 
            FCM_ANEXOMODELOPOP AN ON MP.NRMODELOPOP = AN.NRMODELOPOP";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirModeloPop(array $params)
    {
        $sql = "INSERT INTO FCM_MODELOPOP (
            NRMODELOPOP, NMMODELOPOP, DESCMODELOPOP, LINKMODELOPOP, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO
        ) VALUES (
            :NRMODELOPOP, :NMMODELOPOP, :DESCMODELOPOP, :LINKMODELOPOP, :IDATIVO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO
        )";

        $this->connection->executeStatement($sql, $params);
    }

    public function inserirAnexoModeloPop(array $params)
    {

        $sql = "INSERT INTO FCM_ANEXOMODELOPOP (
        NRANEXOMODELOPOP, NRMODELOPOP, NMARQUIVO, DSCAMINHOARQ, DSTIPO, VRTAMANHO, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO
         ) VALUES (
        :NRANEXOMODELOPOP, :NRMODELOPOP, :NMARQUIVO, :DSCAMINHOARQ, :DSTIPO, :VRTAMANHO, :IDATIVO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO
         )";

        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarModeloPop($params)
    {
        $sql = "UPDATE FCM_MODELOPOP
            SET NMMODELOPOP   = :NMMODELOPOP,
                CDOPERULTATU  = :CDOPERULTATU,
                DESCMODELOPOP = :DESCMODELOPOP,
                IDATIVO       = :IDATIVO,
                LINKMODELOPOP = :LINKMODELOPOP,
                NRORGULTATU   = :NRORGULTATU,
                DTULTATU      = SYSDATE
            WHERE NRMODELOPOP = :NRMODELOPOP";

        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarAnexoModeloPop($params)
    {
        $sql = "UPDATE FCM_ANEXOMODELOPOP
    SET NRMODELOPOP     = :NRMODELOPOP,
        NMARQUIVO      = :NMARQUIVO,
        DSCAMINHOARQ   = :DSCAMINHOARQ,
        DSTIPO         = :DSTIPO,
        VRTAMANHO      = :VRTAMANHO,
        IDATIVO        = :IDATIVO,
        CDOPERULTATU   = :CDOPERULTATU,
        NRORGULTATU    = :NRORGULTATU,
        DTULTATU       = SYSDATE
    WHERE NRANEXOMODELOPOP = :NRANEXOMODELOPOP";

        $this->connection->executeStatement($sql, $params);
    }

    public function excluirModeloPop($parametro)
    {
        $sql = "DELETE FROM FCM_MODELOPOP
                WHERE NRMODELOPOP = :NRMODELOPOP";
        $this->connection->executeStatement($sql, ['NRMODELOPOP' => $parametro['NRMODELOPOP']]);
    }

    public function deletarAnexoPop($params)
    {
        foreach ($params as $anexo) {
            if (isset($anexo['NRANEXOMODELOPOP'])) {
                error_log("Excluindo anexo: " . json_encode($anexo));
                $sql = "DELETE FROM FCM_ANEXOMODELOPOP
                    WHERE NRANEXOMODELOPOP = :NRANEXOMODELOPOP";
                $this->connection->executeStatement($sql, ['NRANEXOMODELOPOP' => $anexo['NRANEXOMODELOPOP']]);
            }
        }
    }

    public function buscarAnexosPorModelo($param)
    {
        $sql = "SELECT * FROM FCM_ANEXOMODELOPOP WHERE NRMODELOPOP = :NRMODELOPOP";
        $result = $this->connection->fetchAllAssociative($sql, ['NRMODELOPOP' => $param['NRMODELOPOP']]);

        return $result;
    }


    public function verificaRelacionamentoPop($param)
    {
        $sql = "SELECT c.NMCHECKLISTAGEND, c.NRCHECKLISTAGEND, c.DTHORAAGEND, c.IDSITUACAOAGEND, l.NMLOCALCLIENTE, cl.NMFANTCLIE
        FROM FCM_RELCHKAGENDMODPOP r
        JOIN FCM_CHECKLISTAGEND c ON r.NRCHECKLISTAGEND = c.NRCHECKLISTAGEND
        JOIN FCM_LOCALCLI l ON c.NRLOCALCLIENTE = l.NRLOCALCLIENTE
        JOIN CLIENTE cl ON cl.CDCLIENTE = l.CDCLIENTE
        WHERE r.NRMODELOPOP = :NRMODELOPOP
        AND c.IDATIVO = 'S'";

        $retorno = $this->connection->executeQuery($sql, $param)->fetchAllAssociative();
        return $retorno;
    }

}