<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class FmRelatorioAuditoriaFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getTarefasAuditoria($listaOperadores, $params)
    {
        $sql = "SELECT FCM_AUDITORIA.NRAUDITORIA, 
                       FCM_AUDITORIA.NMAUDITORIA, 
                       FCM_AUDITORIA.NRLOCALCLIENTE, 
                       FCM_AUDITORIA.IDTIPOMARCACAO, 
                       FCM_LOCALCLI.NMLOCALCLIENTE, 
                       FCM_LOCALCLI.CDCLIENTE, 
                       FCM_LOCALCLI.NRTIPOLOCALCLI, 
                       FCM_MODELOAUDIT.NRMODELOAUDIT, 
                       FCM_MODELOAUDIT.NMMODELOAUDIT, 
                       FCM_AUDITORIA.NRSEQCONTRATO, 
                       FCT_CONTRATO.NRCONTRATOFCT, 
                       FCM_AUDITORIA.CDOPERADOR, 
                       OPERADOR.NMOPERADOR, 
                       FCM_AUDITORIA.IDSITUACAOAUDIT, 
                       TO_CHAR(FCM_AUDITORIA.DTHORAAUDIT, 'DD/MM/YYYY') AS DTHORAAUDIT, 
                       FCM_AUDITORIA.IDTIPORECORRENCIA, 
                       FCM_TIPOLOCALCLI.NMTIPOLOCALCLI, 
                       CLIENTE.NMRAZSOCCLIE, 
                       CLIENTE.NMFANTCLIE, 
                       FCM_AUDITORIA.IDATIVO,
                       FCM_AUDITORIA.NOTATOTAL,      
                       FCM_AUDITORIA.PERCENTUALCONCLUSAO  
                  FROM FCM_AUDITORIA
                  LEFT JOIN FCM_LOCALCLI
                  ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_AUDITORIA.NRLOCALCLIENTE
                  JOIN CLIENTE
                  ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                  JOIN FCM_TIPOLOCALCLI
                  ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                  LEFT JOIN FCM_MODELOAUDIT
                  ON FCM_MODELOAUDIT.NRMODELOAUDIT = FCM_AUDITORIA.NRMODELOAUDIT
                  LEFT JOIN FCT_CONTRATO
                  ON FCT_CONTRATO.NRSEQCONTRATO = FCM_AUDITORIA.NRSEQCONTRATO
                  LEFT JOIN OPERADOR
                  ON OPERADOR.CDOPERADOR = FCM_AUDITORIA.CDOPERADOR
                  WHERE FCM_AUDITORIA.CDOPERADOR IN ($listaOperadores) 
                  OR FCM_AUDITORIA.CDOPERINCLUSAO = :CDOPERADOR";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getImagensAuditoria($parametros)
    {
        $sql = "SELECT NRFOTORESULTAUDIT, DSTYPE, NRSIZE, NRORG, IDATIVO, DTINCLUSAO, DTULTATU,
    NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRCATEGORIAAUDIT, NRAUDITORIA, DSFOTOAUDITORIA
     FROM FCM_FOTORESULTAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getCategoriasAuditoriaRelatorio($parametros)
    {
        $sql = "SELECT FCM_RELACMODCATEG.*, FCM_CATEGORIAAUDIT.NMCATEGORIAAUDIT 
                FROM FCM_RELACMODCATEG
                JOIN FCM_AUDITORIA
                ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_RELACMODCATEG.NRMODELOAUDIT
                JOIN FCM_CATEGORIAAUDIT
                ON FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT = FCM_RELACMODCATEG.NRCATEGORIAAUDIT
                WHERE FCM_AUDITORIA.NRAUDITORIA = :NRAUDITORIA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTopicosAuditoriaExecRelatorio($parametros)
    {
        $sql = "SELECT
                FCM_RELACCATTOPIC.*,
                FCM_TOPICOAUDIT.NMTOPICOAUDIT,
                FCM_OBSRESULTAUDIT.DSOBSERVACAO,
                FCM_OBSRESULTAUDIT.NROBSRESULTAUDIT
                FROM
                    FCM_RELACCATTOPIC
                JOIN FCM_AUDITORIA ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_RELACCATTOPIC.NRMODELOAUDIT
                JOIN FCM_TOPICOAUDIT ON FCM_TOPICOAUDIT.NRTOPICOAUDIT = FCM_RELACCATTOPIC.NRTOPICOAUDIT
                LEFT JOIN FCM_OBSRESULTAUDIT ON FCM_OBSRESULTAUDIT.NRRELACCATTOPIC = FCM_RELACCATTOPIC.NRRELACCATTOPIC AND FCM_OBSRESULTAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA
                WHERE
                FCM_AUDITORIA.NRAUDITORIA = :NRAUDITORIA
                AND FCM_RELACCATTOPIC.NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getCriteriosAuditoriaExecRelatorio($parametros)
    {
        $sql = "SELECT FCM_RELACRESULTCRIT.NRRELACRESULTCRIT, FCM_RELACRESULTCRIT.NRRESULTADOAUDIT, FCM_RELACRESULTCRIT.NRCRITERIOAUDIT, 
                    FCM_RELACRESULTCRIT.IDREALIZADO, FCM_RELACRESULTCRIT.NRTOPICOAUDIT, FCM_CRITERIOAUDIT.NMCRITERIOAUDIT, FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT
                        FROM FCM_RELACRESULTCRIT
                        JOIN FCM_RESULTADOAUDIT
                        ON FCM_RESULTADOAUDIT.NRRESULTADOAUDIT = FCM_RELACRESULTCRIT.NRRESULTADOAUDIT
                        JOIN FCM_CRITERIOAUDIT
                        ON FCM_CRITERIOAUDIT.NRCRITERIOAUDIT = FCM_RELACRESULTCRIT.NRCRITERIOAUDIT
                        JOIN FCM_AUDITORIA
                        ON FCM_RESULTADOAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA
                        WHERE FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT
                        AND FCM_RELACRESULTCRIT.NRTOPICOAUDIT = :NRTOPICOAUDIT
                        AND FCM_RESULTADOAUDIT.NRAUDITORIA = :NRAUDITORIA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function updateCriteriosResultado($parametros, $criterios)
    {
        $sql = "UPDATE FCM_RELACRESULTCRIT
                SET IDREALIZADO = :IDREALIZADO,
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU
                WHERE NRRELACRESULTCRIT IN ($criterios)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function existenciaObservacao($parametros)
    {
        $sql = "SELECT NROBSRESULTAUDIT
                FROM  FCM_OBSRESULTAUDIT
                WHERE NRAUDITORIA = :NRAUDITORIA
                    AND NRRELACCATTOPIC = :NRRELACCATTOPIC";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getImagensTopicosAuditoria($parametros)
    {
        $sql = "SELECT * FROM FCM_FOTORESULTAUDIT
                    WHERE NRAUDITORIA = :NRAUDITORIA
                    AND NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT
                    AND NRTOPICOAUDIT = :NRTOPICOAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function verificarAlteracoes($parametros)
    {
        $sql = "SELECT COUNT (NROBSRESULTAUDIT) as CONT
                FROM  FCM_OBSRESULTAUDIT
                WHERE NRAUDITORIA = :NRAUDITORIA
                AND NRRELACCATTOPIC = :NRRELACCATTOPIC
                AND DSOBSERVACAO LIKE :DSOBSERVACAO";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getCriteriosMarcadosAudit($parametros) 
    {
        // (
        //     SELECT DISTINCT NRCATEGORIAAUDIT FROM FCM_RESULTADOAUDIT
        //     WHERE NRTOPICOAUDIT = :NRTOPICOAUDIT
        // )
        $sql = "SELECT FCM_RELACRESULTCRIT.NRCRITERIOAUDIT, 
                       FCM_RELACRESULTCRIT.NRRESULTADOAUDIT
                FROM FCM_RELACRESULTCRIT
                JOIN FCM_RESULTADOAUDIT
                ON FCM_RESULTADOAUDIT.NRRESULTADOAUDIT = FCM_RELACRESULTCRIT.NRRESULTADOAUDIT
                JOIN FCM_CRITERIOAUDIT
                ON FCM_CRITERIOAUDIT.NRCRITERIOAUDIT = FCM_RELACRESULTCRIT.NRCRITERIOAUDIT
                JOIN FCM_AUDITORIA
                ON FCM_RESULTADOAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA
                WHERE FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT
                AND FCM_RELACRESULTCRIT.NRTOPICOAUDIT = :NRTOPICOAUDIT
                AND FCM_RESULTADOAUDIT.NRAUDITORIA = :NRAUDITORIA
                AND FCM_RELACRESULTCRIT.IDREALIZADO = 'S'";
    
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getCriteriosAudit($parametros) 
    {
        $sql = "SELECT FCM_RELACRESULTCRIT.NRCRITERIOAUDIT, 
                       FCM_RELACRESULTCRIT.NRRESULTADOAUDIT,
                       FCM_RELACRESULTCRIT.IDREALIZADO
                FROM FCM_RELACRESULTCRIT
                JOIN FCM_RESULTADOAUDIT
                ON FCM_RESULTADOAUDIT.NRRESULTADOAUDIT = FCM_RELACRESULTCRIT.NRRESULTADOAUDIT
                JOIN FCM_CRITERIOAUDIT
                ON FCM_CRITERIOAUDIT.NRCRITERIOAUDIT = FCM_RELACRESULTCRIT.NRCRITERIOAUDIT
                JOIN FCM_AUDITORIA
                ON FCM_RESULTADOAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA
                WHERE FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT = (
                    SELECT MAX(NRCATEGORIAAUDIT) FROM FCM_RESULTADOAUDIT 
                    WHERE NRTOPICOAUDIT = :NRTOPICOAUDIT
                )
                AND FCM_RELACRESULTCRIT.NRTOPICOAUDIT = :NRTOPICOAUDIT
                AND FCM_RESULTADOAUDIT.NRAUDITORIA = :NRAUDITORIA";
    
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }    

    public function getCriteriosAvaliacaoSomaNota()
    {
        $sql = "SELECT NRCRITERIOAUDIT FROM FCM_CRITERIOAUDIT
                WHERE IDSOMANOTA = 'S'";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getCriteriosAvaliacaoSomaPercentual()
    {
        $sql = "SELECT NRCRITERIOAUDIT FROM FCM_CRITERIOAUDIT
                WHERE IDSOMAPERCENTUAL = 'S'";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getNotaTopico($parametros)
    {
        $sql = "SELECT NOTATOPICAUDIT FROM FCM_TOPICOAUDIT
                WHERE NRTOPICOAUDIT = :NRTOPICOAUDIT";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function salvarPercentual($parametros)
    {
        $sql = "UPDATE FCM_AUDITORIA
                SET   PERCENTUALCONCLUSAO   = :PERCENTUALCONCLUSAO
                WHERE NRAUDITORIA           = :NRAUDITORIA";
                
        $this->connection->executeStatement($sql, $parametros);
    }

    public function salvarNota($parametros)
    {
        $sql = "UPDATE FCM_AUDITORIA
                SET   NOTATOTAL             = :NOTATOTAL
                WHERE NRAUDITORIA           = :NRAUDITORIA";
                
        $this->connection->executeStatement($sql, $parametros);
    }

    public function getParametrizacao() 
    {
        $sql = "SELECT IDUTILIZAPERCENTUALAUDIT, IDUTILIZANOTAAUDIT
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
}
