<?php

namespace FM\Service;

use FM\Factory\EPIsFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;

class EPIsService
{

    protected $EPIsFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;

    public function __construct(EPIsFactory $EPIsFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FindService $FindService)
    {
        $this->EPIsFactory = $EPIsFactory;
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
        $this->FindService = $FindService;
    }

    public function getEPIs()
    {
        return $this->EPIsFactory->getEPIs();
    }

    public function salvarEPI($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            foreach ($params as $row) {
                if ($row["NREPI"] == 0) {
                    $param = array(
                        "NREPI" => $this->newCode->getCodeByTableName("FCM_EPI", $nrorg, 1, 10, true),
                        "NMEPI" => $row["NMEPI"],
                        "IDATIVO" => $row["IDATIVO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador
                    );

                    $this->EPIsFactory->inserirEpis($param);
                } else if($row["IDATIVO"] === 'S'){
                    $param = array(
                        "NREPI" => $row["NREPI"],
                        "NMEPI" => $row["NMEPI"],
                        "IDATIVO" => $row["IDATIVO"],
                        "CDOPERULTATU" => $operador,
                        "NRORGULTATU" => $nrorg
                    );
                    $this->EPIsFactory->atualizarEPI($param);
                } else {
                    $paramsEpi = ["NREPI" => $row['NREPI']];
                    $tarefasAssociadas = $this->EPIsFactory->getTarefasCheck($paramsEpi);

                    $tarefasFiltradas = array_values(array_filter($tarefasAssociadas, function ($item) {
                        return !in_array($item['IDSITUACAOAGEND'], ['3', '4']);
                    }));

                    if(empty($tarefasFiltradas)) {
                        $param = array(
                            "NREPI" => $row["NREPI"],
                            "NMEPI" => $row["NMEPI"],
                            "IDATIVO" => $row["IDATIVO"],
                            "CDOPERULTATU" => $operador,
                            "NRORGULTATU" => $nrorg
                        );
                        $this->EPIsFactory->atualizarEPI($param);
                    } else {
                        return $tarefasFiltradas;
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    /*public function excluirEPI($params)
    {
        $this->connection->beginTransaction();
        try {
            $this->EPIsFactory->excluirEPI($this->FindService->formatarArray($params["ROW"]));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }*/

    public function excluirEPI($params)
    {
        $this->connection->beginTransaction();

        try {
            $mensagem = ['status' => 'Ok', 'tarefas' => []];

            foreach ($params['NRS'] as $param) {
                $paramsEpi = ["NREPI" => $param['NREPI']];

                $tarefasRelacionadas = $this->EPIsFactory->getTarefasCheck($paramsEpi);
                error_log("Tarefas encontradas para NREPI {$param['NREPI']}: " . json_encode($tarefasRelacionadas));

                if (count($tarefasRelacionadas) == 0) {
                    $this->EPIsFactory->excluirEPI($paramsEpi);
                } else {
                    $mensagem['status'] = 'erro';
                    $mensagem['tarefas'] = array_merge($mensagem['tarefas'], $tarefasRelacionadas);
                }
            }

            $this->connection->commit();
            return $mensagem;
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }
}
