<?php

namespace FM\Service;

use FM\Factory\Fm00003000ChecklistFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\Fm00000000HabilidadesService;

class Fm00003000ChecklistService
{

    protected $ChecklistFactory;
    protected $FindService;
    protected $Environment;
    protected $newCode;
    protected $habilidadeService;
    protected $connection;

    public function __construct(EntityManager $entityManager, Fm00003000ChecklistFactory $ChecklistFactory, FindService $FindService, Environment $Environment, NewCode $newCode, Fm00000000HabilidadesService $Fm00000000HabilidadesService)
    {
        $this->ChecklistFactory          = $ChecklistFactory;
        $this->FindService               = $FindService;
        $this->connection                = $entityManager->getConnection();
        $this->Environment               = $Environment;
        $this->newCode                   = $newCode;
        $this->habilidadeService         = $Fm00000000HabilidadesService;
    }

    public function salvarChecklist($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $nrchecklist = $this->newCode->getCodeByTableName("FCM_CHECKLIST", $NRORG, 1, 10, true);
            $cdOperador = $this->Environment->getCdOperador();

            $idAtivo = isset($parametros['IDATIVO']) ? $parametros['IDATIVO'] : null;

            $this->ChecklistFactory->salvarChecklist(array(
                "NRCHECKLIST"       => $nrchecklist,
                "NMCHECKLIST"       => $parametros['NMCHECKLIST'],
                "NRTIPOLOCALCLI"    => $parametros['NRTIPOLOCALCLI'],
                "NRORG"             => $NRORG,
                "CDOPERINCLUSAO"    => $cdOperador,
                "NRORGINCLUSAO"     => $NRORG,
                "IDATIVO"           => $idAtivo
            ));

            foreach ($parametros['NRATIVIDADE'] as $itens) {
                $nrAtividadeChk = $this->newCode->getCodeByTableName("FCM_ATIVIDADECHK", $NRORG, 1, 10, true);
                $idAtivo = isset($parametros['IDATIVO']) ? $parametros['IDATIVO'] : null;
                $this->ChecklistFactory->salvarItens(array(
                    "NRATIVIDADECHK"    => $nrAtividadeChk,
                    "NRATIVIDADE"       => $itens,
                    "NRCHECKLIST"       => $nrchecklist,
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $cdOperador,
                    "NRORGINCLUSAO"     => $NRORG,
                    "IDATIVO"               => $idAtivo
                ));

                foreach ($parametros['NRTAREFA'] as $atividade) {
                    $retorno = $this->ChecklistFactory->verificaExistenciaAtividade(array(
                        "NRATIVIDADE"   => $itens,
                        "NRTAREFA"      => $atividade
                    ));

                    $nrTarefaChk = $this->newCode->getCodeByTableName("FCM_TAREFACHK", $NRORG, 1, 10, true);
                    $idAtivo = isset($parametros['IDATIVO']) ? $parametros['IDATIVO'] : null;
                    if ($retorno != 0) {
                        $this->ChecklistFactory->salvarAtividade(array(
                            "NRTAREFACHK"       => $nrTarefaChk,
                            "NRTAREFA"          => $atividade,
                            "NRCHECKLIST"       => $nrchecklist,
                            "NRATIVIDADECHK"    => $nrAtividadeChk,
                            "NRORG"             => $NRORG,
                            "CDOPERINCLUSAO"    => $cdOperador,
                            "NRORGINCLUSAO"     => $NRORG,
                            "IDATIVO"               => $idAtivo
                        ));

                        foreach ($parametros['CRITERIOS'] as $criterio) {
                            $idAtivo = isset($parametros['IDATIVO']) ? $parametros['IDATIVO'] : null;
                            if ($criterio['NRTAREFA'] == $atividade) {
                                $this->ChecklistFactory->salvarCriterio(array(
                                    "NRCRITERIOTAREFACHK"   => $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFACHK", $NRORG, 1, 10, true),
                                    "NRCRITERIOTAREFA"      => $criterio['NRCRITERIO'],
                                    "NRTAREFACHK"           => $nrTarefaChk,
                                    "NRORG"                 => $NRORG,
                                    "CDOPERINCLUSAO"        => $cdOperador,
                                    "NRORGINCLUSAO"         => $NRORG,
                                    "IDATIVO"               => $idAtivo
                                ));
                            }
                        }

                        foreach ($parametros['ESCOPOS'] as $escopo) {
                            if ($escopo['NRTAREFA'] == $atividade) {
                                $this->ChecklistFactory->salvarEscopo(array(
                                    "NRESCOPOSTAREFACHK"   => $this->newCode->getCodeByTableName("FCM_ESCOPOSTAREFACHK", $NRORG, 1, 10, true),
                                    "NRCHECKLIST"          => $nrchecklist,
                                    "NRTAREFACHK"          => $nrTarefaChk,
                                    "NRESCOPOSTAREFA"      => $escopo['NRESCOPO'],
                                    "NRORG"                => $NRORG,
                                    "CDOPERINCLUSAO"       => $cdOperador,
                                    "NRORGINCLUSAO"        => $NRORG,
                                    "IDATIVO"               => "S",
                                ));
                            }
                        }
                    }
                }
            }
            $this->connection->commit();
            return "ok";
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    public function getChecklist()
    {
        return $this->ChecklistFactory->getChecklist();
    }

    public function getAtividadesChecklist($parametros)
    {
        $retorno = $this->FindService->formatarArray($parametros['NRATIVIDADE']);

        return $this->ChecklistFactory->getAtividadesChecklist($retorno);
    }

    public function getDadosChecklist($parametros)
{
    $nrTarefa = isset($parametros['NRTAREFA']) ? $parametros['NRTAREFA'] : null;

    $retorno = $this->FindService->formatarArray($nrTarefa);

    return $this->ChecklistFactory->getDadosChecklist($retorno);
}


    // FUNÇÕES DA TAB DE ITENS E CRITÉRIOS
    public function getItensChecklist()
    {
        return $this->ChecklistFactory->getItensChecklist();
    }

    public function salvarNovoCriterio(array $row): array
    {
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            $this->connection->beginTransaction();
            $NRCRITERIOTAREFA = $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFA", $NRORG, 1, 10, true);

            $params = array(
                'NRCRITERIOTAREFA'  => $NRCRITERIOTAREFA,
                'DSCRITERIOTAREFA'  => $row['NOME'],
                'NRTAREFA'          => $row['ATIVIDADES'],
                'NRORG'             => $NRORG,
                'CDOPERINCLUSAO'    => $cdOperador,
                'NRORGINCLUSAO'     => $NRORG
            );
            $this->ChecklistFactory->inserirCriterio($params);
            $this->connection->commit();

            return $this->salvarRelacaoCriterioTarefaChk($params);
        } catch (\Throwable $th) {
            $this->connection->rollback();
            throw $th;
        }
    }

    private function salvarRelacaoCriterioTarefaChk(array $row): array
    {
        try {
            $this->connection->beginTransaction();
            $NRCRITERIOTAREFACHK = $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFACHK", $row["NRORG"], 1, 10, true);

            $params = array(
                'NRORG'                 => $row["NRORG"],
                'NRORGINCLUSAO'         => $row["NRORG"],
                'NRTAREFACHK'           => $row["NRTAREFA"],
                'CDOPERINCLUSAO'        => $row["CDOPERINCLUSAO"],
                'NRCRITERIOTAREFACHK'   => $NRCRITERIOTAREFACHK,
                'NRCRITERIOTAREFA'      => $row["NRCRITERIOTAREFA"],
                "IDATIVO"               => $row["IDATIVO"],
            );

            $this->ChecklistFactory->salvarCriterio($params);
            $this->connection->commit();
            return $params;
        } catch (\Throwable $th) {
            $this->connection->rollback();
            throw $th;
        }
    }

    public function salvarItemChecklist($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();

        try {
            if ($parametros['NRATIVIDADE'] == null) {
                $params = array(
                    "NRATIVIDADE"       => $this->newCode->getCodeByTableName("FCM_ATIVIDADE", $NRORG, 1, 10, true),
                    "NMATIVIDADE"       => $parametros['NMATIVIDADE'],
                    "NRORG"             => $NRORG,
                    "CDOPERINCLUSAO"    => $this->Environment->getCdOperador(),
                    "NRORGINCLUSAO"     => $NRORG,
                    "DSCORIDATIVIDADE"  => $parametros['DSCORIDATIVIDADE']
                );

                $this->ChecklistFactory->inserirItemChecklist($params);
            } else {
                $params = array(
                    "NRATIVIDADE"       => $parametros['NRATIVIDADE'],
                    "CDOPERULTATU"      => $this->Environment->getCdOperador(),
                    "NRORGULTATU"       => $NRORG,
                    "DSCORIDATIVIDADE"  => $parametros['DSCORIDATIVIDADE']
                );
                $this->ChecklistFactory->updateCorItemChecklist($params);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarItem($parametros)
    {
        $this->connection->beginTransaction();
        $nrOrg = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            if($parametros['ATIVAR']) {
                $param = array(
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU"  => $nrOrg,
                    "NRATIVIDADE"  => $parametros['NRATIVIDADE']
                );

                $this->ChecklistFactory->ativarItem($param);
            } else {
                $ChecklistsItem = $this->ChecklistFactory->getChecklistsItem(array("NRATIVIDADE" => $parametros['NRATIVIDADE']));

                if (empty($ChecklistsItem)) {
                    $atividadesItem = $this->ChecklistFactory->getAtividadesItem(array("NRATIVIDADE" => $parametros['NRATIVIDADE']));
    
                    foreach ($atividadesItem as $atividade) {
                        $params = array(
                            "NRORGULTATU"     => $nrOrg,
                            "CDOPERULTATU"    => $cdOperador,
                            "NRITEMCHECKLIST" => $atividade['NRITEMCHECKLIST']
                        );
    
                        $this->ChecklistFactory->inativarTarefasChecklist($params);
                        $this->ChecklistFactory->inativarEscoposTarefasChecklist($params);
                        $this->ChecklistFactory->inativarCriteriosTarefasChecklist($params);
                    }
    
                    $param = array(
                        "CDOPERULTATU" => $cdOperador,
                        "NRORGULTATU"  => $nrOrg,
                        "NRATIVIDADE"  => $parametros['NRATIVIDADE']
                    );
    
                    $this->ChecklistFactory->inativarItem($param);
                } else {
                    return $ChecklistsItem;
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function excluirItem($parametros)
    {
        $this->connection->beginTransaction();
        $nrOrg = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();
        $mensagem = 'erro';
    
        try {
            $numChecklistsItem = (int)$this->ChecklistFactory->getNumChecklistsItem(array("NRATIVIDADE" => $parametros['NRATIVIDADE']));
            $numOutrosRegistrosFilhos = (int)$this->ChecklistFactory->getNumOutrosRegistrosFilhos(array("NRATIVIDADE" => $parametros['NRATIVIDADE']));
    
            if ($numChecklistsItem === 0 && $numOutrosRegistrosFilhos === 0) {
                $atividadesItem = $this->ChecklistFactory->getAtividadesItem(array("NRATIVIDADE" => $parametros['NRATIVIDADE']));
    
                foreach ($atividadesItem as $atividade) {
                    $params = array(
                        "NRORGULTATU"     => $nrOrg,
                        "CDOPERULTATU"    => $cdOperador,
                        "NRITEMCHECKLIST" => $atividade['NRITEMCHECKLIST']
                    );
    
                    $this->ChecklistFactory->excluirTarefasChecklist($params);
                    $this->ChecklistFactory->excluirEscoposTarefasChecklist($params);
                    $this->ChecklistFactory->excluirCriteriosTarefasChecklist($params);
                }
    
                $param = array(
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU"  => $nrOrg,
                    "NRATIVIDADE"  => $parametros['NRATIVIDADE']
                );
    
                $this->ChecklistFactory->excluirItem($param);
                $mensagem = 'Ok';
            }
            $this->connection->commit();
            return $mensagem;
    
        } catch (\Exception $e) {
            $this->connection->rollback();
    
        }
    }    
    
    
    public function getCriteriosChecklist($parametros)
    {
        $filtro = "";
        if(isset($parametros['NRTAREFA'])) {
            $filtro .= "WHERE NRTAREFA IN (" . $parametros["NRTAREFA"] . ")";
        }
        return $this->ChecklistFactory->getCriteriosChecklist($filtro);
    }

    public function salvarTarefaChecklist($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            foreach ($parametros['ROW'] as $row) {
                if($row['NRITEMCHECKLIST'] === 0) {
                    $nrItemChecklist = $this->newCode->getCodeByTableName("FCM_ITEMCHECKLIST", $NRORG, 1, 10, true);

                    $this->ChecklistFactory->inserirTarefaChecklist(array(
                        "NRITEMCHECKLIST"   => $nrItemChecklist,
                        "NRATIVIDADE"       => $parametros['NRATIVIDADE'],
                        "NRTAREFA"          => $row['NRTAREFA'],
                        "NRCRITERIOTAREFA"  => $row['NRCRITERIOTAREFA'],
                        "NRORG"             => $NRORG,
                        "CDOPERINCLUSAO"    => $cdOperador,
                        "NRORGINCLUSAO"     => $NRORG,
                        "NRESCOPOSTAREFA"   => $row['NRESCOPOSTAREFA'],
                    ));
        
                    foreach ($row['NRCRITERIOTAREFA'] as $criterio) {
                        $this->ChecklistFactory->inserirCriterioItemChecklist(array(
                            "NRITEMCHECKLISTCRIT"   => $this->newCode->getCodeByTableName("FCM_ITEMCHECKLISTCRIT", $NRORG, 1, 10, true),
                            "NRITEMCHECKLIST"       => $nrItemChecklist,
                            "NRCRITERIOTAREFA"      => $criterio,
                            "NRORG"                 => $NRORG,
                            "CDOPERINCLUSAO"        => $cdOperador,
                            "NRORGINCLUSAO"         => $NRORG,
                        ));
                    }
        
                    foreach ($row['NRESCOPOSTAREFA'] as $escopo) {
                        $this->ChecklistFactory->inserirEscopoItemChecklist(array(
                            "NRITEMCHECKLISTESC"    => $this->newCode->getCodeByTableName("FCM_ITEMCHECKLISTESC", $NRORG, 1, 10, true),
                            "NRITEMCHECKLIST"       => $nrItemChecklist,
                            "NRESCOPOSTAREFA"       => $escopo,
                            "NRORG"                 => $NRORG,
                            "CDOPERINCLUSAO"        => $cdOperador,
                            "NRORGINCLUSAO"         => $NRORG,
                    
                        ));
                    }
                } else {
                    if($row['IDATIVO'] === 'S') {
                        $this->ChecklistFactory->updateTarefaChecklist(array(
                            "NRITEMCHECKLIST"   => $row['NRITEMCHECKLIST'],
                            "NRTAREFA"          => $row['NRTAREFA'],
                            "NRCRITERIOTAREFA"  => $row['NRCRITERIOTAREFA'],
                            "IDATIVO"           => $row['IDATIVO'],
                            "NRORGULTATU"       => $NRORG,
                            "CDOPERULTATU"      => $cdOperador,
                            "NRESCOPOSTAREFA"   => $row['NRESCOPOSTAREFA'],
                        ));
                    } else {
                        $ChecklistsItem = $this->ChecklistFactory->getChecklistsItem(array("NRATIVIDADE" => $parametros['NRATIVIDADE']));

                        if (empty($ChecklistsItem)) {
                            $this->ChecklistFactory->updateTarefaChecklist(array(
                                "NRITEMCHECKLIST"   => $row['NRITEMCHECKLIST'],
                                "NRTAREFA"          => $row['NRTAREFA'],
                                "NRCRITERIOTAREFA"  => $row['NRCRITERIOTAREFA'],
                                "IDATIVO"           => $row['IDATIVO'],
                                "NRORGULTATU"       => $NRORG,
                                "CDOPERULTATU"      => $cdOperador,
                                "NRESCOPOSTAREFA"   => $row['NRESCOPOSTAREFA'],
                            ));
                        } else {
                            return $ChecklistsItem;
                        }
                    }
                }
            }
           
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarTarefasChecklist($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $mensagem = 'Ok';

        try {
            foreach ($parametros['NRS'] as $row) {
                $numChecklistsTarefa = $this->ChecklistFactory->getNumChecklistsTarefa(array("NRITEMCHECKLIST" => $row['NRITEMCHECKLIST']));

                if ($numChecklistsTarefa == 0) {
                    $params = array(
                        "NRORGULTATU"       => $NRORG,
                        "CDOPERULTATU"      => $this->Environment->getCdOperador(),
                        "NRITEMCHECKLIST"   => $row['NRITEMCHECKLIST']
                    );

                    $this->ChecklistFactory->inativarCriteriosTarefasChecklist($params);

                    $this->ChecklistFactory->inativarEscoposTarefasChecklist($params);

                    $this->ChecklistFactory->inativarTarefasChecklist($params);
                } else {
                    $mensagem = 'erro';
                }
            }

            $this->connection->commit();
            return $mensagem;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTarefasChecklist($parametros)
    {
        $param = array(
            "NRATIVIDADE" => $parametros['NRATIVIDADE']
        );

        return $this->ChecklistFactory->getTarefasChecklist($param);
    }

    // FUNÇÕES DA TAB ATIVIDADE E CRITÉRIOS
    public function getAtividadesCriterios()
    {
        return $this->ChecklistFactory->getAtividadesCriterios();
    }

    public function salvarAtividadeCriterio($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            if ($parametros['NRROW'] == null) {
                if ($parametros['TIPOADICAO'] ==  'Critério') {
                    $params = array(
                        'NRCRITERIOTAREFA'  => $this->newCode->getCodeByTableName("FCM_CRITERIOTAREFA", $NRORG, 1, 10, true),
                        'DSCRITERIOTAREFA'  => $parametros['NOME'],
                        'NRTAREFA'          => $parametros['ATIVIDADES'],
                        'NRORG'             => $NRORG,
                        'CDOPERINCLUSAO'    => $cdOperador,
                        'NRORGINCLUSAO'     => $NRORG,
                        "IDATIVO"               => $parametros["IDATIVO"]
                    );
                    $this->ChecklistFactory->inserirCriterio($params);
                } elseif ($parametros['TIPOADICAO'] ==  'Atividade') {
                    $params = array(
                        'NRTAREFA'              => $this->newCode->getCodeByTableName("FCM_TAREFA", $NRORG, 1, 10, true),
                        'NMTAREFA'              => $parametros['NOME'],
                        'NRORG'                 => $NRORG,
                        'CDOPERINCLUSAO'        => $cdOperador,
                        'NRORGINCLUSAO'         => $NRORG,
                        'NRPRIORIDADETAREFA'    => $parametros['PRIORIDADE'],
                        "IDATIVO"               => $parametros["IDATIVO"]
                    );
                    $this->ChecklistFactory->inserirAtividade($params);

                    foreach ($parametros["HABILIDADES"] as $row) {
                        $this->habilidadeService->relacionarAtividadeHabilidade(array_merge($params, ["NRHABILIDADE" => $row]));
                    }
                }
            } else if($parametros["IDATIVO"] === 'S'){
                if ($parametros['TIPOADICAO'] ==  'Critério') {
                    $params = array(
                        'NRCRITERIOTAREFA'  => $parametros['NRROW'],
                        'DSCRITERIOTAREFA'  => $parametros['NOME'],
                        'CDOPERULTATU'      => $cdOperador,
                        'NRORGULTATU'       => $NRORG,
                        "IDATIVO"               => $parametros["IDATIVO"]
                    );
                    $this->ChecklistFactory->updateCriterio($params);
                } else if ($parametros['TIPOADICAO'] ==  'Atividade') {
                    $params = array(
                        'NRTAREFA'              => $parametros['NRROW'],
                        'NMTAREFA'              => $parametros['NOME'],
                        'CDOPERULTATU'          => $cdOperador,
                        'NRORGULTATU'           => $NRORG,
                        'NRPRIORIDADETAREFA'    => $parametros['PRIORIDADE'],
                        "IDATIVO"               => $parametros["IDATIVO"],
                    );
                    $this->ChecklistFactory->updateAtividade($params);

                    $this->habilidadeService->excluirAtividadeHabilidades($parametros);

                    foreach ($parametros["HABILIDADES"] as $row) {
                        $this->habilidadeService->relacionarAtividadeHabilidade(array_merge($params, ["NRHABILIDADE" => $row]));
                    }
                }
            } else {
                if ($parametros['TIPOADICAO'] ==  'Critério') {
                    $nrCrit = array(
                        'NRCRITERIOTAREFA'=> $parametros['NRROW'],
                    );
                    $atividadesAssociadas = $this->ChecklistFactory->getCriteriosUtil($nrCrit);
                    if(empty($atividadesAssociadas)) {
                        $params = array(
                            'NRCRITERIOTAREFA'  => $parametros['NRROW'],
                            'DSCRITERIOTAREFA'  => $parametros['NOME'],
                            'CDOPERULTATU'      => $cdOperador,
                            'NRORGULTATU'       => $NRORG,
                            "IDATIVO"               => $parametros["IDATIVO"]
                        );
                        $this->ChecklistFactory->updateCriterio($params);
                    } else {
                        return $atividadesAssociadas;
                    }
                } else if ($parametros['TIPOADICAO'] ==  'Atividade') {
                    $nrCrit = array(
                        'NRITEMCHECKLIS'=> $parametros['NRROW'],
                    );
                    $itensAssociados = $this->ChecklistFactory->getAtividadesUtil($nrCrit);
                    if(empty($itensAssociados)) {
                        $params = array(
                            'NRTAREFA'              => $parametros['NRROW'],
                            'NMTAREFA'              => $parametros['NOME'],
                            'CDOPERULTATU'          => $cdOperador,
                            'NRORGULTATU'           => $NRORG,
                            'NRPRIORIDADETAREFA'    => $parametros['PRIORIDADE'],
                            "IDATIVO"               => $parametros["IDATIVO"],
                        );
                        $this->ChecklistFactory->updateAtividade($params);
    
                        $this->habilidadeService->excluirAtividadeHabilidades($parametros);
    
                        foreach ($parametros["HABILIDADES"] as $row) {
                            $this->habilidadeService->relacionarAtividadeHabilidade(array_merge($params, ["NRHABILIDADE" => $row]));
                        }
                    } else {
                        return $itensAssociados;
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarAtividadesCriterios($parametros)
{
    $this->connection->beginTransaction();

    try {
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();
        $contLocaisFilhos = 0;
        $arrayCriterios = [];
        $arrayAtividades = [];
        $mensagem = 'Ok';

        foreach ($parametros['NRS'] as $param) {
            if ($param['TIPO'] == 'Critério') {
                array_push($arrayCriterios, $param['CODIGO']);
            } elseif ($param['TIPO'] == 'Atividade') {
                array_push($arrayAtividades, $param['CODIGO']);
            }
        }

        foreach ($arrayCriterios as $criterio) {
            $numItens = $this->ChecklistFactory->getNumItensCriterio(['NRCRITERIOTAREFA' => $criterio]);

            if ($numItens == 0) {
                $this->ChecklistFactory->inativarCriterio([
                    "CODIGO" => $criterio,
                    "NRORGULTATU" => $NRORG,
                    "CDOPERULTATU" => $cdOperador
                ]);
            } else {
                throw new \Exception("Não foi possível inativar a atividade, pois ele está relacionado a uma tarefa.");
            }
        }

        foreach ($arrayAtividades as $atividade) {
            $filhos = $this->ChecklistFactory->verificarExistenciaFilhos(['CODIGO' => $atividade]);

            if ($filhos == 0) {
                $numItens = $this->ChecklistFactory->getNumItensAtividade(['NRTAREFA' => $atividade]);

                if ($numItens == 0) {
                    $this->ChecklistFactory->inativarAtividade([
                        "CODIGO" => $atividade,
                        "NRORGULTATU" => $NRORG,
                        "CDOPERULTATU" => $cdOperador,
                    ]);
                } else {
                    throw new \Exception("Não foi possível inativar a atividade ', pois ela está relacionada a uma tarefa.");
                }
            } else {
                $contLocaisFilhos++;
            }
        }

        if ($contLocaisFilhos > 0) {
            throw new \Exception('Não foi possível inativar a atividade, pois ela está relacionada a uma tarefa.');
        }

        $this->connection->commit();
        return $mensagem;
    } catch (\Exception $e) {
        $this->connection->rollback();
        throw $e; 
    }
}

    // FUNÇÕES DA TAB DE ESCOPO
    public function getEscoposServicos()
    {
        return $this->ChecklistFactory->getEscoposServicos();
    }

    public function verificarNomeItens(array $params): array
    {
        try {
            $NRORG = $this->Environment->getNrOrg();

            $params = array(
                "NRORG"         => $NRORG,
                "NOME"          => $params['NOME'],
            );
            return $this->ChecklistFactory->verificarNomeItens($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarEscopoServico($params)
{
    $this->connection->beginTransaction();
    $NRORG = $this->Environment->getNrOrg();
    $operador = $this->Environment->getCdOperador();

    try {
        foreach ($params as $row) {
            if ($row["NRESCOPOSTAREFA"] == 0) {
                $param = array(
                    "NRESCOPOSTAREFA"   => $this->newCode->getCodeByTableName("FCM_ESCOPOSTAREFA", $NRORG, 1, 10, true),
                    "NMESCOPOSTAREFA"   => $row["NMESCOPOSTAREFA"],
                    "IDATIVO"           => $row["IDATIVO"],
                    "NRORG"             => $NRORG,
                    "NRORGINCLUSAO"     => $NRORG,
                    "CDOPERINCLUSAO"    => $operador
                );

                $this->ChecklistFactory->inserirEscopoServico($param);
            } else if($row["IDATIVO"] === 'S'){
                $param = array(
                    "NRESCOPOSTAREFA"   => $row["NRESCOPOSTAREFA"],
                    "NMESCOPOSTAREFA"   => $row["NMESCOPOSTAREFA"],
                    "IDATIVO"           => $row["IDATIVO"],
                    "NRORG"             => $NRORG,
                    "CDOPERULTATU"      => $operador,
                    "NRORGINCLUSAO"     => $NRORG,
                    "CDOPERINCLUSAO"    => $operador,
                    "NRORGULTATU"       => $NRORG
                );

                $this->ChecklistFactory->updateEscopoServico($param);
            } else {
                $atividadesAssociadas = $this->ChecklistFactory->getEscoposUtil($row);
                if (empty($atividadesAssociadas)) { 
                    $param = array(
                        "NRESCOPOSTAREFA"   => $row["NRESCOPOSTAREFA"],
                        "NMESCOPOSTAREFA"   => $row["NMESCOPOSTAREFA"],
                        "IDATIVO"           => $row["IDATIVO"],
                        "NRORG"             => $NRORG,
                        "CDOPERULTATU"      => $operador,
                        "NRORGINCLUSAO"     => $NRORG,
                        "CDOPERINCLUSAO"    => $operador,
                        "NRORGULTATU"       => $NRORG
                    );
                    $this->ChecklistFactory->updateEscopoServico($param);
                } else {
                    return $atividadesAssociadas;
                }
            }
        }

        $this->connection->commit();
    } catch (\Exception $e) {
        $this->connection->rollback();
        throw $e;
    }
}


    public function inativarEscopos($parametros)
{
    $this->connection->beginTransaction();
    $NRORG = $this->Environment->getNrOrg();

    try {
        foreach ($parametros['NRS'] as $row) {
            $params = array(
                "NRORGULTATU"       => $NRORG,
                "CDOPERULTATU"      => $this->Environment->getCdOperador(),
                "NRESCOPOSTAREFA"   => $row['NRESCOPOSTAREFA'],
            );

            $numTarefasEscopo = $this->ChecklistFactory->getNumTarefasEscopo($params);

            if ($numTarefasEscopo === 0) {
                $this->ChecklistFactory->inativarEscopos($params);
            } else {
                throw new \Exception('Não foi possível inativar o escopo, pois ele está relacionado a uma tarefa.');
            }
        }

        $this->connection->commit();
    } catch (\Exception $e) {
        $this->connection->rollback();
        throw $e;
    }
}
    // FUNCOES PARA MODAL DE EDIÇÃO

    public function getTipoLocalModalEdicao($params)
    {
        $param = implode('', $params);
        return $this->ChecklistFactory->getTipoLocalModalEdicao($param);
    }

    public function getItensModalEdicao($params)
    {
        $param = implode('', $params);
        return $this->ChecklistFactory->getItensModalEdicao($param);
    }

    public function getAtividadesModalEdicao($params)
    {
        $resposta = array();
        $param = implode('', $params);
        array_push($resposta, array(
            'RETORNO' => $this->ChecklistFactory->getAtividadesModalEdicao($param),
            'NRCHECKLIST' => $this->validaUsoChecklist(),
        ));
        return $resposta;
    }

    public function getDadosChecklistEdicao($parametros)
    {
        $retorno = $this->FindService->formatarArray($parametros['NRTAREFA']);
        return $this->ChecklistFactory->getDadosChecklist($retorno);
    }

    public function validaUsoChecklist()
    {
        $retorno =  $this->ChecklistFactory->validaUsoChecklist();
        $params = array();
        foreach ($retorno['data'] as $dados) {
            if ($dados['VALIDADOR'] != null) {
                array_push($params, $dados['NRCHECKLIST']);
            }
        }
        return $params;
    }

    public function edicaoChecklist($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $nrchecklist = $parametros['CHECKLISTATUAL']['NRCHECKLIST'];

            if (isset($parametros['CHECKLISTATUAL']['NOME'])) {
                if ($parametros['CHECKLISTATUAL']['IDATIVO'] === 'S') {
                    $this->ChecklistFactory->editarChecklist(array(
                        "NRCHECKLIST"       => $nrchecklist,
                        "NMCHECKLIST"       => $parametros['CHECKLISTATUAL']['NMCHECKLIST'],
                        "NRTIPOLOCALCLI"    => $parametros['CHECKLISTATUAL']['NRTIPOLOCALCLI'],
                        "IDATIVO"           => isset($parametros['CHECKLISTATUAL']['IDATIVO']) ? $parametros['CHECKLISTATUAL']['IDATIVO'] : 'S',
                        "NRORGULTATU"       => $NRORG,
                        "CDOPERULTATU"      => $cdOperador,
                    ));
                } else {
                    $tarefasAssociadas = $this->ChecklistFactory->getChecklistsUtil($parametros['CHECKLISTATUAL']);
                    if(empty($tarefasAssociadas)) {
                        $this->ChecklistFactory->editarChecklist(array(
                            "NRCHECKLIST"       => $nrchecklist,
                            "NMCHECKLIST"       => $parametros['CHECKLISTATUAL']['NMCHECKLIST'],
                            "NRTIPOLOCALCLI"    => $parametros['CHECKLISTATUAL']['NRTIPOLOCALCLI'],
                            "IDATIVO"           => isset($parametros['CHECKLISTATUAL']['IDATIVO']) ? $parametros['CHECKLISTATUAL']['IDATIVO'] : 'S',
                            "NRORGULTATU"       => $NRORG,
                            "CDOPERULTATU"      => $cdOperador,
                        ));
                    } else {
                        return $tarefasAssociadas;
                    }
                }

            } else {
                $nratividadechk = [];
                foreach ($parametros['CHECKLISTATUAL']['NRATIVIDADE'] as $item) {
                    $retorno =  $this->ChecklistFactory->getNratividadechk($parametros['CHECKLISTATUAL']['NRCHECKLIST'], $item);
                    array_push($nratividadechk, $retorno[0]['NRATIVIDADECHK']);
                }
                $nratividadechk = implode(', ', $nratividadechk);
                $nrtarefachk = [];
                foreach ($parametros['CHECKLISTATUAL']['NRTAREFA'] as $item) {
                    $retorno =  $this->ChecklistFactory->getNrtarefachk($parametros['CHECKLISTATUAL']['NRCHECKLIST'], $item);
                    array_push($nrtarefachk, $retorno[0]['NRTAREFACHK']);
                }
                $nrtarefachk = implode(', ', $nrtarefachk);

                $this->ChecklistFactory->deleteChecklist($nrchecklist, $nrtarefachk, $nratividadechk);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarChecklist($parametros)
{
    $this->connection->beginTransaction();

    try {
        $mensagem = ['status' => 'Ok', 'tarefas' => []];

        foreach ($parametros['NRS'] as $param) {
            $params = array("NRCHECKLIST" => $param['NRCHECKLIST']);
         
            $tarefasChecklist = $this->ChecklistFactory->getTarefasCheck($params);
            error_log("Tarefas encontradas para NRCHECKLIST {$param['NRCHECKLIST']}: " . json_encode($tarefasChecklist));
        
            if (count($tarefasChecklist) == 0) {
                $this->ChecklistFactory->inativarChecklist($params);
            } else {
                $mensagem['status'] = 'erro';
                $mensagem['tarefas'] = array_merge($mensagem['tarefas'], $tarefasChecklist);
            }
        }

        $this->connection->commit();
        return $mensagem;
    } catch (\Exception $ex) {
        $this->connection->rollback();
        throw $ex;
    }
}
    
}
