<?php

namespace FM\Service;

use FM\Factory\Fm00004000TarefasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;
use FM\Service\Fm00010000ParametrizacaoGeralService;
use FM\Service\Fm00005000VistoriasService;

class Fm00004000TarefasService
{
    protected $TarefasFactory;
    protected $Environment;
    protected $newCode;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $connection;
    protected $parametrizacoes;
    protected $VistoriaService;
    public function __construct(EntityManager $entityManager, Fm00004000TarefasFactory $TarefasFactory, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $Environment, NewCode $newCode, Fm00010000ParametrizacaoGeralService $Fm00010000ParametrizacaoGeralService, Fm00005000VistoriasService $Fm00005000VistoriasService)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->TarefasFactory               = $TarefasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->parametrizacoes              = $Fm00010000ParametrizacaoGeralService;
        $this->VistoriaService              = $Fm00005000VistoriasService;
    }

    /*  Integração SM */
    public function getContratos($parametros)
    {
        return $this->TarefasFactory->getContratos(array(
            "NRESTRUTURAM" => empty($parametros['NRESTRUTURAM']) ? null : $parametros['NRESTRUTURAM'],
        ));
    }
    /*  Integração SM */

    public function getTarefasGrid()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->TarefasFactory->getTarefasGrid($params, $listaOperadores);
    }

    public function getClientes()
    {
        return $this->TarefasFactory->getClientes();
    }

    public function getTipoLocalTarefa($parametros)
    {
        $arrayClientes = (is_array($parametros['CDCLIENTE'])) ? $this->FindService->formatarArray($parametros['CDCLIENTE']) : "'" . $parametros['CDCLIENTE'] . "'";
        return $this->TarefasFactory->getTipoLocalTarefa($arrayClientes);
    }

    public function getLocal($parametros)
    {
        $tipoLocal = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->TarefasFactory->getLocal($tipoLocal, "'" . $parametros['CDCLIENTE'] . "'");
    }

    public function getChecklistTarefa($parametros)
    {

        $param = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->TarefasFactory->getChecklistTarefa($param);
    }

    public function getItensChecklist($parametros)
    {
        return $this->TarefasFactory->getItensChecklist($parametros);
    }

    public function getEscopo($parametros)
    {
        $listaAtividadeChk = $this->FindService->formatarArray($parametros['NRATIVIDADECHK']);
        return $this->TarefasFactory->getEscopo($listaAtividadeChk);
    }

    public function getTarefasSelect($parametros)
    {
        $listaEscopos = $this->FindService->formatarArray($parametros['NRESCOPOTAREFA']);
        $listaAtividadeChk = $this->FindService->formatarArray($parametros['NRATIVIDADECHK']);
        return $this->TarefasFactory->getTarefasSelect($listaEscopos, $listaAtividadeChk);
    }


    public function getCriterios($parametros)
    {
        if (!isset($parametros["MODAL"])) {
            $listaEscopos = $this->FindService->formatarArray($parametros['NRESCOPOTAREFA']);
            $listaAtividadeChk = $this->FindService->formatarArray($parametros['NRATIVIDADECHK']);
            $listaTarefaChk = $this->FindService->formatarArray($parametros['NRTAREFACHK']);
            $param = array(
                "NRCHECKLIST" => $parametros['NRCHECKLIST']
            );
        } else {
            $param = array(
                "NRTAREFACHK" => $parametros['NRTAREFACHK'],
                "NRATIVIDADECHK" => $parametros['NRATIVIDADECHK'],
                "NRCHECKLIST" => $parametros['NRCHECKLIST']
            );
        }

        return $this->TarefasFactory->getCriterios($param, $listaEscopos, $listaAtividadeChk, $listaTarefaChk);
    }

    public function converterDataBrasileiraParaAmericana($dataHora)
    {
        // Divide a string em data e hora
        list($data, $hora) = explode(' ', $dataHora);

        // Divide a data no formato dia/mês/ano
        list($dia, $mes, $ano) = explode('/', $data);

        // Reorganiza para o formato americano
        $dataHoraAmericana = "$mes/$dia/$ano $hora";

        return $dataHoraAmericana; // Saída: 10/30/2024 12:00

    }

    public function verificarConflitoDatas($dadosX, $dadosY)
    {
        // Converter as strings de data/hora para objetos DateTime
        $inicio1 = new \DateTime($this->converterDataBrasileiraParaAmericana($dadosX[0]));
        $fim1 = new \DateTime($this->converterDataBrasileiraParaAmericana($dadosX[1]));

        $inicio2 = new \DateTime($dadosY['DATAINICIAL']);
        $fim2 = new \DateTime($dadosY['DATAFINAL']);

        // Verificar se há conflito de datas
        if (($inicio1 >= $inicio2 && $inicio1 <= $fim2) ||
            ($fim1 >= $inicio2 && $fim1 <= $fim2) ||
            ($inicio1 <= $inicio2 && $fim1 >= $fim2)
        ) {
            return true;
        }
        return false;
    }


    public function validarConflitoEdicao($params, $datas)
    {
        if ($params['CDOPERADOR'] !== null) {
            // Cria um objeto DateTime a partir da string original
            $dadosBanco = $this->TarefasFactory->getTarefasPorOperadorEdicao(array(
                "CDOPERADOR"        => $params['CDOPERADOR'], 
                "DTHORAAGEND"       => explode(' ', $datas[0])[0],
                "NRCHECKLISTAGEND"  => $params["NRCHECKLISTAGEND"],
            ));

            if (count($dadosBanco) >  0) {
                // Compara a data do front com as datas vindas do banco
                foreach ($dadosBanco as $dadoBack) {
                    if ($this->verificarConflitoDatas($datas, $dadoBack)) {
                        $params['GERATAREFA'] = ($params['PARAMETRIZACAO'] === 'N') ? 'S' : 'N';
                    }
                }
            }  else {
                $params['GERATAREFA'] = 'S';
            }
        }
        return $params;
    }

    public function salvarEdicaoTarefas($params)
    {
        $this->connection->beginTransaction();
        try {
            $operadorLogado = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();
            
            $datas = $this->calcularDuracao($params['DTHORAAGEND'], $params['HORAAGEND'], $params['DURACAO']);

            if ($params['IDTAREFAEQUIPE'] === 'N' && $params['NREQUIPE'] !== null) { // Está alterando de operador específico para equipe.
                $this->inserirEquipe($params['NREQUIPE'], $params['NRCHECKLISTAGEND']);
                $params['IDTAREFAEQUIPE'] = 'S';
                $params['GERATAREFA'] = 'S';
            } elseif ($params['IDTAREFAEQUIPE'] === 'S' && $params['NREQUIPE'] !== null) { // Esta alterando de uma equipe para outra
                $this->updateEquipe($params['NREQUIPETAREFA'], $params['NREQUIPE'], $params['NRCHECKLISTAGEND']);
                $params['GERATAREFA'] = 'S';
            } else { // Está alterando para sem operador especifico ou trocando de operador
                if ($params['PARAMETRIZACAO'] !== 'N') {
                    $params = $this->validarConflitoEdicao($params, $datas);
                } else {
                    $params['GERATAREFA'] = 'S';
                }

                if ($params['GERATAREFA'] == 'S') {
                    if ($params['NREQUIPETAREFA'] !== null) { // caso houvesse equipe antes, está apagando esta relacao
                        $params['IDTAREFAEQUIPE'] = 'N';
                        $this->deleteRelacEquipeTarefa($params["NRCHECKLISTAGEND"]);
                    }
                }
            }
        
            if ($params['GERATAREFA'] == 'S') {
                $this->TarefasFactory->updateTarefa(array(
                    "NRCHECKLISTAGEND"      => ($params["NRCHECKLISTAGEND"]),
                    "NMCHECKLISTAGEND"      => $params["NMCHECKLISTAGEND"],
                    "DTHORAAGEND"           => $datas[0],
                    "DTHRFINALAGEND"        => $datas[1],
                    "IDOBRIGAOBSERVACAO"    => ($params["IDOBRIGAOBSERVACAO"] == '1') ? 'N' : 'S',
                    "IDTIPOMARCACAOTAREFA"  => $params["IDTIPOMARCACAOTAREFA"],
                    "CDOPERADOR"            => $params["CDOPERADOR"],
                    "NRPRIORIDADETAREFA"    => $params["NRPRIORIDADETAREFA"],
                    "IDTAREFAEQUIPE"        => $params["IDTAREFAEQUIPE"],
                    "CDOPERULTATU"          => $operadorLogado,
                ));
    
                $this->inserirHistorico($params["NRCHECKLISTAGEND"], $params["NMCHECKLISTAGEND"], 'Os dados da tarefa foram alterados.');
    
                /* Caso a tarefa possua vistoria */
                if ($params['IDCRIAVISTORA'] == '1') {
                    $this->TarefasFactory->updateResponsavelVistoria(array(
                        "NRCHECKLISTAGEND"  => $params["NRCHECKLISTAGEND"], 
                        "CDOPERADOR"        => $params["ENCARREGADO"],
                        "CDOPERULTATU"      => $operadorLogado, 
                        "NRORGULTATU"       => $NRORG,
                    ));
    
                    $this->VistoriaService->inserirHistoricoVistoria($params["NRGERENCIAMENTOCHK"], $params["NMCHECKLISTAGEND"], 'O responsável da vistoria foi alterado');
                }
    
                $this->validarEdicaoEpi($params["NRCHECKLISTAGEND"], $params["NREPI"]);
                
    
                /* Caso a tarefa possua modelo de procedimento */
                if ($params['NRRELCHKAGENDMODPOP'] !== null) {
                    $this->TarefasFactory->updateModeloPop(array(
                        "NRRELCHKAGENDMODPOP"   => $params["NRRELCHKAGENDMODPOP"], 
                        "NRMODELOPOP"           => $params["NRMODELOPOP"],
                        "DSOBSERVACAO"          => $params["DSOBSERVACAO"],
                        "CDOPERULTATU"          => $operadorLogado, 
                        "NRORGULTATU"           => $NRORG,
                    ));
                } elseif ($params['NRMODELOPOP'] !== null) {
                    $this->inserirRelacModeloPOP($params['NRMODELOPOP'], $params["NRCHECKLISTAGEND"], $params["DSOBSERVACAO"]);
                }      
            } else {
                throw new \Exception("Operação bloqueada!<br><br> Há conflitos de horário na tarefa que você está editando com uma tarefa existente. Altere o horário, o Operador, a Equipe ou desative a parametrização para permitir a inserção de múltiplas tarefas no mesmo horário.");
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public fuNction validarEdicaoEpi($nrchecklistagend, $nrepi)
    {
        $original = $this->TarefasFactory->getEpisRelacionados(array(
            "NRCHECKLISTAGEND" => $nrchecklistagend,
        ));
        $modificado = $nrepi;

        // Função para verificar os itens removidos
        $removidos = array_filter($original, function ($itemOriginal) use ($modificado) {
            foreach ($modificado as $itemModificado) {
                if (intval($itemOriginal['NREPI']) === $itemModificado) {
                    return false; // Se o ID está presente, não foi removido
                }
            }
            return true; // Se não encontrou o ID no array modificado, foi removido
        });

        if (count($removidos) > 0) {
            foreach ($removidos as $removido) {
                $this->TarefasFactory->deletarEpis(null, $removido['NRRELCHKAGENDEPI']);
                // deletarEpi                   
            }
        }
    
        // Função para verificar os itens adicionados
        $adicionados = array_filter($modificado, function ($itemModificado) use ($original) {
            foreach ($original as $itemOriginal) {
                if ($itemModificado === intval($itemOriginal['NREPI'])) {
                    return false; // Se o ID está presente no original, não foi adicionado
                }
            }
            return true; // Se não encontrou o ID no array original, foi adicionado
        });

        if (count($adicionados) > 0) {
            foreach ($adicionados as $adicionado) {
                $this->inserirRelacEpiTarefa($adicionado, $nrchecklistagend);
            }
        }
    }
    public function getFuncionario($parametros)
    {
        $local = '';
        $habilidade = '';

        if (!empty($parametros['NRLOCALCLIENTE'])) {
            if (is_array($parametros['NRLOCALCLIENTE'])) {
                $local .= $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']);
            } else {
                $local .= "'" . $parametros['NRLOCALCLIENTE'] . "'";
            }

            // if ($param[0]['IDUTILIZAHABILIDADE'] == 'S') {
            //     if (is_array($parametros['NRTAREFACHK'])) {
            //         $habilidade .= "AND NRTAREFACHK IN (". $this->FindService->formatarArray($parametros['NRTAREFACHK']) . ")";
            //     } else {
            //         $habilidade .= "AND NRTAREFACHK = '".$parametros['NRTAREFACHK']."'";
            //     }
            // }
            return $this->TarefasFactory->getFuncionario($local, $habilidade);
        }
    }
    
    public function getEquipeTarefa($parametros)
    {
        $local = '';
        $habilidade = '';

        if (!empty($parametros['NRLOCALCLIENTE'])) {
            if (is_array($parametros['NRLOCALCLIENTE'])) {
                $local .= $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']);
            } else {
                $local .= "'" . $parametros['NRLOCALCLIENTE'] . "'";
            }

            // if ($param[0]['IDUTILIZAHABILIDADE'] == 'S') {
            //     if (is_array($parametros['NRTAREFACHK'])) {
            //         $habilidade .= "AND NRTAREFACHK IN (". $this->FindService->formatarArray($parametros['NRTAREFACHK']) . ")";
            //     } else {
            //         $habilidade .= "AND NRTAREFACHK = '".$parametros['NRTAREFACHK']."'";
            //     }
            // }
            return $this->TarefasFactory->getEquipeTarefa($local);
        }
    }

    public function getSupervisorTarefa($parametros)
    {
        $operador = $this->Environment->getCdOperador();
        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $locais = is_array($parametros['NRLOCALCLIENTE']) ? $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']) : "'" . $parametros['NRLOCALCLIENTE'] . "'";
            
            if ($parametros['RESP'] == 'EQUIPE') {
                $operadores = $this->TarefasFactory->buscarOperadoresEquipe(array(
                    "NREQUIPE" => $parametros['CODIGO'],
                ));
                $novoArrayOperadores = array();
                
                foreach ($operadores as $operador) {
                    array_push($novoArrayOperadores, $operador["CDOPERADOR"]);
                }
                
                $operadores = $this->FindService->formatarArray($novoArrayOperadores);
            } else {
                $operadores = $parametros['CODIGO'];
            }
            
            $data = $this->TarefasFactory->getSupervisorTarefa($locais, $operadores);
            return $data;
        }
    }
    public function getItensEpi()
    {
        return $this->TarefasFactory->getEpis();
    }

    public function getItensPop()
    {
        return $this->TarefasFactory->getModeloPop();
    }

    public function inserirEquipe($nrequipe, $nrchecklistagend) {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $operadorLogado = $this->Environment->getCdOperador();

            $this->TarefasFactory->inserirEquipe(array(
                "NREQUIPETAREFA"    => $this->newCode->getCodeByTableName("FCM_EQUIPETAREFA", $NRORG, 1, 10, true), 
                "NREQUIPE"          => $nrequipe, 
                "NRCHECKLISTAGEND"  => $nrchecklistagend, 
                "NRORG"             => $NRORG, 
                "CDOPERINCLUSAO"    => $operadorLogado, 
                "NRORGINCLUSAO"     => $NRORG,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function updateEquipe($nrequipetarefa, $nrequipe, $nrchecklistagend)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $operadorLogado = $this->Environment->getCdOperador();

            $this->TarefasFactory->updateEquipe(array(
                "NREQUIPETAREFA"    => $nrequipetarefa, 
                "NREQUIPE"          => $nrequipe, 
                "NRCHECKLISTAGEND"  => $nrchecklistagend, 
                "CDOPERULTATU"      => $operadorLogado, 
                "NRORGULTATU"       => $NRORG,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function inserirHistorico($nrchecklistagend, $nmchecklistagend, $descricao)
    {
        $this->connection->beginTransaction();
        try {

            $NRORG = $this->Environment->getNrOrg();
            $operadorLogado = $this->Environment->getCdOperador();

            $this->TarefasFactory->atualizaHistoricoTarefa(array(
                "NRCHKAGENDHISTORICO" => $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $NRORG, 1, 10, true),
                "NRCHECKLISTAGEND"    => $nrchecklistagend,
                "NMCHECKLISTAGEND"    => $nmchecklistagend,
                "DSCHKAGENDHISTORICO" => $descricao,
                "IDTIPOHISTORICO"     => 1,
                "NRORG"               => $NRORG,
                "CDOPERINCLUSAO"      => $operadorLogado,
                "NRORGINCLUSAO"       => $NRORG,
                "CDOPERULTATU"        => $operadorLogado,
                "NRORGULTATU"         => $NRORG
            ));
        
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function inserirRelacModeloPOP($nrmodelopop, $nrchecklistagend, $obs)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $operadorLogado = $this->Environment->getCdOperador();

            $this->TarefasFactory->insertModeloPop(array(
                "NRRELCHKAGENDMODPOP"   => $this->newCode->getCodeByTableName("FCM_RELCHKAGENDMODPOP", $NRORG, 1, 10, true),
                "NRMODELOPOP"           => $nrmodelopop,
                "NRCHECKLISTAGEND"      => $nrchecklistagend,
                "DSOBSERVACAO"          => $obs,
                "IDATIVO"               => 'S',
                "CDOPERINCLUSAO"        => $operadorLogado,
                "CDOPERULTATU"          => $operadorLogado,
                "NRORG"                 => $NRORG,
                "NRORGINCLUSAO"         => $NRORG,
                "NRORGULTATU"           => $NRORG,
            ));
         $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function inserirRelacEpiTarefa($nrepi, $nrchecklistagend)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $operadorLogado = $this->Environment->getCdOperador();

            $this->TarefasFactory->insertEpis(array(
                "NRRELCHKAGENDEPI"  => $this->newCode->getCodeByTableName("FCM_RELCHKAGENDEPI", $NRORG, 1, 10, true),
                "NREPI"             => $nrepi,
                "NRCHECKLISTAGEND"  => $nrchecklistagend,
                "IDATIVO"           => 'S',
                "CDOPERINCLUSAO"    => $operadorLogado,
                "NRORG"             => $NRORG,
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function deleteRelacEquipeTarefa($nrchecklistagend)
    {
        $this->connection->beginTransaction();

        try {
            $this->TarefasFactory->deleteRelacEquipeTarefa(array(
                "NRCHECKLISTAGEND" => $nrchecklistagend,
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function insertNovaTarefa($param)
    {
        $this->connection->beginTransaction();
        try {
                $NRORG = $this->Environment->getNrOrg();
                $operadorLogado = $this->Environment->getCdOperador();

                switch ($param['OBSERVACOES']) {
                    case 1:
                        $observacoesTarefa = "N";
                        $observacoesVistoria = "N";
                        break;
                    case 2:
                        $observacoesTarefa = "S";
                        $observacoesVistoria = "N";
                        break;
                    case 3:
                        $observacoesTarefa = "N";
                        $observacoesVistoria = "S";
                        break;
                    default:
                        $observacoesTarefa = "S";
                        $observacoesVistoria = "S";
                        break;
                }

            foreach ($param["DADOSTAREFAS"] as $row) {
                if ($row["GERATAREFA"] == 'S') {
                    $nrchecklistagend = $this->newCode->getCodeByTableName("FCM_CHECKLISTAGEND", $NRORG, 1, 10, true);
                    $dataInicial = new \DateTime($row['DATAINICIAL']);
                    $dataFinal = new \DateTime($row['DATAFINAL']);
                    
                    $this->TarefasFactory->insertChecklistagend(array(
                        "NRCHECKLISTAGEND"      => $nrchecklistagend,
                        "NMCHECKLISTAGEND"      => $param['NOMETAREFA'],
                        "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                        "NRCHECKLIST"           => $param['NRCHECKLIST'],
                        "CDOPERADOR"            => (empty($row['FUNCIONARIO'])) ? null : $row['FUNCIONARIO'],
                        "IDSITUACAOAGEND"       => 1,
                        "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                        "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                        "CDOPERINCLUSAO"        => $operadorLogado,
                        "IDOBRIGAOBSERVACAO"    => $observacoesTarefa,
                        "NRSEQCONTRATO"         => isset($row['NRSEQCONTRATO']) ? $row['NRSEQCONTRATO'] : null,
                        "IDTIPOMARCACAOTAREFA"  => $row['IDTIPOMARCACAOTAREFA'],
                        "IDTIPORECORRENCIA"     => $param['RECORRENCIA'],
                        "NRPRIORIDADETAREFA"    => $param['NRPRIORIDADETAREFA'],
                        "IDTAREFAEQUIPE"        => (empty($row['FUNCIONARIO']) && !empty($row['EQUIPE'])) ? 'S' : 'N',
                        "NRESCOPOSTAREFA"       => $row['NRESCOPOSTAREFA'],
                    ));

                    /* Inserção de equipe como responsáveis da tarefa */
                    if (empty($row['FUNCIONARIO']) && !empty($row['EQUIPE'])) {
                        $this->inserirEquipe($row['EQUIPE'], $nrchecklistagend);
                    }

                    /* Inserção do histórico da tarefa */
                    $this->inserirHistorico($nrchecklistagend, $param['NOMETAREFA'], '"Tarefa criada."');

                    /* Inserção das atividades que serão realizadas na tarefa */
                    foreach ($param['NRTAREFACHK'] as $atividade) {
                        $escopos = $this->tratarNRESCOPOSTAREFA($atividade['NRESCOPOSTAREFA']);

                        foreach($escopos as $escopo) {
                            if ($row['NRESCOPOSTAREFA'] === $escopo) {
                                $this->TarefasFactory->insertTarefa(array(
                                    "NRTAREFAAGEND" => $this->newCode->getCodeByTableName("FCM_TAREFAAGEND", $NRORG, 1, 10, true),
                                    "NRCHECKLISTAGEND" => $nrchecklistagend,
                                    "NRTAREFACHK" => $atividade['NRTAREFACHK'],
                                    "CDOPERADOR" => (empty($row['FUNCIONARIO'])) ? null : $row['FUNCIONARIO'],
                                    "DTHORAAGEND" => $dataInicial->format('d/m/Y H:i'),
                                    "DSOBSTAREFAAGEND" => null,
                                    "CDOPERINCLUSAO" => $operadorLogado,
                                    "NRTAREFARECORRENTE" =>  isset($param["NRTAREFARECORRENTE"]) ? $param["NRTAREFARECORRENTE"] : null,
                                ));
                            }
                        }
                    }
                    /* Inserção na tabela de relação dos EPIS que serão utilizados na tarefa,  */
                    if (isset($param['NREPI']) && !empty($param['NREPI'])) {
                        foreach ($param['NREPI'] as $epi) {
                            $this->inserirRelacEpiTarefa($epi, $nrchecklistagend);
                        }
                    }
                
                    /* Inserção dos Modelos de Procedimento Padrão que serão utilizados na tarefa */
                    if (isset($param['NRMODELOPOP']) && !empty($param['NRMODELOPOP'])) {
                        $this->inserirRelacModeloPOP($param['NRMODELOPOP'], $nrchecklistagend, $param['DSOBSERVACAO']);
                    }    
                    
                    if ($param["VISTORIA"] == 1) {
                        $nrgerenciamentochk = $this->newCode->getCodeByTableName("FCM_GERENCIAMENTOCHK", $NRORG, 1, 10, true);
                        $params = array(
                            "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                            "NMGERENCIAMENTOCHK"    => $param['NOMETAREFA'],
                            "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                            "NRCHECKLIST"           => $param['NRCHECKLIST'],
                            "CDOPERADOR"            => $row['ENCARREGADO'],
                            "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                            "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                            "OPERADOR"              => $operadorLogado,
                            "NRCHECKLISTAGEND"      => $nrchecklistagend,
                            "IDOBRIGAOBSERVACAO"    => $observacoesVistoria,
                            "NRSEQCONTRATO"         => (isset($row['NRSEQCONTRATO'])) ? $row['NRSEQCONTRATO'] : null,
                        );
        
                        $this->TarefasFactory->insertChk($params);
        
                        $parametroHistorico = array (
                            "NRGERCHKHISTORICO"  => $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true),
                            "NRGERENCIAMENTOCHK" => $nrgerenciamentochk,
                            "NMGERENCIAMENTOCHK" => $param['NOMETAREFA'],
                            "DSGERENCIAMENTOCHK" => "Vistoria criada a partir de uma tarefa.",
                            "IDTIPOHISTORICO"    => 1,
                            "NRORG"              => $NRORG,
                            "CDOPERINCLUSAO"     => $operadorLogado,
                            "NRORGINCLUSAO"      => $NRORG,
                            "CDOPERULTATU"       => $operadorLogado,
                            "NRORGULTATU"        => $NRORG
                        );
        
                        $this->TarefasFactory->insertChkHistorico($parametroHistorico);
        
                        foreach ($param['NRTAREFACHK'] as $atividade) {
                            $escopos = $this->tratarNRESCOPOSTAREFA($atividade['NRESCOPOSTAREFA']);

                            foreach($escopos as $escopo) {
                                if ($row['NRESCOPOSTAREFA'] === $escopo) {
                                    $nrtarefaagendchk = $this->newCode->getCodeByTableName("FCM_TAREFAGERCHK", $NRORG, 1, 10, true);
                                    $this->TarefasFactory->insertChkTarefa(array(
                                        "NRTAREFAGERCHK"        => $nrtarefaagendchk,
                                        "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                                        "NRTAREFACHK"           => $atividade['NRTAREFACHK'],
                                        "CDOPERADOR"            => (empty($row['FUNCIONARIO'])) ? null : $row['FUNCIONARIO'],
                                        "DTHORATAREFAGERCHK"    => $dataInicial->format('d/m/Y H:i'),
                                        "DSOBSTAREFAGERCHK"     => null,
                                        "OPERADOR"              => $operadorLogado,
                                    ));
                
                                    foreach ($param['NRCRITERIOTAREFACHK'] as $criterios) {
                                        $valida = $this->TarefasFactory->validaCriterio(array(
                                            "NRTAREFACHK"       => $atividade['NRTAREFACHK'],
                                            "NRCRITERIOTAREFA"  => $criterios['NRCRITERIOTAREFA']
                                        ));

                                        if ($valida > 0) {
                                            $this->TarefasFactory->insertChkCriterio(array(
                                                "NRCRITERIOTARGERCHK"       => $this->newCode->getCodeByTableName("FCM_CRITERIOTARGERCHK", $NRORG, 1, 10, true),
                                                "NMCRITERIOTARGERCHK"       => $criterios['DSCRITERIOTAREFA'],
                                                "NRTAREFAGERCHK"            => $nrtarefaagendchk,
                                                "NRCRITERIOTAREFACHK"       => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                                                "OPERADOR"                  => $operadorLogado
                                            ));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    // Função auxiliar para tratar o campo NRESCOPOSTAREFA
    function tratarNRESCOPOSTAREFA($string) {
        // Verificar se o campo está vazio
        if (empty($string)) {
            return [];
        }
        
        // Remover espaços em branco e dividir a string em um array
        $array = explode(", ", $string);

        // Converter os elementos para inteiros
        return array_map('intval', $array);
    }

    public function inativarTarefas($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $param = implode(',', $parametros["NRS"]);
            $this->TarefasFactory->deletarTarefas($param);
            $this->TarefasFactory->deletarEpis($param);
            $this->TarefasFactory->deletarModpop($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTarefasSelectAll()
    {
        return $this->TarefasFactory->getTarefasSelectAll();
    }

    public function getHistoricoTarefa($parametros)
    {
        return $this->TarefasFactory->getHistoricoTarefa(array(
            "NRCHECKLISTAGEND" => $parametros['NRCHECKLISTAGEND'],
        ));
    }

    public function calcularDuracao($dataInicial, $horaInicial, $duracao)
    {
        // Converter a duração para minutos
        $hora = intval(substr($duracao, 0, 2));  // extrai horas da duração
        $minuto = intval(substr($duracao, 3, 2)); // extrai minutos da duração
        $totalMinutos = ($hora * 60) + $minuto;
    
        // Concatenar a data e a hora fornecida
        $dataHoraAgendada = $dataInicial . ' ' . $horaInicial;
    
        // Criar o objeto DateTime a partir da data e hora completa
        $horarioInicial = \DateTime::createFromFormat('d/m/Y H:i', $dataHoraAgendada);
        if (!$horarioInicial) {
            throw new \Exception("Erro ao converter a data e hora: $dataHoraAgendada");
        }
    
        // Clonar o objeto inicial para manipulação da hora final
        $horarioFinal = clone ($horarioInicial);
    
        // Adicionar a duração em minutos
        $horarioFinal->add(new \DateInterval('PT' . $totalMinutos . 'M'));
    
        // Retornar o horário inicial e final no formato desejado
        return [$horarioInicial->format('d/m/Y H:i'), $horarioFinal->format('d/m/Y H:i')];
    }
    
}
