<?php

namespace FM\Service;

use FM\Factory\FmModeloPopFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;
use FM\Service\FileServerService;
use Exception;

class FmModeloPopService
{
    protected $FmModeloPopFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct(
        FmModeloPopFactory $FmModeloPopFactory,
        Environment $Environment,
        NewCode $newCode,
        EntityManager $entityManager,
        FindService $FindService,
        FileServerService $FileServerService,
        $fileServerUrl
    ) {
        $this->FmModeloPopFactory = $FmModeloPopFactory;
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
        $this->FindService = $FindService;
        $this->FileServerService = $FileServerService;
        $this->fileServerUrl = $fileServerUrl;
    }

    public function getModeloPop()
    {
        return $this->FmModeloPopFactory->getModeloPop();
    }
   
    public function salvarModeloPop($params) 
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            if (!isset($params["NRMODELOPOP"]) || empty($params["NRMODELOPOP"])) {
                $NRMODELOPOP = $this->newCode->getCodeByTableName("FCM_MODELOPOP", $nrorg, 1, 10, true);
                $paramModeloPop = array(
                    "NRMODELOPOP" => $NRMODELOPOP,
                    "NMMODELOPOP" => $params["NMMODELOPOP"],
                    "LINKMODELOPOP" => $params["LINKMODELOPOP"],
                    "DESCMODELOPOP" => $params["DESCMODELOPOP"],
                    "IDATIVO" => isset($params["IDATIVO"]) ? $params["IDATIVO"] : 'S',
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador
                );
                $this->FmModeloPopFactory->inserirModeloPop($paramModeloPop);
                $this->salvarAnexoModeloPop($params, $paramModeloPop, true);
            } else if($params["IDATIVO"] === 'S'){
                $paramModeloPop = array(
                    "NRMODELOPOP" => $params["NRMODELOPOP"],
                    "NMMODELOPOP" => $params["NMMODELOPOP"],
                    "LINKMODELOPOP" => $params["LINKMODELOPOP"],
                    "DESCMODELOPOP" => $params["DESCMODELOPOP"],
                    "IDATIVO" => isset($params["IDATIVO"]) ? $params["IDATIVO"] : null,
                    "CDOPERULTATU" => $operador,
                    "NRORGULTATU" => $nrorg
                );
                $this->FmModeloPopFactory->atualizarModeloPop($paramModeloPop);
                $this->salvarAnexoModeloPop($params, $paramModeloPop, false);
            } else {
                $param = ["NRMODELOPOP" => $params["NRMODELOPOP"]];
                $tarefasAssociadas = $this->FmModeloPopFactory->verificaRelacionamentoPop($param);

                $tarefasFiltradas = array_values(array_filter($tarefasAssociadas, function ($item) {
                    return !in_array($item['IDSITUACAOAGEND'], ['3', '4']);
                }));

                if(empty($tarefasFiltradas)) {
                    $paramModeloPop = array(
                        "NRMODELOPOP" => $params["NRMODELOPOP"],
                        "NMMODELOPOP" => $params["NMMODELOPOP"],
                        "LINKMODELOPOP" => $params["LINKMODELOPOP"],
                        "DESCMODELOPOP" => $params["DESCMODELOPOP"],
                        "IDATIVO" => isset($params["IDATIVO"]) ? $params["IDATIVO"] : null,
                        "CDOPERULTATU" => $operador,
                        "NRORGULTATU" => $nrorg
                    );
                    $this->salvarAnexoModeloPop($params, $paramModeloPop, false);
                    $this->FmModeloPopFactory->atualizarModeloPop($paramModeloPop);
                } else {
                    return $tarefasFiltradas;
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function salvarAnexoModeloPop($params, $paramModeloPop, $inserir) {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
        
            if ($params['ARQUIVO'] !== []) {
                $file = $this->FileServerService->uploadFiles($params["ARQUIVO"], 'anexo_modelo_documento_cda');
                $file[0]["size"] = $file[0]["size"] / 1024;
            } else {
                $file = [[
                    "path" => isset($params["DSCAMINHOARQ"]) ? $params["DSCAMINHOARQ"] : '',
                    "size" => isset($params["VRTAMANHO"]) ? $params["VRTAMANHO"] : 0,
                    "name" => isset($params["NMARQUIVO"]) ? $params["NMARQUIVO"] : '',
                    "type" => isset($params["DSTIPO"]) ? $params["DSTIPO"] : '',
                ]];
            }
            if ($inserir) {
                $NRANEXOMODELOPOP = $this->newCode->getCodeByTableName("FCM_ANEXOMODELOPOP", $nrorg, 1, 10, true);
                $paramsAnexo = array(
                    "NRANEXOMODELOPOP" => $NRANEXOMODELOPOP,
                    "NRMODELOPOP" => $paramModeloPop["NRMODELOPOP"],
                    "NMARQUIVO" => $file[0]["name"],
                    "DSCAMINHOARQ" => $this->fileServerUrl . "/files/" . $file[0]["path"],
                    "DSTIPO" => $file[0]["type"],
                    "VRTAMANHO" => $file[0]["size"],
                    "IDATIVO" => 'S',
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador
                );
                $this->FmModeloPopFactory->inserirAnexoModeloPop($paramsAnexo);
            } else {
                $paramsAnexo = array(
                    "NRANEXOMODELOPOP" => $params["NRANEXOMODELOPOP"],
                    "NRMODELOPOP" => $params["NRMODELOPOP"],
                    "NMARQUIVO" => $file[0]["name"],
                    "IDATIVO" => 'S',
                    "DSCAMINHOARQ" => $this->fileServerUrl . "/files/" . $file[0]["path"],
                    "DSTIPO" => $file[0]["type"],
                    "VRTAMANHO" => $file[0]["size"],
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador
                );
                $this->FmModeloPopFactory->atualizarAnexoModeloPop($paramsAnexo);
            }
                
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
        
    public function excluirModeloPop($parametros) 
    {
        $this->connection->beginTransaction();
    
        try {
            $mensagem = ['status' => 'Ok', 'tarefas' => []];
    
            foreach ($parametros['ROW'] as $nr) {
                $param = ["NRMODELOPOP" => $nr];
    
                $relacionamentos = $this->FmModeloPopFactory->verificaRelacionamentoPop($param);
                if (count($relacionamentos) == 0) {
                    
                    $anexos = $this->FmModeloPopFactory->buscarAnexosPorModelo($param);
                    if (!empty($anexos)) {
                        $this->FmModeloPopFactory->deletarAnexoPop($anexos);
                    }
    
                    $this->FmModeloPopFactory->excluirModeloPop($param);
                } else {
                    $mensagem['status'] = 'erro';
                    if (!empty($relacionamentos)) {
                        $mensagem['tarefas'] = array_merge($mensagem['tarefas'], $relacionamentos);
                    }
                }
            }
    
            $this->connection->commit();
            return $mensagem;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    
}
