<?php

namespace FM\Service;

use FM\Factory\FmRelatorioAuditoriaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;
use FM\Service\FileServerService;
use FM\Service\FuncionariosHierarqService;

class RelatorioAuditoriaService
{

    protected $FmRelatorioAuditoriaFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;
    protected $FileServerService;
    protected $fileServerUrl;
    protected $FuncionariosHierarqService;

    public function __construct(FmRelatorioAuditoriaFactory $FmRelatorioAuditoriaFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FindService $FindService, FileServerService $FileServerService, $fileServerUrl, FuncionariosHierarqService $FuncionariosHierarqService)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->FmRelatorioAuditoriaFactory  = $FmRelatorioAuditoriaFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->FindService                  = $FindService;
        $this->FileServerService            = $FileServerService;
        $this->fileServerUrl                = $fileServerUrl;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
    }

    public function getTarefasAuditoria()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->FmRelatorioAuditoriaFactory->getTarefasAuditoria($listaOperadores, $params);
    }

    public function getCriteriosAuditoria()
    {
        return $this->FmRelatorioAuditoriaFactory->getCriteriosAuditoria();
    }

    public function getImagensAuditoria($parametros)
    {
        return $this->FmRelatorioAuditoriaFactory->getImagensAuditoria($parametros);
    }


    public function getCriteriosAvaliacao()
    {
        $parametrizacao = $this->getParametrizacao();
        $criterios = $this->FmRelatorioAuditoriaFactory->getCriteriosAvaliacao();

        if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'N') {
            foreach($criterios['data'] as &$criterio) {
                $criterio['IDSOMANOTA'] = 'N';
            }
        }

        if ($parametrizacao[0]['IDUTILIZAPERCENTUALAUDIT'] === 'N') {
            foreach($criterios['data'] as &$criterio) {
                $criterio['IDSOMAPERCENTUAL'] = 'N';
            }
        }
        return $criterios;              
    }

    public function getCategoriasAuditoriaRelatorio($parametros)
    {
        try {
            $categorias = $this->FmRelatorioAuditoriaFactory->getCategoriasAuditoriaRelatorio(array(
                "NRAUDITORIA"   => $parametros['NRAUDITORIA']
            ));

            $categoriasPercentual = $this->getPercentualCategoria($categorias, $parametros);

            return $this->getNotaCategoria($categoriasPercentual, $parametros);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTopicosAuditoriaExecRelatorio($parametros)
    {
        return $this->FmRelatorioAuditoriaFactory->getTopicosAuditoriaExecRelatorio(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT"   => $parametros['NRCATEGORIAAUDIT'],
        ));
    }
    

    public function getCriteriosAuditoriaExecRelatorio($parametros)
    {
        return $this->FmRelatorioAuditoriaFactory->getCriteriosAuditoriaExecRelatorio(array(
            "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
            "NRAUDITORIA"       => $parametros['NRAUDITORIA'],
            "NRTOPICOAUDIT"     => $parametros['NRTOPICOAUDIT'],
        ));
    }

    public function getImagensTopicosAuditoria($parametros)
    {
        return $this->FmRelatorioAuditoriaFactory->getImagensTopicosAuditoria(array(
            "NRAUDITORIA"        => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT"  => $parametros['NRCATEGORIAAUDIT'],
            "NRTOPICOAUDIT"     => $parametros['NRTOPICOAUDIT'],
        ));
    }

    
    public function calcularNota($params) {

        try {
            $parametrizacao = $this->getParametrizacao();

            if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'S') {
                // Pega todas as categorias
                $CriteriosSomaNota = $this->FmRelatorioAuditoriaFactory->getCriteriosAvaliacaoSomaNota();
                $categorias = $this->getCategoriasAuditoriaRelatorio($params);
                $resultados = []; 
                $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');
                

                foreach ($categorias['data'] as $categoria) {
                    $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'],
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                    ));

                    foreach ($topicos['data'] as $topico) {
                        $criterios = $this->FmRelatorioAuditoriaFactory->getCriteriosMarcadosAudit(array(
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                            'NRAUDITORIA'   => $params['NRAUDITORIA'],
                        ));
            

                        foreach ($criterios as $criterio) {
                            if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                                $resultados[] = array(
                                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                                    'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                                    'NRCRITERIOAUDIT' => $criterio['NRCRITERIOAUDIT'],
                                );
                            }
                        }
                    }
                }
                
                $notaTotal = 0;
                // Calcula a nota total dos tópicos que passaram na validação
                foreach ($resultados as $resultado) {
                    $notaTopico = $this->FmRelatorioAuditoriaFactory->getNotaTopico($resultado);
                    
                    if (!empty($notaTopico)) {
                        $notaTotal += $notaTopico[0]['NOTATOPICAUDIT']; // Soma as notas do tópico
                    }
                }
            
                $this->FmRelatorioAuditoriaFactory->salvarNota(array(
                    'NOTATOTAL' => $notaTotal,
                    'NRAUDITORIA'   => $params['NRAUDITORIA'],
                )); 
            }

            return round($notaTotal, 2);
        
        } catch (\Exception $e) {
            throw $e;
        }  
    }

    public function calcularPercentual($params) {

        try{
            $parametrizacao = $this->getParametrizacao();

            if($parametrizacao[0]["IDUTILIZAPERCENTUALAUDIT"] === 'S') {
                $CriteriosSomaPercentual = $this->FmRelatorioAuditoriaFactory->getCriteriosAvaliacaoSomaPercentual();
                $categorias = $this->getCategoriasAuditoriaRelatorio($params);
    
                $criteriosValidos = array_column($CriteriosSomaPercentual, 'NRCRITERIOAUDIT');
    
                $criteriosSomaPercentualTotal = 0;
                $criteriosSomaPercentualMarcados = 0;
    
                foreach($categorias['data'] as $categoria) {
                    $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'],
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                    ));
                    foreach($topicos['data'] as $topico) {
                        $criterios = $this->FmRelatorioAuditoriaFactory->getCriteriosAudit(array(
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                            'NRAUDITORIA'   => $params['NRAUDITORIA'],
                        ));
                        foreach($criterios as $criterio) {
                            if(in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
    
                                $criteriosSomaPercentualTotal++;
                                if($criterio['IDREALIZADO'] == 'S') {
                                    $criteriosSomaPercentualMarcados++;
                                }
                            }
                        }
                    }
                }
    
                if($criteriosSomaPercentualTotal == 0) {
                    $this->FmRelatorioAuditoriaFactory->salvarPercentual(array(
                        'PERCENTUALCONCLUSAO' => 0,
                        'NRAUDITORIA'   => $params['NRAUDITORIA'],
                    ));
                }else {
                    $percentualConclusao = ($criteriosSomaPercentualMarcados/$criteriosSomaPercentualTotal) * 100;
                    $percentualConclusao = round($percentualConclusao, 0);
                    $this->FmRelatorioAuditoriaFactory->salvarPercentual(array(
                        'PERCENTUALCONCLUSAO' => $percentualConclusao,
                        'NRAUDITORIA'   => $params['NRAUDITORIA'],
                    ));
                }
            } 

            return $percentualConclusao;

        }catch (\Exception $e) {
            throw $e;
        } 
    }

    public function getParametrizacao() 
    {
        try {
            return $this->FmRelatorioAuditoriaFactory->getParametrizacao();
        } catch (\Exception $e) {
            throw $e;
        } 
    }

    public function getNotaCategoria($categorias, $params)
    {
        try {
            $parametrizacao = $this->getParametrizacao();
    
            if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'S') {
                $CriteriosSomaNota = $this->FmRelatorioAuditoriaFactory->getCriteriosAvaliacaoSomaNota();
                $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');
    
                foreach ($categorias['data'] as &$categoria) {
                    $resultados = [];
                    
                    $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                    ));
    
                    // Validação do retorno de $topicos
                    if (!is_array($topicos) || !isset($topicos['data']) || !is_array($topicos['data'])) {
                        $topicos = ['data' => []];
                    }
    
                    foreach ($topicos['data'] as $topico) { // Linha 1044 ajustada
                        $arryCrit = array(
                            'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'] ?? null,
                            'NRAUDITORIA'   => $params['NRAUDITORIA'] ?? null,
                        );
                        
                        $criterios = $this->FmRelatorioAuditoriaFactory->getCriteriosMarcadosAudit($arryCrit);
                        
                        foreach ($criterios as $criterio) {
                            if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                                $resultados[] = array(
                                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                                    'NRTOPICOAUDIT'    => $topico['NRTOPICOAUDIT'],
                                    'NRCRITERIOAUDIT'  => $criterio['NRCRITERIOAUDIT'],
                                );
                            }
                        }
                    }
    
                    $notaTotalCategoria = 0;
    
                    foreach ($resultados as $resultado) {
                        $notaTopico = $this->FmRelatorioAuditoriaFactory->getNotaTopico($resultado);
    
                        if (!empty($notaTopico)) {
                            $notaTotalCategoria += $notaTopico[0]['NOTATOPICAUDIT'];
                        }
                    }
                    $categoria['NOTA'] = round($notaTotalCategoria, 2);
                }
    
                return $categorias;
            } else {
                foreach ($categorias['data'] as &$categoria) {
                    $categoria['NOTA'] = 'N/A';
                }
    
                return $categorias;
            }
        } catch (\Exception $e) {
            error_log("Erro em getNotaCategoria: " . $e->getMessage());
            throw $e;
        }
    }

    public function getPercentualCategoria($categorias, $params)
    {
        try {
            $parametrizacao = $this->getParametrizacao();
    
            if ($parametrizacao[0]["IDUTILIZAPERCENTUALAUDIT"] !== 'S') {
                foreach ($categorias['data'] as &$categoria) {
                    $categoria['PERCENTUAL'] = 'N/A';
                }
    
                return $categorias;
            }
    
            $CriteriosSomaPercentual = $this->FmRelatorioAuditoriaFactory->getCriteriosAvaliacaoSomaPercentual();
            $criteriosValidos = array_column($CriteriosSomaPercentual, 'NRCRITERIOAUDIT');
            $percentuaisPorCategoria = [];
    
            foreach ($categorias['data'] as &$categoria) {
                $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                    'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                ));
    
                if (!is_array($topicos) || !isset($topicos['data']) || !is_array($topicos['data'])) {
                    $topicos = ['data' => []];
                }
    
                $criteriosSomaPercentualTotal = 0;
                $criteriosSomaPercentualMarcados = 0;
    
                foreach ($topicos['data'] as $topico) {
                    $criterios = $this->FmRelatorioAuditoriaFactory->getCriteriosAudit(array(
                        'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'] ?? null,
                        'NRAUDITORIA'   => $params['NRAUDITORIA'] ?? null,
                    ));
    
                    foreach ($criterios as $criterio) {
                        if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                            $criteriosSomaPercentualTotal++;
                            if ($criterio['IDREALIZADO'] == 'S') {
                                $criteriosSomaPercentualMarcados++;
                            }
                        }
                    }
                }
    
                if ($criteriosSomaPercentualTotal > 0) {
                    $percentualConclusao = ($criteriosSomaPercentualMarcados / $criteriosSomaPercentualTotal) * 100;
                    $percentuaisPorCategoria = round($percentualConclusao, 0);
                } else {
                    $percentuaisPorCategoria = 0;
                }
    
                $categoria['PERCENTUAL'] = $percentuaisPorCategoria;
            }
    
            return $categorias;
        } catch (\Exception $e) {
            error_log("Erro em getPercentualCategoria: " . $e->getMessage());
            throw $e;
        }
    }    

    public function getNotaPorCategoria($params) {
        try {
            $CriteriosSomaNota = $this->FmRelatorioAuditoriaFactory->getCriteriosAvaliacaoSomaNota();

            $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');
    
            $topicos = $this->getTopicosAuditoriaExecRelatorio(array(
                'NRAUDITORIA' => $params['NRAUDITORIA'],
                'NRCATEGORIAAUDIT' => $params['NRCATEGORIAAUDIT'],
            ));
    
            foreach ($topicos['data'] as $topico) {
                $criterios = $this->FmRelatorioAuditoriaFactory->getCriteriosMarcadosAudit(array(
                    'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                    'NRAUDITORIA'   => $params['NRAUDITORIA'],
                ));
    
    
                foreach ($criterios as $criterio) {
                    if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                        $resultados[] = array(
                            'NRCATEGORIAAUDIT' => $params['NRCATEGORIAAUDIT'],
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                            'NRCRITERIOAUDIT' => $criterio['NRCRITERIOAUDIT'],
                        );
                    }
                }
            }
            $notaTotalCategoria = 0;
    
            foreach ($resultados as $resultado) {
                $notaTopico = $this->FmRelatorioAuditoriaFactory->getNotaTopico($resultado);
                
                if (!empty($notaTopico)) {
                    $notaTotalCategoria += $notaTopico[0]['NOTATOPICAUDIT']; 
                }
            }  
            
            return round($notaTotalCategoria, 2);
        }catch (\Exception $e) {
            throw $e;
        }
    }
}
