<?php

namespace FM\Service;

use FM\Factory\RelatoriosVistoriaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use FM\Service\FuncionariosHierarqService;

class RelatoriosVistoriaService
{

    protected $RelatoriosVistoriaFactory;
    protected $FuncionariosHierarqService;
    protected $Environment;
    protected $newCode;

    public function __construct(RelatoriosVistoriaFactory $RelatoriosVistoriaFactory, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode)
    {
        $this->RelatoriosVistoriaFactory    = $RelatoriosVistoriaFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function getRelatoriosVistoria()
    {

        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        $params = array(
            "CDOPERADOR" => $cdoperador,
        );

        return $this->RelatoriosVistoriaFactory->getRelatoriosVistoria($listaOperadores, $params);
    }

    public function getCriteriosVistoria($parametros)
    {
        return $this->RelatoriosVistoriaFactory->getCriteriosVistoria($parametros['NRGERENCIAMENTOCHK']);
    }

    public function getFotosVistoria($parametros)
{
    // Obtenha as fotos usando a função da fábrica
    $fotos = $this->RelatoriosVistoriaFactory->getFotosVistoria($parametros['NRGERENCIAMENTOCHK']);
    
    // Verifique se o resultado está vazio e retorne uma resposta vazia ou padrão
    if (empty($fotos)) {
        return [
            'data' => [],  // Retorna um array vazio, significando que não há fotos
            'message' => 'Nenhuma foto encontrada'  // Mensagem indicando que não há fotos
        ];
    }
    
    // Se houver fotos, retorne os dados normalmente
    return [
        'data' => $fotos,  // Retorna as fotos obtidas da consulta
        'message' => 'Fotos carregadas com sucesso'
    ];
}

    public function getFiltroVistoriaLocal()
    {
        $listaLocais = $this->FuncionariosHierarqService->getLocaisHierarquia($this->Environment->getCdOperador());

        return $this->RelatoriosVistoriaFactory->getFiltroVistoriaLocal($listaLocais);
    }

    public function getFiltroVistoriaChecklist()
    {
        return $this->RelatoriosVistoriaFactory->getFiltroVistoriaChecklist();
    }

    public function getFiltroVistoriaCliente()
    {
        return $this->RelatoriosVistoriaFactory->getFiltroVistoriaCliente();
    }

    public function getFiltroVistoriaSituacao()
    {
        return $this->RelatoriosVistoriaFactory->getFiltroVistoriaSituacao();
    }
}
