import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    LoadingService,
    Modal,
    ModalService,
    TextInput,
} from '@zeedhi/common';
import {
Http, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class EPIsController {
    public modalErroExcluir!: Modal;

    public modalErroInatividade!: Modal;

    public async criarModais() {
        this.modalErroExcluir = await ModalService.createFromJson('fm_EPIs/modal/fm_modal_exclusao', true);
        this.modalErroInatividade = await ModalService.createFromJson('fm_EPIs/modal/fm_modal_inatividade', true);
    }

    public async abrirModalErro() {
        this.modalErroExcluir.show();
    }

    public async abrirModalErroInatividade() {
        this.modalErroInatividade.show();
    }

    public fecharModalErro() {
        this.modalErroExcluir.hide();
    }

    public async destruirModais() {
        this.modalErroExcluir.destroy();
        this.modalErroInatividade.destroy();
    }

    public fecharModalInatividade() {
        this.modalErroInatividade.hide();
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NREPI: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMEPI_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public salvarEPI({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/salvarEPI', {
                ROW: component.getEditedRows(),
            }).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data);
                } else {
                    Utilities.showSuccessMessage();
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'EPIsController.excluirEPI', 'Excluir EPI', 'Deseja realmente excluir os registros selecionados?');
    }

    public excluirEPI() {
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_gridEpis');

        Utilities.deleteInfoGrid('/excluirEPI', { component: grid }).then((response: any) => {
            DialogService.hide();
            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.tarefas);
            }
        }).catch(() => {
            Utilities.showErrorMessage('Ocorreu um erro ao tentar excluir os EPIs.');
        }).finally(() => {
            grid.selectedRows = [];
            grid.datasource.get();
            LoadingService.hide();
        });
    }

    public async montarModalErro(tarefas: any) {
        await this.abrirModalErro();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NMFANTCLIE: element.NMFANTCLIE,
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                IDSITUACAOAGEND: element.IDSITUACAOAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
