import { Utilities } from '@/util/Utilities';
import {
    Modal, ModalService, Select,
} from '@zeedhi/common';
import {
    IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { ITekGridColumn, TekGrid } from '@zeedhi/teknisa-components-common';

export class DashboardTarefasController {
    private modalConflito!: Modal;

    public currentRow: any;

    public async criarModais() {
        this.modalConflito = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00000_000_dash_tarefas/modal/fm_modal_conflito_transferencia' });
    }

    public destruirModais() {
        this.modalConflito?.destroy();
    }

    public fecharModalConflito() {
        this.modalConflito.hide();
    }

    public disableSelection({ row }: IDictionary) {
        const select = Metadata.getInstance<Select>('operador-origem');
        return row.CDOPERADOR !== select.value;
    }

    public changeOperadorDestino({ component }: IEventParam<Select>) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        Metadata.getInstance<TekGrid>('grid-tarefas-transferir').selectedRows = [];

        if (component.value) {
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: component.selectValue.NRHABILIDADE,
                CDOPERADOR: component.value,
            });
        }
    }

    public focusOperadores({ component, row }: any) {
        component.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
            NRHABILIDADE: row.NRHABILIDADE,
        });
    }

    public editedRows: any = '';

    public validarGrid({ component }: IEventParam<TekGrid>) {
        if (Utilities.validateGrid(component)) {
            Utilities.showSuccessMessage();
            this.editedRows = component.getEditedRows();
        }
    }

    public setFiltro({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
        this.changeMaxWidth({ component });
    }

    public disabledAction: boolean = false;

    public rowStyleConditions(row: any) {
        switch (row.IDSITUACAOAGEND) {
            case '2':
                return { cssClass: 'linha-grid-andamento' };
            case '3':
                return { cssClass: 'linha-grid-concluida' };
            default:
                if (row.ATRASO !== null && row.ATRASO.split(' ')[1] === 'dias' && row.ATRASO.split(' ')[0] >= 1) {
                    return { cssClass: 'linha-grid-atraso' };
                }
            return { cssClass: '' };
        }
    }

    private clicks: any = 0;

    // função que simula um double click
    public rowClick({ row }: any) {
        this.currentRow = row;
        this.clicks += 1;
        if (this.clicks >= 2) {
            // this.abrirModal();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        }, 500);
    }

    public canEditRow({ row }: IEventParam<TekGrid>) {
        return (row.TAREFABANCO !== 'N');
    }

    public disabledSelectionGrid({ row }: any) {
        return (row.IDSITUACAOAGEND === '4' || row.IDSITUACAOAGEND === '3');
    }

    public visibleActionGrid({ row }: any) {
        return !(row.IDSITUACAOAGEND === '4' || row.IDSITUACAOAGEND === '3');
    }

    public changeMaxWidth({ component }: IEventParam<TekGrid>) {
        /* eslint-disable */
        const coluna: ITekGridColumn | undefined = component.columns.find((item: any) => { return item.name === 'NMTAREFAS' });
        /* eslint-enable */

        if (coluna) {
            coluna.maxWidth = '250px';
            coluna.overflow = 'wrap';
        }
    }
}
