/**
 * Nome: Heitor Medeiros
 * Data: 15/05/2023
 * Descrição: Tela responsável por criar Habilidade para os operadores. Posteriormente essas habilidaes serão
 * vinculadas às atividades. A Atividade só poderá ser realizada pelo operador que tiver a habilidade necessária.
 * Autor: Heitor Medeiros
 */
import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, Tabs, TextInput, Modal,
    ModalService,
} from '@zeedhi/common';
import {
    Http, IEventParam, MemoryDatasource, Metadata,

} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00000000HabilidadesController {
    public modalErroExcluir!: Modal;

    public modalErroInativar!: Modal;

    public async criarModais() {
        this.modalErroExcluir = await ModalService.createFromJson('fm_00000_000_habilidades/modal/fm_0000_000_modal_exclusao', true);
        this.modalErroInativar = await ModalService.createFromJson('fm_00000_000_habilidades/modal/fm_0000_000_modal_inatividade', true);
    }

    public async abrirModalErro() {
        this.modalErroExcluir.show();
    }

    public async abrirModalErroInatividade() {
        this.modalErroInativar.show();
    }

    public fecharModalErro() {
        this.modalErroExcluir.hide();
    }

    public fecharModalInatividade() {
        this.modalErroInativar.hide();
    }

    public async destruirModais() {
        this.modalErroExcluir.destroy();
        this.modalErroInativar.destroy();
    }

    // FUNÇÕES FRAME GRID HABILIDADES
    // FUNÇÕES FRAME GRID HABILIDADES
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        // Remova o IDATIVO: 'S' para não definir um valor padrão
        component.addNewRow({ NRHABILIDADE: newId }, 'start');

        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMHABILIDADE_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public salvarHabilidades({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/salvarHabilidades', {
                ROW: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00000000HabilidadesController.excluirHabilidades', 'Excluir', 'Deseja realmente excluir os registros selecionados?');
    }

    public excluirHabilidades({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('gridHabilidades');
        LoadingService.show();

        const habilidadesSelecionadas = grid.selectedRows.map((element: any) => ({
            NRHABILIDADE: element.NRHABILIDADE,
        }));

        Http.post('/excluirHabilidades', habilidadesSelecionadas).then((response: any) => {
            DialogService.hide();
            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.habilidades);
            } else {
                Utilities.showSuccessMessage();
                grid.datasource.get();
            }
        }).catch(() => {
            Utilities.showErrorMessage('Ocorreu um erro ao tentar excluir as habilidades.');
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
        });
    }

    // FUNÇÕES TAB OPERADORES
    public addNewRowOperador({ component }: IEventParam<TekGrid>) {
        this.selectEditavel = true;
        const newId = 0;
        component.addNewRow({ NRRELACHABOPER: newId }, 'start');
    }

    public salvarOperadorHabilidades({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const newRow = component.getEditedRows();
            Http.post('/relacionarOperadorHabilidade', {
                OPERADOR: newRow[0].CDOPERADOR,
                NRHABILIDADE: this.currentRow.NRHABILIDADE,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.selectEditavel = false;
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public filtroOperadorHabilidade() {
        const gridOperador = Metadata.getInstance<TekGrid>('gridOperador');
        gridOperador.datasource.setFilter({
            HABILIDADE: this.currentRow.NRHABILIDADE,
        });
    }

    public excluirOperadorHabilidades({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('gridOperador');
        LoadingService.show();
        Http.post('/excluirOperadorHabilidades', {
            ROW: grid.selectedRows.map((element: any) => element.NRRELACHABOPER),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    // FUNÇÕES TAB ATIVIDADES
    public addNewRowAtividade({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRRELACHABATIV: newId }, 'start');
    }

    public filtroAtividadeHabilidade() {
        const gridAtividade = Metadata.getInstance<TekGrid>('gridAtividades');
        gridAtividade.datasource.setFilter({
            HABILIDADE: this.currentRow.NRHABILIDADE,
        });
    }

    public salvarAtividadeHabilidades({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const newRow = component.getEditedRows();
            Http.post('/relacionarAtividadeHabilidade', {
                NRTAREFA: newRow[0].NMTAREFA,
                NRHABILIDADE: this.currentRow.NRHABILIDADE,
            }).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response);
                } else {
                    Utilities.showSuccessMessage();
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirAtividadeHabilidades({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('gridAtividades');
        LoadingService.show();
        Http.post('/excluirAtividadeHabilidades', {
            ROW: grid.selectedRows.map((element: any) => element.NRRELACHABATIV),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    // CONTROLE DAS TELAS
    private clicks: any = 0;

    private currentRow: any;

    public visibilidadeFrameGridPincipal: Boolean = true;

    public visibilidadeFrameOperadoresAtividades: Boolean = false;

    public selectEditavel: Boolean = false;

    public tituloHeaderFrameOperadoresAtividades: String = '';

    public recarregarTabs() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabsHabilidades');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabAtividades') {
            setTimeout(() => {
                this.filtroAtividadeHabilidade();
            }, 1500);
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tabOperadores') {
            setTimeout(() => {
                this.filtroOperadorHabilidade();
            }, 1500);
        }
    }

    public rowClick({ row }: any) {
        this.clicks += 1;
        setTimeout(() => {
            if (this.clicks === 1) {
                this.currentRow = row;
                this.visibilidadeFrameOperadoresAtividades = true;
                this.visibilidadeFrameGridPincipal = false;
                this.tituloHeaderFrameOperadoresAtividades = row.NMHABILIDADE;
                this.recarregarTabs();
            }
            this.clicks = 0;
        }, 1000);
    }

    public voltarGridHabilidades() {
        this.visibilidadeFrameOperadoresAtividades = false;
        this.visibilidadeFrameGridPincipal = true;
    }

    public async montarModalErro(tarefas: any) {
        await this.abrirModalErro();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-habilidades');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRTAREFA: element.NRTAREFA,
                NMTAREFA: element.NMTAREFA,
                DTINCLUSAO: element.DTINCLUSAO,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.data.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NMFANTCLIE: element.NMFANTCLIE,
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                IDSITUACAOAGEND: element.IDSITUACAOAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
