import { Utilities } from '@/util/Utilities';
import {
    DialogService, GridEditable, LoadingService, Modal, ModalService,
} from '@zeedhi/common';
import {
    Http, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm0000000EquipesController {
    public visibilidadeFrameOperadores: boolean = false;

    public visibilidadeFrameEquipes: boolean = true;

    public currentRow: any;

    public tituloHeaderFrameOperadoresEquipes: String = '';

    public selectEditavel: boolean = false;

    public modalErroExcluirEquipe!: Modal;

    public modalErroInativarEquipe!: Modal;

    public async criarModais() {
        this.modalErroExcluirEquipe = await ModalService.createFromJson('fm_0000_000_equipes/modal/fm_0000_000_modal_exclusao', true);
        this.modalErroInativarEquipe = await ModalService.createFromJson('fm_0000_000_equipes/modal/fm_0000_000_modal_inatividade', true);
    }

    public async abrirModalErro() {
        this.modalErroExcluirEquipe.show();
    }

    public async abrirModalErroInatividade() {
        this.modalErroInativarEquipe.show();
    }

    public async destruirModais() {
        this.modalErroExcluirEquipe.destroy();
        this.modalErroInativarEquipe.destroy();
    }

    public fecharModalErro() {
        this.modalErroExcluirEquipe.hide();
    }

    public fecharModalInatividade() {
        this.modalErroInativarEquipe.hide();
    }

    public rowClick({ row, component }: any) {
        if (component.editing === false) {
            this.currentRow = row;
            this.tituloHeaderFrameOperadoresEquipes = row.NMEQUIPE;
            this.visibilidadeFrameEquipes = false;
            this.visibilidadeFrameOperadores = true;
            this.setFilterOperadorEquipe();
        }
    }

    public voltarGridEquipes() {
        this.visibilidadeFrameOperadores = false;
        this.visibilidadeFrameEquipes = true;
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        const newRow = {
            NREQUIPE: newId,
            NMEQUIPE: '',
        };
        component.addNewRow(newRow, 'start');
    }

    public async saveEditingAddNewRow({ component, event }: IEventParam<GridEditable>) {
        event?.preventDefault();
        await component.saveEditedRows();
        component.editingNewRows = false;
    }

    public salvarEquipes({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/salvarEquipes', {
                ROW: component.getEditedRows(),
            }).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response);
                } else {
                    Utilities.showSuccessMessage();
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirEquipes() {
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-equipes');
        DialogService.hide();
        LoadingService.show();
        Http.post('/excluirEquipes', {
            ROW: grid.selectedRows.map((element: any) => element.NREQUIPE),
        }).then((response) => {
            if (response.data !== 'ok') {
                this.montarModalErro(response.data);
                return;
            }
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
        });
    }

    public setFilterOperadorEquipe() {
        const gridOperador = Metadata.getInstance<TekGrid>('grid-operadores');
        gridOperador.datasource.setFilter({
            EQUIPE: this.currentRow.NREQUIPE,
        });
    }

    public addNewRowOperador({ component }: IEventParam<TekGrid>) {
        this.selectEditavel = true;
        const newId = 0;
        component.addNewRow({ NRRELACEQUIPEOPER: newId }, 'start');
    }

    public salvarOperadorEquipe({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const newRow = component.getEditedRows();
            Http.post('/relacionarOperadorEquipe', {
                OPERADOR: newRow[0].CDOPERADOR,
                NREQUIPE: this.currentRow.NREQUIPE,
                IDATIVO: this.currentRow.IDATIVO,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.selectEditavel = false;
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public excluirOperadorEquipeDialog() {
        DialogService.show({
            name: 'apagar-operador-dialog',
            title: 'Excluir o operador',
            type: 'warning',
            text: 'Deseja Realmente excluir o operador dessa equipe?',
            buttons: [
                {
                    name: 'no', component: 'ZdButton', label: 'Nao', events: { click: () => DialogService.hide() }, outline: true,
                },
                {
                    name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => this.excluirOperadorEquipe() },
                },
            ],
        });
    }

    public excluirEquipeDialog({ event }: any) {
        event.preventDefault();
        DialogService.show({
            name: 'apagar-equipe-dialog',
            title: 'Excluir Equipe',
            type: 'warning',
            text: 'Deseja Realmente excluir a Equipe?',
            buttons: [
                {
                    name: 'no', component: 'ZdButton', label: 'Nao', events: { click: () => DialogService.hide() }, outline: true,
                },
                {
                    name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => this.excluirEquipes() },
                },
            ],
        });
    }

    public excluirOperadorEquipe() {
        const grid = Metadata.getInstance<TekGrid>('grid-operadores');
        LoadingService.show();
        DialogService.hide();
        Http.post('/excluirOperadorEquipe', {
            ROW: grid.selectedRows.map((element: any) => element.NRRELACEQUIPEOPER),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
        });
    }

    public async montarModalErro(tarefas: any) {
        await this.abrirModalErro();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.data.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.data.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NMFANTCLIE: element.NMFANTCLIE,
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                IDSITUACAOAGEND: element.IDSITUACAOAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
