import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    Form, LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { Http, MemoryDatasource, Metadata } from '@zeedhi/core';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';

export class Fm00003003AtividadeCriterioController {
    private modalAdicionarAtvCriterio!: Modal;

    private currentRow: any = '';

    public tituloModal: String = 'Adicionar Atividade ou Critério';

    private clicks = 0;

    private parametrizacao: String = '';

    public modalErroInatividade!: Modal;

    public async criarModais() {
        this.modalErroInatividade = await ModalService.createFromJson('fm_00003_000_checklist/modal/fm_00003_006_modal_inatividade_atividadescrit.json', true);
    }

    public async abrirModalErroInatividade() {
        this.modalErroInatividade.show();
    }

    public fecharModalInatividade() {
        this.modalErroInatividade.hide();
    }

    public destruirModais() {
        this.modalErroInatividade.destroy();
    }

    // função que simula um double click
    public rowClick({ row }: any) {
        this.clicks += 1;
        this.currentRow = row;

        if (this.clicks >= 2) {
            this.abrirModalEdicao();
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        },
            500);
    }

    public async abrirModal() {
        this.modalAdicionarAtvCriterio = await ModalService.createFromJsonPost('', { jsonPath: '/fm_00003_000_checklist/modal/fm_00003_003_salvar_atividade_criterio' });
        await this.modalAdicionarAtvCriterio.show();
    }

    public async abrirModalInserir() {
        await this.abrirModal();
        this.tituloModal = 'Adicionar Atividade ou Critério';

        if (this.currentRow !== '') {
            if (this.currentRow.TIPO === 'Atividade') {
                Metadata.getInstance<Select>('TIPOADICAO').value = 'Critério';
                Metadata.getInstance<Select>('ATIVIDADES').value = this.currentRow.NR;
            }
        }
    }

    public async getParametrizacoes() {
        Http.get('/getParametrizacoes').then((response) => {
            if (response.data.length > 0) {
                this.parametrizacao = response.data[0].IDUTILIZAHABILIDADE;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public async abrirModalEdicao() {
        await this.abrirModal();
        const nr = Metadata.getInstance<TextInput>('NRROW');
        const tipoAdicao = Metadata.getInstance<Select>('TIPOADICAO');
        this.tituloModal = 'Editar Atividade ou Critério';

        if (this.currentRow !== '') {
            nr.value = this.currentRow.CODIGO;
            tipoAdicao.value = this.currentRow.TIPO;
            tipoAdicao.disabled = true;
            Metadata.getInstance<TextInput>('NOME').value = this.currentRow.NOME;
            Metadata.getInstance<Select>('PRIORIDADE').value = this.currentRow.PRIORIDADE;
            /* eslint-disable */
            Metadata.getInstance<Select>('ATIVIDADES').value = parseInt(this.currentRow.CODIGOPAI.split('T')[1]);
            /* eslint-enable */
            Metadata.getInstance<Select>('IDATIVO').value = this.currentRow.IDATIVO;

            const compHabilidade = Metadata.getInstance<SelectMultiple>('HABILIDADES');
            let habilidades: number[] = [];
            if (typeof this.currentRow.HABILIDADE === 'string') {
                habilidades = this.currentRow.HABILIDADE
                    .trim()
                    .split(',')
                    .filter((part: string) => part.trim() !== '')
                    .map((part: string) => parseInt(part.trim(), 10));
            } else if (Array.isArray(this.currentRow.HABILIDADE)) {
                habilidades = this.currentRow.HABILIDADE.map((value: any) => parseInt(value, 10));
            }

            setTimeout(() => { compHabilidade.setValue(habilidades); }, 200);

            if (this.parametrizacao === 'S') {
                compHabilidade.disabled = false;
            } else {
                compHabilidade.disabled = true;
                compHabilidade.clearValidations();
            }
        }
    }

    public fecharModal() {
        this.modalAdicionarAtvCriterio.hide();
        this.modalAdicionarAtvCriterio?.destroy();
    }

    public habilitarCampoAtividade({ component }: any) {
        const campoAtividade = Metadata.getInstance<Select>('ATIVIDADES');
        const campoPrioridade = Metadata.getInstance<Select>('PRIORIDADE');
        const campoHabilidade = Metadata.getInstance<SelectMultiple>('HABILIDADES');

        if (component.value === 'Critério') {
            campoAtividade.disabled = true;
            campoHabilidade.isVisible = false;
            campoAtividade.addValidation('required', {});
            campoPrioridade.isVisible = false;
            campoHabilidade.clearValidations();
        } else {
            campoAtividade.isVisible = false;
            campoHabilidade.isVisible = true;
            campoAtividade.clearValidations();
            campoPrioridade.isVisible = true;
            campoHabilidade.addValidation('required', {});
        }
    }

    public salvarAtividadeCriterio() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-atividades-criterios');
        const form = Metadata.getInstance<Form>('form-ac');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/salvarAtividadeCriterio', {
                NRROW: form.value.NRROW,
                NOME: form.value.NOME,
                ATIVIDADES: form.value.ATIVIDADES,
                TIPOADICAO: form.value.TIPOADICAO,
                HABILIDADES: form.value.HABILIDADES,
                PRIORIDADE: form.value.PRIORIDADE,
                IDATIVO: form.value.IDATIVO,
            }).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data);
                    grid.datasource.get();
                    this.fecharModal();
                } else {
                    Utilities.showSuccessMessage();
                    grid.datasource.get();
                    this.fecharModal();
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public dialogInativarAtividadesCriterios({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003003AtividadeCriterioController.inativarAtividadesCriterios', 'Inativar Registro', 'Deseja realmente inativar todos os registros selecionados?');
    }

    public inativarAtividadesCriterios() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-atividades-criterios');

        Utilities.deleteInfoGrid('/inativarAtividadesCriterios', { component: grid }).then((response: any) => {
            DialogService.hide();
            if (response.data === 'erroFilhos') {
                Utilities.showDialog('Atenção!', 'warning', 'Alguns registros não puderam ser inativados pois estão relacionados a uma atividade!');
            } else if (response.data === 'erroAtvCrit') {
                Utilities.showDialog('Atenção!', 'warning', 'Alguns registros não puderam ser inativados pois estão relacionados a uma atividade!');
            }
        });
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade-atividadescrit');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRATIVIDADE: element.NRATIVIDADE,
                NMATIVIDADE: element.NMATIVIDADE,
            };
            grid.datasource.post(dados);
        });

        grid.datasource.get();
    }
}
