/* eslint-disable */
import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, Modal, ModalService, TextInput,
} from '@zeedhi/common';
import { Http, IEventParam, MemoryDatasource, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00003004EscopoServicoController {
    private modalEscopo!: Modal;

    public controlarEdicao: boolean = false;

    public modalErroInativarEscopo!: Modal;

    public async criarModais() {
        this.modalErroInativarEscopo = await ModalService.createFromJson('fm_00003_000_checklist/modal/fm_00003_005_modal_inatividade_escopo', true);
    }

    public async abrirModalErroInatividade() {
        this.modalErroInativarEscopo.show();
    }

    public fecharModalInatividade() {
        this.modalErroInativarEscopo.hide();
    }

    public async abrirModal() {
        await this.modalEscopo.show();
        this.modalEscopo = await ModalService.createFromJsonPost('', { jsonPath: '/fm_00003_000_checklist/modal/fm_00003_004_salvar_escopo' });
    }

    public fecharModal() {
        this.modalEscopo.hide();
        this.modalEscopo?.destroy();
    }

    public destruirModais() {
        this.modalErroInativarEscopo.destroy();
    }

    public salvarEscopo({ event }: any) {
        event.preventDefault();

        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-servico');

        if (Utilities.validateGrid(grid)) {
            LoadingService.show();
            Http.post('/salvarEscopoServico', grid.getEditedRows()).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data);
                } else {
                    Utilities.showSuccessMessage();
                }
            }).finally(() => {
                grid.cancelEditedRows();
                grid.datasource.get();
                LoadingService.hide();
            });
        }
    }
    
    public canEditRow({ row }: any) {
        return true;
    }
    
    public adicionarNovaLinha({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRESCOPOSTAREFA: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMESCOPOSTAREFA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public dialogInativarEscopos({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003004EscopoServicoController.inativarEscopos', 'Inativar Registros', 'Deseja realmente inativar todos os registros selecionados?');
    }

    public inativarEscopos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-servico');
        Utilities.deleteInfoGrid('/inativarEscopos', { component: grid }).then(() => {
            grid.datasource.get();
            DialogService.hide();
        });
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade-escopo');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRTAREFA: element.NRTAREFA,
                NMTAREFA: element.NMTAREFA,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
