import { Utilities } from '@/util/Utilities';
import {
    DateRange, Form, LoadingService, Modal, ModalService, Row, DialogService, Select, SelectMultiple, TextInput, Time, Date as Data, Button,
    Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata, Validation, ViewService,
} from '@zeedhi/core';
import { TekGrid, TekGridColumn } from '@zeedhi/teknisa-components-common';
import { FieldGroup } from '@zeedhi/zd-field-group';

/**
 * validation que valida se um select do grid é required caso ele possua datasource.data
 ** INTEGRAÇÃO COM SM
 */
Validation.register('dataRequired', (config?: { id?: any }) => {
    const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
    return (value: any) => {
        if (!config?.id) return '';

        // busca o select pelo id da linha passado via conditions
        const comp = grid.getComponent<Select>(config?.id, 'NRSEQCONTRATO');

        const hasData = comp.datasource.data.length > 0;

        // válido se não tiver dados
        if (!hasData) return true;

        // se tiver dados, aplica Validations.required
        const validateFunction = Validation.get('required')(config);
        return validateFunction(value);
    };
});
export class Fm00004000TarefasMassaController {
    private modalPreencherInformacoes!: Modal;

    private cadastroTarefaMassa!: Modal;

    private modalConflito!: Modal;

    public modalConflitoEdicaoTarefa!: Modal;

    private modalHistoricoTarefa!: Modal;

    public visibilidadeModalConflitoEdicaoTarefa: boolean = false;

    public visibilidadeGridItens: boolean = false;

    public visibilidadeForm: boolean = true;

    public parametrizacao = '';

    public row: any = [];

    public disabledPreencher: boolean = true;

    public disabledPeriodo: boolean = false;

    // public addNewRow({ component }: IEventParam<TekGrid>) {
    //     const newId = 0;
    //     component.addNewRow({ NRCRITERIOTAREFACHK: newId, IDATIVO: 'S' }, 'start');
    //     setTimeout(() => {
    //         const nameInput = Metadata.getInstance<TextInput>(`DSCRITERIOTAREFA_editable_${newId}`);
    //         nameInput.setFocus();
    //     }, 100);
    // }

    public onMountedTipoInsercao({ component }: IEventParam<Select>) {
        component.datasource.data = [];
        (component.datasource as MemoryDatasource).allData = [];

        const valores = [
            {
                name: 'Considerar todos os dias',
                value: 1,
            },
            {
                name: 'Considerar dias úteis (Criar tarefa no próximo dia útil)',
                value: 2,
            },
            {
                name: 'Considerar dias úteis (Não criar tarefa)',
                value: 3,
            },
        ];

        setTimeout(() => {
            if (this.parametrizacao === 'N') {
                const codARemover = 2;
                const index = valores.findIndex((item: any) => item.value === codARemover);

                if (index !== -1) {
                    valores.splice(index, 1);
                }
            } else {
                valores.splice(1, 0, {
                    name: 'Considerar dias úteis (Criar tarefa no próximo dia útil)',
                    value: 2,
                });
            }

            component.datasource.updateData(valores);
        }, 500);
    }

    public changeRecorrencia({ component }: IEventParam<Select>) {
        const data = Metadata.getInstance<Data>('select-data');
        const xDias = Metadata.getInstance<TextInput>('input-x-dias');
        const periodo = Metadata.getInstance<DateRange>('select-periodo');
        const dtFinal = Metadata.getInstance<Data>('select-data-limite');

        switch (component.value) {
            case 'SR':
                this.disabledPeriodo = false;
                data.removeValidation('required');
                data.disabled = true;
                data.value = null;
                periodo.addValidation('required', {});
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.removeValidation('required');
                dtFinal.disabled = true;
                break;
            case 'M':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'MP':
                periodo.addValidation('required', {});
                this.disabledPeriodo = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'Q':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'QP':
                periodo.addValidation('required', {});
                this.disabledPeriodo = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'S':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'SP':
                periodo.addValidation('required', {});
                this.disabledPeriodo = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'P':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = false;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            default:
                break;
        }
    }

    public preencherGrid() {
        const form = Metadata.getInstance<Form>('form-preencher-massa');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        const tipoMarcacao = Metadata.getInstance<Select>('IDTIPOMARCACAOTAREFA').value;
        const hora = Metadata.getInstance<Time>('selectHoraInicio').value;
        const duracao = Metadata.getInstance<TextInput>('selectDuracao').value;
        const operador = Metadata.getInstance<Select>('select-operador').value;
        const encarregado = Metadata.getInstance<Select>('ENCARREGADO').value;
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value;

        const columnOper = grid.columns.filter(({ name } : { name: string }) => name === 'OPERADOR')[0];
        const columnEquipe = grid.columns.filter(({ name } : { name: string }) => name === 'EQUIPE')[0];
        const columnEncarregado = grid.columns.filter(({ name } : { name: string }) => name === 'ENCARREGADO')[0];

        columnOper.setLookupDataFilter({ NRLOCALCLIENTE: local });
        columnEquipe.setLookupDataFilter({ NRLOCALCLIENTE: local });
        columnEncarregado.setLookupDataFilter({ CODIGO: operador, NRLOCALCLIENTE: local, RESP: this.responsavel });

        if (Utilities.validateForm(form)) {
            grid.selectedRows.forEach((element: any) => {
                const newRow: any = {
                    ...element,
                    IDTIPOMARCACAOTAREFA: tipoMarcacao,
                    HORAINICIO: hora,
                    DURACAO: duracao,
                    OPERADOR: operador,
                    ENCARREGADO: encarregado,
                };
                grid.datasource.put(newRow);
            });
            grid.datasource.get();
            grid.selectedRows = [];
            this.fecharModal();
        }
    }

    public habilitarAcao({ component }: IEventParam<TekGrid>) {
        this.disabledPreencher = (component.selectedRows.length <= 0);
    }

    // TRANSFERÊNCIA AUTOMÁTICA DE OPERADORES
    private logModal!: Modal;

    public abrirModal() {
        this.modalPreencherInformacoes.show();
        this.validaParametrizacao();

        setTimeout(() => {
            Metadata.getInstance<Form>('form-preencher-massa').clearForm();
            //  trecho do modal de preenchimento em massa
            const hora = Metadata.getInstance<Time>('selectHoraInicio');
            const duracao = Metadata.getInstance<TextInput>('selectDuracao');
            const att = Metadata.getInstance<Row>('row-atencao');
            const operador = Metadata.getInstance<Select>('select-operador');
            const encarregado = Metadata.getInstance<Select>('ENCARREGADO');
            const equipe = Metadata.getInstance<Select>('equipe');
            const responsaveis = Metadata.getInstance<Select>('responsaveis-tarefa');

            if (this.parametrizacao !== 'N') {
                hora.disabled = true;
                // hora.removeValidation('required');
                duracao.disabled = true;
                duracao.removeValidation('required');
                duracao.removeValidation('minLength');
                att.isVisible = true;
            } else {
                hora.disabled = false;
                hora.addValidation('required');
                duracao.disabled = false;
                duracao.addValidation('required');
                att.isVisible = false;
            }

            if (responsaveis.value) {
                if (responsaveis.value && responsaveis.value === '1') {
                    operador.removeValidation('required');
                    operador.disabled = true;
                    equipe.removeValidation('required');
                    equipe.disabled = true;
                } else if (responsaveis.value === '2') {
                    operador.datasource.setFilter({
                        NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
                    });
                    operador.addValidation('required', {});
                    operador.disabled = false;
                    equipe.removeValidation('required');
                    equipe.disabled = true;
                } else {
                    equipe.datasource.setFilter({
                        NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
                    });
                    equipe.addValidation('required', {});
                    equipe.disabled = false;
                    operador.removeValidation('required');
                    operador.disabled = true;
                }
            }

            if (Metadata.getInstance<Select>('selectCriarVistoria').value === '2') {
                encarregado.disabled = true;
                encarregado.removeValidation('required');
            } else {
                encarregado.disabled = false;
                encarregado.addValidation('required', {});
            }
        }, 500);
    }

    public changeEncarregadoPreencherMassa({ component }: IEventParam<Select>) {
        this.responsavel = (component.name === 'select-operador') ? 'OPERADOR' : 'EQUIPE';
        Metadata.getInstance<Select>('ENCARREGADO').datasource.setFilter({
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
            CODIGO: component.value,
            RESP: this.responsavel,
        });
    }

    public modConflito: boolean = false;

    public abrirModalConflito(parametros: any) {
        this.modalConflito.show();
        this.modConflito = true;
        setTimeout(() => {
            this.preencherGridConflitos(parametros);
        }, 500);
    }

    public destruirModal() {
        this.modalPreencherInformacoes?.destroy();
        this.modalConflito?.destroy();
        // TRANSFERÊNCIA AUTOMÁTICA DE OPERADORES
        this.logModal?.destroy();
    }

    public fecharModal() {
        this.modalPreencherInformacoes.hide();
    }

    public fecharModalConflito() {
        this.modalConflito.hide();
    }

    public habilitaOpcaoDias: boolean = true;

    public modalRequest: Promise<any> | null = null;

    public async exibirModalCadastroMassa({ event }: any) {
        event.preventDefault();
        LoadingService.show();
        if (this.modalRequest === null && (this.cadastroTarefaMassa === undefined || !this.cadastroTarefaMassa.isVisible)) {
            this.modalRequest = ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_005_modal_cadastro_tarefas_massa' });

            this.cadastroTarefaMassa = await this.modalRequest;
            this.modalRequest = null;
            this.validaParametrizacao();
            this.habilitaOpcaoDias = (this.parametrizacao === 'N');
            this.cadastroTarefaMassa.show();
            this.modConflito = false;
        }
        LoadingService.hide();
    }

    public async criarModal() {
        this.modalPreencherInformacoes = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_00004_006_modal_preencher_massa', true);
        this.modalConflito = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_00004_004_modal_conflito_datas', true);
        this.modalConflitoEdicaoTarefa = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_00004_004_modal_conflito_datas_modal_edicao', true);
        this.logModal = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_modal_log_transferencia', true);
    }

    public async fecharModalTarefasMassa() {
        this.cadastroTarefaMassa.hide();
        this.cadastroTarefaMassa?.destroy();
    }

    public clickButtonToExpand({ component }: any) {
        const form = Metadata.getInstance<Form>(`formTarefas_${component.name}`);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public operador: any = '';

    public focusOperadores({ component }: any) {
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        if (grid.editing) {
            component.datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
                NRTAREFACHK: Metadata.getInstance<SelectMultiple>('NRTAREFACHK').value,
            });
        }
    }

    public focusEquipes({ component }: any) {
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        if (grid.editing) {
            component.datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
            });
        }
    }

    private responsavel: string = '';

    public changeEncarregado({ component }: any) {
        // responsavel -> define se é equipe ou operador unico
        /* eslint-disable */
        this.responsavel = component.name.split('_')[0];
        /* eslint-enable */
        this.operador = component.value;
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        const columnEncarregado = grid.columns.filter(({ name } : { name: string }) => name === 'ENCARREGADO')[0];
        columnEncarregado.setLookupDataFilter({ CODIGO: component.value, NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value, RESP: this.responsavel });
    }

    public focusEncarregado({ component }: any) {
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        if (grid.editing) {
            component.datasource.setFilter({
                CODIGO: this.operador,
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
                RESP: this.responsavel,
            });
        }
    }

    public validaVistoriaGrid({ component }: any) {
        const observacao = Metadata.getInstance<Select>('selectObservacoesObrigatorias');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        const filedGroupCriterios = Metadata.getInstance<FieldGroup>('field-group-criterio');
        const columnEncarregado = grid.columns.filter(({ name } : { name: string }) => name === 'ENCARREGADO')[0];

        if (component.value) {
            if (component.value === '1') {
                observacao!.datasource!.data!.push(
                    { value: '3', text: 'Somente na vistoria' },
                    { value: '4', text: 'Na tarefa e na vistoria' },
                );
                columnEncarregado.editable = true;
                filedGroupCriterios.isVisible = true;
                filedGroupCriterios.expanded = true;
            } else {
                filedGroupCriterios.isVisible = false;
                const novaRow = grid.datasource.data.map((row: any) => ({ ...row, ENCARREGADO: null }));
                grid.datasource.updateData(novaRow);
                columnEncarregado.editable = false;
                observacao.datasource?.get();
            }
        } else {
            columnEncarregado.editable = true;
        }
    }

    public changeTipoDeLocal({ component }: IEventParam<Select>) {
        const tipoLocal = Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI');
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');

        tipoLocal.value = null;
        local.value = null;
        checklist.value = null;

        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            local.disabled = true;
        }
    }

    public changeLocal({ component }: IEventParam<SelectMultiple>) {
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');

        if (component.value.length > 0) {
            local.disabled = false;
            checklist.disabled = false;

            local.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'N',
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
            });

            checklist.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'N',
            });
        } else {
            local.value = null;
            checklist.value = null;
            local.disabled = true;
            checklist.disabled = true;
        }
    }

    public changeItensChecklist({ component }: IEventParam<Select>) {
        const itenschecklist = Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK');
        if (component.value) {
            itenschecklist.disabled = false;
            itenschecklist.datasource.setFilter({
                NRCHECKLIST: component.value,
            });
        } else {
            itenschecklist.value = null;
            itenschecklist.disabled = true;
        }
    }

    public changeEscopo({ component }: IEventParam<SelectMultiple>) {
        const escopo = Metadata.getInstance<Select>('NRESCOPOSTAREFA');
        if (component.value.length > 0) {
            escopo.disabled = false;
            escopo.datasource.setFilter({
                NRATIVIDADECHK: component.value,
            });
        } else {
            escopo.value = null;
            escopo.disabled = true;
        }
    }

    public changeTarefa({ component }: IEventParam<SelectMultiple>) {
        const atividade = Metadata.getInstance<SelectMultiple>('NRTAREFACHK');
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');

        if (component.value.length > 0) {
            atividade.disabled = false;
            atividade.datasource.setFilter({
                NRATIVIDADECHK: itenschecklist.value,
                NRESCOPOTAREFA: component.value,
            });
        } else {
            atividade.value = null;
            atividade.disabled = true;
        }
    }

    public defineTarefa() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        const fieldGroup = Metadata.getInstance<FieldGroup>('field-group-dados-agendamento');
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');
        const escopo = Metadata.getInstance<SelectMultiple>('NRESCOPOSTAREFA');

        if (grid.editing) grid.cancelEditedRows();
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        if (local.value.length > 0 && escopo.value.length > 0) {
            this.changeTarefa({ component: escopo });
            escopo.selectValue.forEach((esc: any) => {
                local.selectValue.forEach((element: any) => {
                    const dados: any = {
                        SEQUENCIAL: `${element.NRLOCALCLIENTE}${esc.NRESCOPOSTAREFA}`,
                        NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                        NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                        NMESCOPOSTAREFA: esc.NMESCOPOSTAREFA,
                        NRESCOPOSTAREFA: esc.NRESCOPOSTAREFA,
                        NRESTRUTURAM: element.NRESTRUTURAM,
                        NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
                    };
                    grid.datasource.post(dados);
                });
            });
            grid.datasource.get();
            fieldGroup.expanded = true;
            // Coluna de operador
            grid.columns[6].setLookupDataFilter({ NRLOCALCLIENTE: local.value });
            // Coluna de equipe
            grid.columns[7].setLookupDataFilter({ NRLOCALCLIENTE: local.value });
        }
    }

    public get itensCheckList(): object {
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');
        return itenschecklist.value;
    }

    public get escoposChecklist(): object {
        return Metadata.getInstance<Select>('NRESCOPOSTAREFA').value;
    }

    public get itensCheckListRow(): object {
        const itenschecklist = Metadata.getInstance<Select>('informacoesItensChecklist');
        return itenschecklist.value;
    }

    public setCriterios({ component }: IEventParam<SelectMultiple>) {
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        const escopo = Metadata.getInstance<Select>('NRESCOPOSTAREFA');
        const gridCriterios = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');
        const prioridade = Metadata.getInstance<Select>('select-prioridade');

        if (component.value.length > 0) {
            gridCriterios.datasource?.setFilter({
                NRTAREFACHK: component.value,
                NRATIVIDADECHK: itenschecklist.value,
                NRCHECKLIST: checklist.value,
                NRESCOPOTAREFA: escopo.value,
            });
            gridCriterios.datasource?.get();
            gridCriterios.isVisible = true;

            // trecho de código adicionado para validação de prioridade
            const arrayPrioritario: any = component.selectValue.find((array: any) => array.IDPRIORIDADE === 1);
            if (arrayPrioritario) {
                prioridade.disabled = true;
                prioridade.value = arrayPrioritario.NRPRIORIDADETAREFA;
            } else {
                prioridade.disabled = false;
                prioridade.value = null;
            }
        } else {
            component.value = null;
            gridCriterios.isVisible = false;
        }
    }

    public executorHora() {
        const dados: any = [];
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        grid.datasource.data.forEach((element: any) => {
            dados.push({
                HORAINICIO: element.HORAINICIO,
                DURACAO: element.DURACAO,
                FUNCIONARIO: element.OPERADOR,
                ENCARREGADO: element.ENCARREGADO,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRESCOPOSTAREFA: element.NRESCOPOSTAREFA,
                NRSEQCONTRATO: element.NRSEQCONTRATO,
                IDTIPOMARCACAOTAREFA: element.IDTIPOMARCACAOTAREFA,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
                IGNORARTAREFA: 'N',
                EQUIPE: element.EQUIPE,
            });
        });
        return dados;
    }

    public validarMarcacaoCriterios() {
        const criarVistoria = Metadata.getInstance<Select>('selectCriarVistoria');
        const gridCriterios = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');

        if ((criarVistoria.value === '1' && gridCriterios.selectedRows.length > 0) || criarVistoria.value === '2') {
            return true;
        }
        return false;
    }

    public async inserirTarefa() {
        const formTarefas = Metadata.getInstance<Form>('formCadastroTarefas');
        const formCriterios = Metadata.getInstance<Form>('formTarefas_dropBoxCriterioAvaliacao');
        const formAgendamento = Metadata.getInstance<Form>('formTarefas_dropBoxDadosAgendamento');
        const tekGridCriteriosDeAvaliacaoCadastro = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');
        const diaSemana = Metadata.getInstance<SelectMultiple>('selectDiaSemana').value;
        const dataRange = Metadata.getInstance<DateRange>('select-periodo').value;
        const data = Metadata.getInstance<DateRange>('select-data').value;
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        const gridValid = await this.validateGrid(grid);

        if (gridValid && Utilities.validateForm(formTarefas) && Utilities.validateForm(formCriterios) && Utilities.validateForm(formAgendamento) && this.validarMarcacaoCriterios()) {
            const parametros = {
                NOMETAREFA: Metadata.getInstance<TextInput>('nomeTarefa').value,
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
                NRTIPOLOCALCLI: Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI').value,
                NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
                NRCHECKLIST: Metadata.getInstance<Select>('NRCHECKLIST').value,
                NRATIVIDADECHK: Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK').value,
                NRTAREFACHK: Metadata.getInstance<SelectMultiple>('NRTAREFACHK').selectValue,
                NRCRITERIOTAREFACHK: tekGridCriteriosDeAvaliacaoCadastro.selectedRows.map((element: any) => element),
                DATERANGE: (dataRange.length === 0) ? [data, data] : dataRange,
                DIASEMANA: diaSemana,
                VISTORIA: Metadata.getInstance<Select>('selectCriarVistoria').value,
                OBSERVACOES: Metadata.getInstance<Select>('selectObservacoesObrigatorias').value,
                EXECUTOR: this.executorHora(),
                RECORRENCIA: Metadata.getInstance<Select>('select-recorrencia').value,
                QTDDIAS: Metadata.getInstance<TextInput>('input-x-dias').value,
                DTTERMINO: Metadata.getInstance<Data>('select-data-limite').value,
                NRPRIORIDADETAREFA: Metadata.getInstance<Select>('select-prioridade').value,
                NREPI: Metadata.getInstance<SelectMultiple>('select-epi').value,
                NRMODELOPOP: Metadata.getInstance<SelectMultiple>('select-modelopop').value,
                DSOBSERVACAO: Metadata.getInstance<Textarea>('campo-observacao').value,
                RESPONSAVEIS: Metadata.getInstance<Select>('responsaveis-tarefa').value,
            };
            this.salvarTarefa(parametros, this.executorHora(), false);
        } else {
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
    }

    public salvarTarefa(parametros: any, executor: any, conflito: boolean) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tarefas');

        LoadingService.show();
        Http.post('/insertTarefasRecorrentes', {
            NOMETAREFA: parametros.NOMETAREFA,
            CDCLIENTE: parametros.CDCLIENTE,
            NRTIPOLOCALCLI: parametros.NRTIPOLOCALCLI,
            NRLOCALCLIENTE: parametros.NRLOCALCLIENTE,
            NRCHECKLIST: parametros.NRCHECKLIST,
            NRATIVIDADECHK: parametros.NRATIVIDADECHK,
            NRTAREFACHK: parametros.NRTAREFACHK,
            NRCRITERIOTAREFACHK: parametros.NRCRITERIOTAREFACHK,
            DATERANGE: parametros.DATERANGE,
            DIASEMANA: parametros.DIASEMANA,
            VISTORIA: parametros.VISTORIA,
            OBSERVACOES: parametros.OBSERVACOES,
            EXECUTOR: executor,
            RECORRENCIA: (conflito === true) ? 'SR' : parametros.RECORRENCIA,
            QTDDIAS: parametros.QTDDIAS,
            DTTERMINO: parametros.DTTERMINO,
            CONFLITO: conflito,
            PARAMETRIZACAO: this.parametrizacao,
            NRPRIORIDADETAREFA: parametros.NRPRIORIDADETAREFA,
            NREPI: parametros.NREPI,
            NRMODELOPOP: parametros.NRMODELOPOP,
            DSOBSERVACAO: parametros.DSOBSERVACAO,
            RESPONSAVEIS: parametros.RESPONSAVEIS,
        }).then((response: IDictionary) => {
            if (response.data !== 'OK') {
                this.abrirModalConflito(response.data);
                Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
            } else {
                Utilities.showSuccessMessage();
                this.fecharModalConflito();
                this.fecharModalTarefasMassa();
                grid.datasource.get();
            }
        }).catch((error: any) => {
            if (error && error.response && error.response.data && error.response.data.error && error.response.data.error.includes('Undefined index:')) {
                let [erroCampo] = error.response.data.error.split('\'').filter((elemento: any) => elemento.includes('Undefined index:'));
                erroCampo = erroCampo ? erroCampo.replaceAll('\'', '').replace('Undefined index: ', '') : erroCampo;
                if (!erroCampo) {
                    Utilities.showErrorMessage(error);
                } else {
                    Utilities.showDialog('Erro', 'error', 'Verifique se todos os campos obrigatórios foram preenchidos corretamente.');
                }
            } else {
                Utilities.showErrorMessage(error);
            }
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public beforeCancel() {
        Metadata.getInstance<Select>('responsaveis-tarefa').disabled = false;
        Metadata.getInstance<Select>('selectCriarVistoria').disabled = false;
    }

    public validarGrid({ component, event }: IEventParam<TekGrid>) {
        if (component.isGridValid(true)) {
            Utilities.showSuccessMessage();
            this.editedRows = component.getEditedRows();
            Metadata.getInstance<Select>('responsaveis-tarefa').disabled = false;
            Metadata.getInstance<Select>('selectCriarVistoria').disabled = false;
        } else {
            event?.preventDefault();
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
    }

    public validaParametrizacao() {
        Http.get('getParametroConflito').then((response) => {
            this.parametrizacao = (response.data.length !== 0) ? response.data[0].IDVALIDACRIATAREFA : null;
        });
    }

    // Rotina log de transferência automática de operadores
    public async abrirModalLog({ component, event }: IEventParam<Modal>) {
        event?.stopPropagation();
        await this.logModal.show();
        Metadata.getInstance<TekGrid>('grid-log-tarefas').datasource.setFilter({
            NRCHECKLISTAGEND: component.name.split('_')[1],
        });
    }

    public fecharModalLog() {
        this.logModal.hide();
    }

    public excluirTarefas({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00004000TarefasMassaController.aposConfirmarDelete', 'Excluir Registro?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        LoadingService.show();
        Http.post('/inativarTarefas', {
            NRS: component.selectedRows.map((element: any) => element.NRCHECKLISTAGEND),
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public setFiltroGridPrincipal({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    public rowClickFiltroGrid() {
        this.visibilidadeGridItens = true;
        this.visibilidadeForm = !this.visibilidadeGridItens;
    }

    public dialogConfirmarCriacaoVistoria(msg: string) {
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Atenção!!',
            text: msg,
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: '{{Fm00004000TarefasMassaController.acaoDialog}}',
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: '{{Fm00004000TarefasMassaController.acaoDialog}}',
                    },
                },
            ],
        });
    }

    public acaoDialog({ component }: IEventParam<Button>) {
        const encarregado = Metadata.getInstance<Select>('ENCARREGADO');
        const criaVistoria = Metadata.getInstance<Select>('selectCriarVistoria');

        // Significa que o usuário não deseja continuar com a criação da vistoria para um Operador Máximo na Hierarquia
        if (component.label === 'Não') {
            criaVistoria.value = '2';
        }

        encarregado.removeValidation('required');
        encarregado.disabled = true;
        DialogService.hide();
    }

    public changeResponsaveis({ component }: IEventParam<SelectMultiple>) {
        const gridAgendamento = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        const criarVistoria = Metadata.getInstance<Select>('selectCriarVistoria');
        // const columnOper = (gridAgendamento.getColumn('OPERADOR') as TekGridColumn);
        const columnOper = gridAgendamento.columns.filter(({ name } : { name: string }) => name === 'OPERADOR')[0];
        const columnEquipe = gridAgendamento.columns.filter(({ name } : { name: string }) => name === 'EQUIPE')[0];
        let colunas: { OPERADOR?: any, EQUIPE?: any } = {}; // Declaração inicial sem tipos definidos

        if (component.value) {
            // Caso a opção seja 'sem responsável' não é possível ter vistoria, pois não terá encarregados.
            if (component.value === '1') { // Sem operador específico
                criarVistoria.value = '2';
                criarVistoria.disabled = true;
                columnOper.editable = false;
                columnEquipe.editable = false;

                colunas = {
                    OPERADOR: null,
                    EQUIPE: null,
                };
            } else if (component.value === '2') { // Operador específico
                criarVistoria.disabled = false;
                columnOper.editable = true;
                columnEquipe.editable = false;

                colunas = {
                    EQUIPE: null,
                };
            } else { // equipe
                columnOper.editable = false;
                columnEquipe.editable = true;

                colunas = {
                    OPERADOR: null,
                };
            }
            this.limparColunasGrid({ component: gridAgendamento }, colunas);
        } else {
            criarVistoria.disabled = false;
            columnOper.editable = false;
            columnEquipe.editable = false;
        }
    }

    public limparColunasGrid({ component }: IEventParam<TekGrid>, colunas: any) {
        const novaRow = component.datasource.data.map((row: any) => ({ ...row, ...colunas }));
        component.datasource.updateData(novaRow);
    }

    public getRowKey({ row }: any) {
        return row.SEQUENCIAL;
    }

    /* INTEGRAÇÃO COM SM */
    /* carrega o datasource dos selects */
    public async loadSelects(grid: TekGrid) {
        const rows = grid.datasource.data;

        const promises = rows.map(async (row) => {
            const select = grid.getComponent<Select>(row.SEQUENCIAL, 'NRSEQCONTRATO');
            // faz a requisição para buscar os dados do select
            await select.datasource.setFilter({
                NRESTRUTURAM: grid.datasource.currentRow.NRESTRUTURAM,
            });
        });

        await Promise.all(promises);
    }
    /* INTEGRAÇÃO COM SM */

    public async validateGrid(grid: TekGrid): Promise<boolean> {
        // const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        // se não tiver em edição
        // 1. entra em edição
        // 2. espera a criação dos selects
        // 3. espera carregar o datasource dos selects
        // 4. valida o grid
        LoadingService.show('Validando grid de agendamentos');
        if (!grid.editing) {
            grid.editing = true;
            await this.onInlineEdit({ component: grid });
            // cria uma promise que só resolve após criar os selects, carregar, validar...
            return new Promise((resolve) => {
                // espera a criação dos selects no próximo tick
                ViewService.nextTick(async () => {
                    await this.loadSelects(grid);

                    const valid = grid.isGridValid(true);

                    if (valid) grid.editing = false;
                    LoadingService.hide();
                    resolve(valid);
                });
            });
        }

        // se não tiver em edição, valida normalmente
        const valid = grid.isGridValid(true);
        LoadingService.hide();
        return valid;
    }

    // evento de inlineEdit
    public async onInlineEdit({ component }: IEventParam<TekGrid>) {
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');
        const resp = Metadata.getInstance<Select>('responsaveis-tarefa');
        const criaVistoria = Metadata.getInstance<Select>('selectCriarVistoria');

        resp.disabled = true;
        criaVistoria.disabled = true;

        let resolvePromise: <T>(value: T | PromiseLike<T>) => void;

        const prm = new Promise((resolve) => {
            resolvePromise = resolve;
        });

        // Coluna de operador
        const columnOper = component.columns.filter(({ name } : { name: string }) => name === 'OPERADOR')[0];
        columnOper.setLookupDataFilter({ NRLOCALCLIENTE: local.value });
        // Coluna de equipe
        const columnEquipe = component.columns.filter(({ name } : { name: string }) => name === 'EQUIPE')[0];
        columnEquipe.setLookupDataFilter({ NRLOCALCLIENTE: local.value });

        ViewService.nextTick(async () => {
            const promises: Promise<any>[] = component.datasource.data.map(async (rows: any) => {
                if (resp.value !== '1') {
                    let cod: any;
                    if (resp.value === '2') {
                        const operador = component.getComponent<Select>(rows.SEQUENCIAL, 'OPERADOR');
                        await operador.datasource.setFilter({
                            NRLOCALCLIENTE: local.value,
                        });
                        operador.setValue(rows.OPERADOR);
                        cod = rows.OPERADOR;
                        this.responsavel = 'OPERADOR';
                    } else if (resp.value === '3') {
                        const equipe = component.getComponent<Select>(rows.SEQUENCIAL, 'EQUIPE');
                        await equipe.datasource.setFilter({
                            NRLOCALCLIENTE: local.value,
                        });
                        equipe.setValue(rows.EQUIPE);
                        cod = rows.EQUIPE;
                        this.responsavel = 'EQUIPE';
                    }

                    if (criaVistoria.value === '1') {
                        const encarregado = component.getComponent<Select>(rows.SEQUENCIAL, 'ENCARREGADO');
                        await encarregado.datasource.setFilter({
                            CODIGO: cod,
                            NRLOCALCLIENTE: local.value,
                            RESP: this.responsavel,
                        });
                        encarregado.setValue(rows.ENCARREGADO);
                    }
                }
            });
            this.loadSelects(component);

            await Promise.all(promises);
            resolvePromise(null);
        });
        return prm;
    }

    /* inicio rotina de conflitos */
    public dadosBanco: any = {};

    public preencherGridConflitos(parametros: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-datas');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        /* eslint-disable */
        let cont = 0;

        for (const key in parametros.DADOSCONFLITO) {
            const dados: any = {
                CHAVE: cont,
                NRLOCALCLIENTE: parametros.DADOSCONFLITO[key].NRLOCALCLIENTE,
                NRESCOPOSTAREFA: parametros.DADOSCONFLITO[key].NRESCOPOSTAREFA,
                DATAINICIAL: parametros.DADOSCONFLITO[key].DATAINICIAL,
                DATAFINAL: parametros.DADOSCONFLITO[key].DATAFINAL,
                DATAFORMATADAINICIAL: parametros.DADOSCONFLITO[key].DATAFORMATADAINICIAL,
                DATAFORMATADAFINAL: parametros.DADOSCONFLITO[key].DATAFORMATADAFINAL,
                IDTIPOMARCACAOTAREFA: parametros.DADOSCONFLITO[key].IDTIPOMARCACAOTAREFA,
                HORAINICIO: parametros.DADOSCONFLITO[key].HORAINICIO,
                DURACAO: parametros.DADOSCONFLITO[key].DURACAO,
                OPERADOR: parametros.DADOSCONFLITO[key].FUNCIONARIO,
                ENCARREGADO: parametros.DADOSCONFLITO[key].ENCARREGADO,
                IGNORARTAREFA: 'N',
                EQUIPE: parametros.DADOSCONFLITO[key].EQUIPE,
            };
            grid.datasource.post(dados);
            cont++;
        }

        grid.datasource.get();
        // Coluna de operador
        (grid.getColumn('OPERADOR') as TekGridColumn).setLookupDataFilter({ NRLOCALCLIENTE: parametros.NRLOCALCLIENTE });
        (grid.getColumn('EQUIPE') as TekGridColumn).setLookupDataFilter({ NRLOCALCLIENTE: parametros.NRLOCALCLIENTE });
        grid.datasource.data.forEach((element): any => {
            this.responsavel = (element.OPERADOR !== null) ? 'OPERADOR' : 'EQUIPE';
            // Coluna do encarregado
            (grid.getColumn('ENCARREGADO') as TekGridColumn).setLookupDataFilter({ CODIGO: (element.OPERADOR !== null) ? element.OPERADOR : null, NRLOCALCLIENTE: parametros.NRLOCALCLIENTE, RESP: this.responsavel });
        });
        this.dadosBanco = parametros;
    }

    public editedRows: any = '';

    public salvarConflitos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-datas');
        const dados: any = [];

        if (this.editedRows.length !== 0) {
            grid.datasource.data.forEach((element: any) => {
                dados.push({
                    HORAINICIO: element.HORAINICIO,
                    DURACAO: element.DURACAO,
                    NRESCOPOSTAREFA: element.NRESCOPOSTAREFA,
                    FUNCIONARIO: element.OPERADOR,
                    ENCARREGADO: element.ENCARREGADO,
                    NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                    NRSEQCONTRATO: element.NRSEQCONTRATO,
                    IDTIPOMARCACAOTAREFA: element.IDTIPOMARCACAOTAREFA,
                    DATAINICIAL: element.DATAINICIAL.split(' ')[0],
                    DATAFINAL: element.DATAFINAL.split(' ')[0],
                    IGNORARTAREFA: element.IGNORARTAREFA,
                });
            });
            const parametros = {
                NOMETAREFA: this.dadosBanco.NOMETAREFA,
                CDCLIENTE: this.dadosBanco.CDCLIENTE,
                NRTIPOLOCALCLI: this.dadosBanco.NRTIPOLOCALCLI,
                NRLOCALCLIENTE: this.dadosBanco.NRLOCALCLIENTE,
                NRCHECKLIST: this.dadosBanco.NRCHECKLIST,
                NRATIVIDADECHK: this.dadosBanco.NRATIVIDADECHK,
                NRTAREFACHK: this.dadosBanco.NRTAREFACHK,
                NRCRITERIOTAREFACHK: this.dadosBanco.NRCRITERIOTAREFACHK,
                DATERANGE: this.dadosBanco.DATERANGE,
                DIASEMANA: this.dadosBanco.DIASEMANA,
                VISTORIA: this.dadosBanco.VISTORIA,
                OBSERVACOES: this.dadosBanco.OBSERVACOES,
                QTDDIAS: this.dadosBanco.QTDDIAS,
                DTTERMINO: this.dadosBanco.DTTERMINO,
                PARAMETRIZACAO: this.dadosBanco.PARAMETRIZACAO,
                NRPRIORIDADETAREFA: this.dadosBanco.NRPRIORIDADETAREFA,
                NREPI: this.dadosBanco.NREPI,
                NRMODELOPOP: this.dadosBanco.NRMODELOPOP,
                DSOBSERVACAO: this.dadosBanco.DSOBSERVACAO,
            };
            this.salvarTarefa(parametros, dados, true);
        } else {
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
    }
    /* fim rotina de conflitos */

    private currentRow: any = {};

    public async abrirModalHistoricoTarefa() {
        this.modalHistoricoTarefa?.destroy();
        this.modalHistoricoTarefa = await ModalService.createFromJson('/fm_00004_000_tarefas/modal/fm_modal_historico_tarefa', true);
        this.modalHistoricoTarefa.show();
    }

    public fecharModalHistoricoTarefa() {
        this.modalHistoricoTarefa.hide();
        this.modalHistoricoTarefa.destroy();
    }

    public async ativaAbrirModalHistoricoTarefa() {
        await this.abrirModalHistoricoTarefa();
        Metadata.getInstance<TekGrid>('grid-historico-tarefa').datasource.setFilter({
            NRCHECKLISTAGEND: this.currentRow.NRCHECKLISTAGEND,
        });
    }

    public setCurrentRow({ row }: any) {
        this.currentRow = row;
    }

    public rowStyleConditions(row: any) {
        switch (row.IDSITUACAOAGEND) {
            case '2':
                return { cssClass: 'linha-grid-andamento' };
            case '3':
                return { cssClass: 'linha-grid-concluida' };
            case '4':
                return { cssClass: 'linha-grid-cancelada' };
            case '6':
                return { cssClass: 'linha-grid-cancelada' };
            default:
                if (row.ATRASO !== null && row.ATRASO.split(' ')[1] === 'dias' && row.ATRASO.split(' ')[0] >= 1) {
                    return { cssClass: 'linha-grid-atraso' };
                }
            return { cssClass: '' };
        }
    }
}
