import { Utilities } from '@/util/Utilities';
import {
    Form,
    LoadingService,
    Modal, ModalService, Select,
} from '@zeedhi/common';
import {
    dayjs, Http, IDictionary, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { ITekGridColumn, TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00004001TransferirTarefasController {
    private modalTransferencia!: Modal;

    public visibilidadeGrid: boolean = true;

    public currentRow: any;

    private modalConflito!: Modal;

    public async abrirModalTransferencia({ component, row }: IEventParam<TekGrid>) {
        this.modalTransferencia = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_003_modal_transferir_tarefas' });
        this.modalTransferencia.show();
        this.visibilidadeGrid = (component.name === 'action-transferir');
        setTimeout(() => {
            this.validacoes(row);
        }, 500);
    }

    public fecharModalTransferencia() {
        this.modalTransferencia.destroy();
        this.modalTransferencia.hide();
    }

    public validacoes(row: any) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        const operadorOrigem = Metadata.getInstance<Select>('operador-origem');

        operadorDestino.value = null;
        operadorOrigem.value = null;

        if (!this.visibilidadeGrid) {
            operadorOrigem.disabled = true;
            operadorDestino.disabled = false;
            operadorOrigem.value = row.CDOPERADOR;
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: row.NRHABILIDADE,
                CDOPERADOR: row.CDOPERADOR,
            });
            this.currentRow = row;
            if (row.CDOPERADOR === null) {
                operadorOrigem.removeValidation('required');
            } else {
                operadorOrigem.addValidation('required', {});
            }
        } else {
            operadorOrigem.disabled = false;
            operadorDestino.disabled = false;
            this.defineTarefa();
        }
    }

    public defineTarefa() {
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        gridInterno.datasource.data = [];
        (gridInterno.datasource as MemoryDatasource).allData = [];

        gridPrincipal.selectedRows.forEach((element: any) => {
            const dados: any = {
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                CDOPERADOR: element.CDOPERADOR,
                DTHORAAGEND: element.DTHORAAGEND,
                NRHABILIDADE: element.NRHABILIDADE,
                DTHRFINALAGEND: element.DTHRFINALAGEND,
                NRPRIORIDADETAREFA: element.NRPRIORIDADETAREFA,
                IDPRIORIDADE: element.IDPRIORIDADE,
            };
            gridInterno.datasource.post(dados);
        });
        gridInterno.datasource.get();
    }

    public changeOperadorDestino({ component }: IEventParam<Select>) {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino');
        Metadata.getInstance<TekGrid>('grid-tarefas-transferir').selectedRows = [];
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (component.value) {
            operadorDestino.datasource.setFilter({
                NRHABILIDADE: component.selectValue.NRHABILIDADE,
                CDOPERADOR: component.value,
            });
        }

        if (this.visibilidadeGrid) {
            gridInterno.datasource.setFilter({
                CDOPERADOR: component.value,
            });
        }
    }

    public transferirTarefa() {
        const operadorDestino = Metadata.getInstance<Select>('operador-destino').value;
        const operadorOrigem = Metadata.getInstance<Select>('operador-origem').value;
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const gridInterno = Metadata.getInstance<TekGrid>('grid-tarefas-transferir');

        if (Utilities.validateForm(Metadata.getInstance<Form>('form-transferencia'))) {
            if (operadorOrigem !== operadorDestino) {
                if (this.visibilidadeGrid) {
                    if (gridInterno.selectedRows.length > 0) {
                        this.salvarTransferencia(operadorDestino, gridInterno.selectedRows).then(async (response: IDictionary) => {
                            if (response.data !== 'OK') {
                                this.abrirModalConflito(response.data);
                                Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
                            } else {
                                Utilities.showSuccessMessage();
                                gridPrincipal.datasource.get();
                                gridInterno.deleteRows();
                                gridInterno.datasource.clearFilter();
                                operadorOrigem.value = null;
                                operadorDestino.value = null;

                                if (gridInterno.datasource.data.length === 0) {
                                    this.fecharModalTransferencia();
                                    gridPrincipal.selectedRows = [];
                                }
                            }
                        }).finally(() => {
                            LoadingService.hide();
                        });
                    } else {
                        Utilities.alertMessage('Selecione pelo menos 1 tarefa para transferir', 'warning');
                    }
                }

                if (!this.visibilidadeGrid) {
                    this.salvarTransferencia(operadorDestino, [this.currentRow]).then((response: IDictionary) => {
                        if (response.data !== 'OK') {
                            this.abrirModalConflito(response.data);
                            Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
                        } else {
                            Utilities.showSuccessMessage();
                            this.fecharModalTransferencia();
                            gridPrincipal.datasource.get();
                            // this.fecharModalConflito();
                        }
                    }).catch((error: any) => {
                        Utilities.showErrorMessage(error);
                    }).finally(() => {
                        LoadingService.hide();
                    });
                }
            } else {
                Utilities.alertMessage('Selecione um Operador de Destino diferente do Operador de Origem', 'warning');
            }
        }
    }

    public async salvarTransferencia(destino: any, tarefa: any) {
        return Http.post('/transferirTarefas', {
            OPERADORDESTINO: destino,
            TAREFAS: tarefa,
        });
    }

    public async abrirModalConflito(parametros: any) {
        this.modalConflito.show();
        setTimeout(() => {
            this.preencherGridConflitos(parametros);
         }, 500);
    }

    public fecharModalConflito() {
        this.modalConflito.hide();
    }

    public preencherGridConflitos(parametros: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-tarefas');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        /* eslint-disable */
        // if (Array.isArray(parametros)) {
        for (const key in parametros) {
            const dados: any = {
                TAREFABANCO: parametros[key].TAREFABANCO,
                NRCHECKLISTAGEND: parametros[key].NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: parametros[key].NMCHECKLISTAGEND,
                NRLOCALCLIENTE: parametros[key].NRLOCALCLIENTE,
                DTHORAAGEND: parametros[key].DTHORAAGEND,
                HORAINICIAL: parametros[key].HORAINICIAL,
                DURACAO: parametros[key].DURACAO,
                CDOPERADOR: parametros[key].CDOPERADOR,
                IGNORARTAREFA: null,
                NRHABILIDADE: parametros[key].NRHABILIDADE,
                IDPRIORIDADE: parametros[key].IDPRIORIDADE,
            };
            grid.datasource.post(dados);

            const coluna: ITekGridColumn | undefined = grid.columns.find((item: any) => { return item.name === 'CDOPERADOR'});
            coluna?.setLookupDataFilter({ NRLOCALCLIENTE: parametros[key].NRLOCALCLIENTE,  NRHABILIDADE: parametros[key].NRHABILIDADE });
        }
        grid.datasource.get();
        // }
        /* eslint-enable */
        // this.dadosBanco = parametros;
    }

    public sortFunction(value1: any, value2: any): boolean {
        return (dayjs(value1, 'DD/MM/YYYY H:m').isAfter(dayjs(value2, 'DD/MM/YYYY H:m')));
    }
}
