import { Utilities } from '@/util/Utilities';
import {
    LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput, Time, Date as Data,
    Textarea, Form,
} from '@zeedhi/common';
import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

interface RowTarefas {
    DSOBSERVACAO: string;
    NREPI: string;
    NRMODELOPOP: number;
    CDCLIENTE: string;
    CDOPERADOR: string;
    CDOPERADORPAI: string;
    DTHORAAGEND: string;
    DTHRFINALAGEND: string;
    IDATIVO: string;
    IDOBRIGAOBSERVACAO: string;
    IDSITUACAOAGEND: string;
    IDTIPOMARCACAOTAREFA: string;
    IDTIPORECORRENCIA: string;
    IDTAREFAEQUIPE: string;
    NMCHECKLIST: string;
    NMCHECKLISTAGEND: string;
    NMFANTCLIE: string;
    NMLOCALCLIENTE: string;
    NMOPERADOR: string;
    NMRAZSOCCLIE: string;
    NMTIPOLOCALCLI: string;
    NRATIVIDADECHK: number;
    NRCHECKLIST: number;
    NRCHECKLISTAGEND: number;
    NRCONTRATOFCT: string;
    NRCRITERIOTAREFACHK: string;
    NRESTRUTURAM: number;
    NRLOCALCLIENTE: number;
    NRPRIORIDADETAREFA: number;
    NRSEQCONTRATO: number;
    NRTAREFACHK: string;
    NRTIPOLOCALCLI: number;
    NRESCOPOSTAREFA: number;
    NREQUIPE: number;
    NREQUIPETAREFA: string;
    NRGERENCIAMENTOCHK: number,
    NRRELCHKAGENDMODPOP: number,
}

export class FmEdicaoTarefasController {
    private exibirInformacoesTarefaModal!: Modal;

    public visibilidadeForm: boolean = true;

    public parametrizacao = '';

    public habilitaOpcaoDias: boolean = (this.parametrizacao === 'N');

    public disabledNmTarefa:boolean = false;

    public disabledDtAgend: boolean = false;

    public disabledHora: boolean = false;

    public disabledDuracao: boolean = false;

    public disabledObs: boolean = false;

    public disabledIdMarcacao: boolean = false;

    public disabledResponsaveis: boolean = false;

    public disabledOperador: boolean = false;

    public disabledEquipe: boolean = false;

    public disabledEncarregado: boolean = false;

    public disabledPrioridade: boolean = false;

    public disabledEpis: boolean = false;

    public disabledModeloPop: boolean = false;

    public disabledObsModeloPop: boolean = false;

    public disabledBtnSalvar: boolean = false;

    public async exibirModalInformacoesTarefas({ row, event }: { row: RowTarefas, event: any }): Promise<void> {
        LoadingService.show();
        event.preventDefault();
        this.exibirInformacoesTarefaModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_001_modal_tarefas' });
        this.exibirInformacoesTarefaModal.show();
        setTimeout(() => {
            this.popularModalInforamcoesTarefas(row);
        }, 0);
        LoadingService.hide();
    }

    public async fecharModalInformacoesTarefas() {
        this.exibirInformacoesTarefaModal.hide();
        this.exibirInformacoesTarefaModal?.destroy();
    }

    public validaParametrizacao() {
        Http.get('getParametroConflito').then((response) => {
            this.parametrizacao = (response.data.length !== 0) ? response.data[0].IDVALIDACRIATAREFA : null;
        });
    }

    public validarCamposHabilitados(row: RowTarefas) {
        this.disabledNmTarefa = (row.IDSITUACAOAGEND !== '1');
        this.disabledDtAgend = (row.IDSITUACAOAGEND !== '1');
        this.disabledHora = (row.IDSITUACAOAGEND !== '1');
        this.disabledDuracao = (row.IDSITUACAOAGEND !== '1');
        this.disabledObs = (row.IDSITUACAOAGEND !== '1');
        this.disabledIdMarcacao = (row.IDSITUACAOAGEND !== '1');
        this.disabledResponsaveis = (row.IDSITUACAOAGEND !== '1');
        this.disabledOperador = (row.IDSITUACAOAGEND !== '1');
        this.disabledEquipe = (row.IDSITUACAOAGEND !== '1');
        this.disabledEncarregado = (row.IDSITUACAOAGEND !== '1');
        this.disabledPrioridade = (row.IDSITUACAOAGEND !== '1');
        this.disabledEpis = (row.IDSITUACAOAGEND !== '1');
        this.disabledModeloPop = (row.IDSITUACAOAGEND !== '1');
        this.disabledObsModeloPop = (row.IDSITUACAOAGEND !== '1');
        this.disabledBtnSalvar = (row.IDSITUACAOAGEND !== '1');
    }

    private popularModalInforamcoesTarefas(row: RowTarefas): void {
        this.validarCamposHabilitados(row);
        Metadata.getInstance<TextInput>('informacoesNomeTarefa').value = row.NMCHECKLISTAGEND;
        Metadata.getInstance<Textarea>('campo-observacao').value = row.DSOBSERVACAO;
        Metadata.getInstance<TextInput>('codigo-tarefa').value = row.NRCHECKLISTAGEND;
        Metadata.getInstance<TextInput>('nrequipetarefa').value = row.NREQUIPETAREFA;
        Metadata.getInstance<TextInput>('idtarefaequipe').value = row.IDTAREFAEQUIPE;
        Metadata.getInstance<Select>('nrgerenciamentochk').value = row.NRGERENCIAMENTOCHK;
        Metadata.getInstance<Select>('NRRELCHKAGENDMODPOP').value = row.NRRELCHKAGENDMODPOP;
        this.setValueSelectCamposModal<Select>('informacoesNomeCliente', row.CDCLIENTE);
        this.changeCampoChecklist(row);
        this.changeCampoTipoDeLocal(row);
        this.changeCampoLocalModalInformacoes(row);
        this.changeItensChecklistModalInformacoes(row);
        this.changeEscopoModalInformacoes(row);
        this.changeAtividadeModalInformacoes(row);
        this.changeContratosModalInformacoes(row);
        this.setVistoria(row);
        this.setResponsaveisInfos(row);
        this.changeFuncionarioModalInformacoes(row);
        this.setEquipeModalInformacoes(row);
        this.setEncarregadoModalInformacoes(row);
        this.setDataTarefaModalInformacoes(row);
        this.setHorarioTarefaModalInformacoes(row);
        this.setDuracaodaTarefaModalInformacoes(row);
        this.setObervacoesObrigatorias(row);
        this.setValueSelectCamposModal<Select>('select-Informacoes-prioridade', row.NRPRIORIDADETAREFA);
        this.setValueSelectCamposModal<Select>('selectInformacoesOpcoesMarcacao', row.IDTIPOMARCACAOTAREFA);
        this.setValueSelectCamposModal<Select>('select-modelopop-info', row.NRMODELOPOP);
        if (row.NREPI.length > 0 || row.NREPI[0] !== '') this.setEpisModalInformacoes(row);
    }

    private setVistoria(row: RowTarefas): void {
        const vistoria = Metadata.getInstance<Select>('selectInformacoesCriarVistoria');
        const encarregado = Metadata.getInstance<Select>('selectInformacoesEncarregado1');

        if (row.NRGERENCIAMENTOCHK !== null) {
            vistoria.value = '1';
            encarregado.disabled = false;
            encarregado.addValidation('required', {});
        } else {
            vistoria.value = '2';
            encarregado.disabled = true;
            encarregado.removeValidation('required');
        }
    }

    private setDuracaodaTarefaModalInformacoes(row: RowTarefas): void {
        const horarioInicialString: string = row.DTHORAAGEND.slice(11, 16);
        const horarioFinalString: string = row.DTHRFINALAGEND.slice(11, 16);

        const horarioInicial: Date = new Date(`1970-01-01T${horarioInicialString}:00`);
        const horarioFinal: Date = new Date(`1970-01-01T${horarioFinalString}:00`);

        const diferencaMinutos: number = (horarioFinal.getTime() - horarioInicial.getTime()) / (1000 * 60);

        const duracaoTarefa = diferencaMinutos.toString();

        Metadata.getInstance<Time>('selectInformacoesdDuracao1').value = (`00:${duracaoTarefa}`);
    }

    private setDataTarefaModalInformacoes(row: RowTarefas): void {
        const data = row.DTHORAAGEND.slice(0, 10);
        Metadata.getInstance<Data>('Informacoes-DTHORAAGEND').value = data;
    }

    private setHorarioTarefaModalInformacoes(row: RowTarefas): void {
        const horario = row.DTHORAAGEND.slice(11, 16);
        Metadata.getInstance<Time>('selectInformacoesHoraInicio1').value = horario;
    }

    private setEpisModalInformacoes(row: RowTarefas): void {
        let intArray: number[] = [];

        if (Array.isArray(row.NREPI)) {
            // Caso NREPI seja um array de strings, mapeia para inteiros
            intArray = row.NREPI.map((str: any) => parseInt(str, 10));
        } else if (typeof row.NREPI === 'string') {
            // Caso NREPI seja uma string, converte diretamente para um array com um único número
            intArray = [parseInt(row.NREPI, 10)];
        }
        // Configura o valor com o array de inteiros
        Metadata.getInstance<SelectMultiple>('select-epi-info').setValue(intArray);
    }

    private setValueSelectCamposModal<T extends Select | SelectMultiple>(field: string, props: T | T[] | string | number | number[]): void {
        Metadata.getInstance<T>(field).setValue(props, false);
    }

    private setResponsaveisInfos(row: RowTarefas): void {
        const responsaveis = Metadata.getInstance<Select>('responsaveis-tarefa');
        let valor: string;

        if (row.CDOPERADOR === null && row.NREQUIPE === null) {
            valor = '1';
        } else if (row.CDOPERADOR !== null) {
            valor = '2';
        } else {
            valor = '3';
        }

        this.setValueSelectCamposModal<Select>(responsaveis.name, valor);
        this.changeResponsaveisInfos({ component: responsaveis });
    }

    private setEncarregadoModalInformacoes(row: RowTarefas): void {
        const encarregado = Metadata.getInstance<Select>('selectInformacoesEncarregado1');
        const responsaveis = Metadata.getInstance<Select>('responsaveis-tarefa');

        let codigo = null;
        let resp = null;

        if (responsaveis.value === '2') {
            codigo = row.CDOPERADOR;
            resp = 'OPERADOR';
        } else if (responsaveis.value === '3') {
            codigo = row.NREQUIPE;
            resp = 'EQUIPE';
        }

        encarregado.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
            CODIGO: codigo,
            RESP: resp,
        });

        this.setValueSelectCamposModal<Select>(encarregado.name, row.CDOPERADORPAI);
    }

    public changeResponsaveisInfos({ component }: IEventParam<Select>) {
        const operador = Metadata.getInstance<Select>('selectInformacoesFuncionario1');
        const equipe = Metadata.getInstance<Select>('selectInformacoesEquipe');

        switch (component.value) {
            case '1':
                operador.disabled = true;
                operador.value = null;
                operador.removeValidation('required');
                equipe.disabled = true;
                equipe.value = null;
                equipe.removeValidation('required');
                break;
            case '2': // operador especifico
                operador.disabled = false;
                operador.addValidation('required', {});
                equipe.disabled = true;
                equipe.value = null;
                equipe.removeValidation('required');
                break;
            case '3': // equipe
                operador.value = null;
                operador.disabled = true;
                operador.removeValidation('required');
                equipe.disabled = false;
                equipe.addValidation('required', {});
                break;
            default:
                break;
        }
    }

    private async setEquipeModalInformacoes(row: RowTarefas) {
        const equipe = Metadata.getInstance<Select>('selectInformacoesEquipe');
        await equipe.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(equipe.name, row.NREQUIPE);
    }

    private changeFuncionarioModalInformacoes(row: RowTarefas): void {
        const funcionario = Metadata.getInstance<Select>('selectInformacoesFuncionario1');
        funcionario.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(funcionario.name, row.CDOPERADOR);
    }

    public changeEncarregado({ component }: any) {
        const encarregado = Metadata.getInstance<Select>('selectInformacoesEncarregado1');

        encarregado.datasource.setFilter({
            CODIGO: component.value,
            NRLOCALCLIENTE: Metadata.getInstance<Select>('informacoesLocalCli').value,
            RESP: (component.name === 'selectInformacoesEquipe') ? 'EQUIPE' : 'OPERADOR',
        });
    }

    private changeContratosModalInformacoes(row: RowTarefas): void {
        const selectContratos = Metadata.getInstance<Select>('informacoes-NRSEQCONTRATO');
        selectContratos.datasource.setFilter({
            NRESTRUTURAM: row.NRESTRUTURAM,
        });
        this.setValueSelectCamposModal<Select>(selectContratos.name, row.NRSEQCONTRATO);
    }

    private async changeAtividadeModalInformacoes(row: RowTarefas) {
        const atividade = Metadata.getInstance<SelectMultiple>('informacoesAtividadeTarefa');
        const arrayInteiros: number[] = (row.NRTAREFACHK as unknown as string[]).map((num: string): number => parseInt(num, 10));

        await atividade.datasource.setFilter({
            NRATIVIDADECHK: row.NRATIVIDADECHK,
            NRESCOPOTAREFA: [row.NRESCOPOSTAREFA],
        });
        this.setValueSelectCamposModal<SelectMultiple>(atividade.name, arrayInteiros);
    }

    private async changeEscopoModalInformacoes(row: RowTarefas) {
        const escopo = Metadata.getInstance<SelectMultiple>('informacoesEscopoTarefa');
        await escopo.datasource.setFilter({
            NRATIVIDADECHK: row.NRATIVIDADECHK,
        });
        this.setValueSelectCamposModal<Select>(escopo.name, row.NRESCOPOSTAREFA);
    }

    private async changeItensChecklistModalInformacoes(row: RowTarefas) {
        const itenschecklist = Metadata.getInstance<SelectMultiple>('informacoesItensChecklist');
        const arrayInteiros: number[] = (row.NRATIVIDADECHK as unknown as string[]).map((num: string): number => parseInt(num, 10));

        await itenschecklist.datasource.setFilter({
            NRCHECKLIST: row.NRCHECKLIST,
        });
        itenschecklist.setValue(arrayInteiros, false);
    }

    private changeCampoLocalModalInformacoes(row: RowTarefas): void {
        const local = Metadata.getInstance<Select>('informacoesLocalCli');
        local.datasource.setFilter({
            NRTIPOLOCALCLI: row.NRTIPOLOCALCLI,
            UNICA: 'S',
            CDCLIENTE: row.CDCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(local.name, row.NRLOCALCLIENTE);
    }

    private async changeCampoChecklist(row: RowTarefas) {
        const checklist = Metadata.getInstance<Select>('informacoesNRCHECKLIST');
        await checklist.datasource.setFilter({
            NRTIPOLOCALCLI: row.NRTIPOLOCALCLI,
            UNICA: 'S',
        });
        checklist.value = row.NRCHECKLIST;
    }

    private setObervacoesObrigatorias(row: RowTarefas): void {
        const observacoes = Metadata.getInstance<Select>('selectInformacoesObservacoesObrigatorias');

        if (row.IDOBRIGAOBSERVACAO === 'N') {
            this.setValueSelectCamposModal<Select>(observacoes.name, '1');
        } else {
            this.setValueSelectCamposModal<Select>(observacoes.name, '2');
        }
    }

    private changeCampoTipoDeLocal(row: RowTarefas): void {
        const tipoLocal = Metadata.getInstance<Select>('informacoesTipoLocalCli');
        tipoLocal.datasource.setFilter({
            CDCLIENTE: row.CDCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(tipoLocal.name, row.NRTIPOLOCALCLI);
    }

    public editarTarefa() {
        const form = Metadata.getInstance<Form>('formVisualizacaoTarefasModal');
        const formAgendamento = Metadata.getInstance<Form>('form-dadosAgendamento');
        const responsaveis = Metadata.getInstance<Select>('responsaveis-tarefa').value;
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-tarefas');

        this.validaParametrizacao();

        if (Utilities.validateForm(form) && Utilities.validateForm(formAgendamento)) {
            Http.post('salvarEdicaoTarefas', {
                NRCHECKLISTAGEND: Metadata.getInstance<TextInput>('codigo-tarefa').value,
                NMCHECKLISTAGEND: Metadata.getInstance<TextInput>('informacoesNomeTarefa').value,
                DTHORAAGEND: Metadata.getInstance<TextInput>('Informacoes-DTHORAAGEND').value,
                HORAAGEND: Metadata.getInstance<Time>('selectInformacoesHoraInicio1').value,
                DURACAO: Metadata.getInstance<Time>('selectInformacoesdDuracao1').value,
                IDOBRIGAOBSERVACAO: Metadata.getInstance<Select>('selectInformacoesObservacoesObrigatorias').value,
                IDTIPOMARCACAOTAREFA: Metadata.getInstance<Select>('selectInformacoesOpcoesMarcacao').value,
                CDOPERADOR: (responsaveis === '2') ? Metadata.getInstance<Select>('selectInformacoesFuncionario1').value : null,
                NREQUIPE: (responsaveis === '3') ? Metadata.getInstance<Select>('selectInformacoesEquipe').value : null,
                NREQUIPETAREFA: Metadata.getInstance<TextInput>('nrequipetarefa').value,
                IDTAREFAEQUIPE: Metadata.getInstance<TextInput>('idtarefaequipe').value,
                IDCRIAVISTORA: Metadata.getInstance<TextInput>('selectInformacoesCriarVistoria').value,
                ENCARREGADO: Metadata.getInstance<Select>('selectInformacoesEncarregado1').value,
                NRGERENCIAMENTOCHK: Metadata.getInstance<Select>('nrgerenciamentochk').value,
                NRPRIORIDADETAREFA: Metadata.getInstance<Select>('select-Informacoes-prioridade').value,
                NREPI: Metadata.getInstance<Select>('select-epi-info').value,
                NRMODELOPOP: Metadata.getInstance<Select>('select-modelopop-info').value,
                NRRELCHKAGENDMODPOP: Metadata.getInstance<Select>('NRRELCHKAGENDMODPOP').value,
                DSOBSERVACAO: Metadata.getInstance<Select>('campo-observacao').value,
                PARAMETRIZACAO: this.parametrizacao,
            }).then(() => {
                Utilities.showSuccessMessage();
                gridPrincipal.datasource.get();
                this.fecharModalInformacoesTarefas();
            }).catch((error: any) => {
                if (error && error.response && error.response.data && error.response.data.error && error.response.data.error.includes('Undefined index:')) {
                    let [erroCampo] = error.response.data.error.split('\'').filter((elemento: any) => elemento.includes('Undefined index:'));
                    erroCampo = erroCampo ? erroCampo.replaceAll('\'', '').replace('Undefined index: ', '') : erroCampo;
                    if (!erroCampo) {
                        Utilities.showErrorMessage(error);
                    } else {
                        Utilities.showDialog('Erro', 'error', 'Verifique se todos os campos obrigatórios foram preenchidos corretamente.');
                    }
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }
}
