/* eslint-disable */
import {
    Http, IDictionary, IEvent, IEventParam, Loader, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { Button, CheckboxMultiple, FileInput, Form, IterableComponentRender, Loading, LoadingService, Modal, ModalService, Row, Text, Textarea } from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import { TekGrid, TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';
import { IterableCarousel } from '@zeedhi/teknisa-components-common';
import FileUpload from '../FileUpload';

export class RelatorioAuditoriasController {
    public visibilidadeTemplate: boolean = true;

    private modal!: Modal;

    public currentRow: any;

    public textArea: boolean = true;

    public checkbox: boolean = true;

    public botaoAnexar: boolean = true;

    public btnSalvarRascunho: boolean = false;

    public btnFinalizarAuditoria: boolean = false;

    public btnApagarImg: boolean = true;

    public btnMicrofone: boolean = true;

    get visibilidadeDoFrame() {
        return !this.visibilidadeTemplate;
    }

    public async abrirModal() {
        LoadingService.show();
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00028_000_relatorios/modal/visualizar_infos_relatorio_auditorias' });
        this.modal.show();
        LoadingService.hide();
    }

    public closeModal() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public setFiltro({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    public getTopicos({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.setFilter({
            NRCATEGORIAAUDIT: component.name.split('_')[1],
            NRAUDITORIA: this.currentRow.NRAUDITORIA,
        });
    }

    public getCriterios({ component }: IEventParam<TekIterableComponentRender>) {
        component.datasource.setFilter({
            NRTOPICOAUDIT: component.name.split('_')[1],
            NRCATEGORIAAUDIT: component.name.split('_')[2],
            NRAUDITORIA: this.currentRow.NRAUDITORIA,
        });
    }

    public validationsTextArea({ component }: IEventParam<Textarea>) {
        if (this.currentRow.IDOBRIGAOBSERVACAO === 'S') {
            component.addValidation('required', {});
        } else {
            component.removeValidation('required');
        }
    }

    public exibirImagemAnexada({ component }: any) {
        const carousel = Metadata.getInstance<IterableCarousel>(`carousel-imagens_${component.name.split('_')[1]}`);
        carousel.iterable.datasource.updateData([]);
        const imagens: any = [];

        if (component.value.length) {
            carousel.isVisible = true;
            component.value.forEach((elemento: any, posicao: any) => {
                imagens.push({
                    id: posicao,
                    src: window.URL.createObjectURL(elemento),
                });
            });
        } else {
            carousel.isVisible = false;
        }
        carousel.iterable.datasource.updateData(imagens);
    }

    public async anexarImagens({ component }: IEventParam<Button>) {
        const imagensConvertidas = await FileUpload.converterImagens(`file-input-anexar-imagem_${component.name.split('_')[1]}`);
        const fileInput = Metadata.getInstance<FileInput>(`file-input-anexar-imagem_${component.name.split('_')[1]}`);
        const iterableImagensAnexadas = Metadata.getInstance<IterableCarousel>(`carousel-imagens-anexadas_${component.name.split('_')[1]}_${component.name.split('_')[2]}`);

        LoadingService.show();
        if (fileInput.value) {
            Http.post('/anexarImagemAuditoriaRelatorio', {
                IMAGENS: imagensConvertidas,
                NRAUDITORIA: this.currentRow.NRAUDITORIA,
                NRCATEGORIAAUDIT: component.name.split('_')[2],
                NRTOPICOAUDIT: component.name.split('_')[1],
            }).then(() => {
                Utilities.showSuccessMessage();
                this.fecharAnexoImagem({ component });
                iterableImagensAnexadas.iterable.datasource.get();
                if (this.currentRow.IDSITUACAOAUDIT === '1') {
                    this.btnSalvarRascunho = false;
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Nenhuma imagem foi anexada', 'error');
        }
    }

    public abrirAnexoImagem({ component }: IEventParam<Button>) {
        component.disabled = true;
        Metadata.getInstance<IterableCarousel>(`carousel-imagens_relatorio_${component.name.split('_')[1]}`).iterable.datasource.updateData([]);
        Metadata.getInstance<FileInput>(`file-input-anexar-relatorio=imagem_${component.name.split('_')[1]}`).value = null;

        const form = Metadata.getInstance<Form>(`form-anexar-relatorio-fotos_${component.name.split('_')[1]}`);
        const row = Metadata.getInstance<Row>(`row-btn-can-relatorio-ane_${component.name.split('_')[1]}`);
        const iterableImgAnexadas = Metadata.getInstance<IterableCarousel>(`carousel-imagens-anexadas_${component.name.split('_')[1]}_${component.name.split('_')[2]}`);

        form.isVisible = true;
        row.isVisible = true;
        iterableImgAnexadas.isVisible = false;
    }

    public fecharAnexoImagem({ component }: IEventParam<Button>) {
        this.botaoAnexar = false;
        const form = Metadata.getInstance<Form>(`form-anexar-fotos_${component.name.split('_')[1]}`);
        const row = Metadata.getInstance<Row>(`row-btn-can-ane_${component.name.split('_')[1]}`);
        const iterableImgAnexadas = Metadata.getInstance<IterableCarousel>(`carousel-imagens-anexadas_${component.name.split('_')[1]}_${component.name.split('_')[2]}`);
        const iterableImgs = Metadata.getInstance<IterableCarousel>(`carousel-imagens_${component.name.split('_')[1]}`);

        form.isVisible = false;
        row.isVisible = false;
        iterableImgAnexadas.isVisible = true;
        iterableImgs.isVisible = false;
    }

    public onMountedImagensAnexadas({ component }: IEventParam<IterableCarousel>) {
        component.iterable.datasource.setFilter({
            NRAUDITORIA: this.currentRow.NRAUDITORIA,
            NRCATEGORIAAUDIT: component.name.split('_')[2],
            NRTOPICOAUDIT: component.name.split('_')[1],
        });
    }

    public rowStyleConditions(row: any) {
        if (row.ATRASO !== null) {
            if (row.ATRASO.split(' ')[1] === 'dias') {
                if (row.ATRASO.split(' ')[0] >= 1) {
                    return { cssClass: 'linha-grid-verificada' };
                }
            }
        }

        return { cssClass: '' };
    }

    private clicks: any = 0;


    public async rowClick({ row }: any) {
        this.currentRow = row;
        this.clicks += 1;
        if (this.clicks >= 2) {
           await this.abrirModal();
            await this.onMountedCategorias({ row });
            await this.onMountedValidacoes({ row });
            this.clicks = 0;
        }
        setTimeout(() => {
            this.clicks = 0;
        }, 500);
    }

    public onMountedCategorias({ row }: any) {
        Metadata.getInstance<TekIterableComponentRender>('iterable-categorias').datasource.setFilter({
            NRAUDITORIA: row.NRAUDITORIA,
        });
    }

    public onMountedValidacoes({ row }: any) {
        if (row.IDSITUACAOAUDIT === '1') {
            this.btnFinalizarAuditoria = true;
            this.btnSalvarRascunho = true;
            this.botaoAnexar = false;
            this.textArea = false;
            this.checkbox = false;
            this.btnApagarImg = false;
            this.btnMicrofone = false;
        } else if (row.IDSITUACAOAUDIT === '2') {
            this.btnFinalizarAuditoria = false;
            this.btnSalvarRascunho = false;
            this.botaoAnexar = false;
            this.textArea = false;
            this.checkbox = false;
            this.btnApagarImg = true;
            this.btnMicrofone = false;
        } else {
            this.btnFinalizarAuditoria = true;
            this.btnSalvarRascunho = true;
            this.botaoAnexar = true;
            this.textArea = true;
            this.checkbox = true;
            this.btnApagarImg = false;
            this.btnMicrofone = true;
        }
    }
}
