import { Utilities } from '@/util/Utilities';
import { DialogService, LoadingService, TextInput } from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00040002TopicosAuditoriaController {
    public parametrizacaoNota: Boolean = false;

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRTOPICOAUDIT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>('NMTOPICOAUDIT_editable_0');
            nameInput.setFocus();
        }, 100);
    }

    /* eslint-disable */
    public canEditRow({ row }: any) {
        return row['__added_row'] === true;
    }
    /* eslint-enable */

    public salvarTopicosAuditoria({ component, event }: any) {
        event.preventDefault();
        const topicos = component.getEditedRows();

        if (topicos.length > 0) {
            if (topicos[0].NMTOPICOAUDIT != null) {
                LoadingService.show();
                Http.post('/salvarTopicosAuditoria', topicos).then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Nome"', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Nome"!', 'error');
        }
    }

    public excluirTopicos({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040002TopicosAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-topicos');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            TOPICOS: component.selectedRows.map((element: any) => element.NRTOPICOAUDIT),
        }).then((response: IDictionary) => {
            if (response.data !== 0) {
                Utilities.showDialog('Operação Bloqueada', 'warning', 'Alguns tópicos não puderam ser excluídos pois estão sendo utilizados em algum Modelo de Auditoria.');
            }
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public getParametrizacao() {
        Http.get('/getParametrizacaoCriterios').then((response: any) => {
            this.parametrizacaoNota = (response.data[0].IDUTILIZANOTAAUDIT === 'S');
        });
    }
}
