import { Utilities } from '@/util/Utilities';
import {
     Button, DateRange, Frame, Modal, ModalService,
     Tabs,
    } from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { DashboardGrid, IDashboardGridItem } from '@zeedhi/zd-dashboard-grid-common';
import { ECharts } from '@zeedhi/zd-echarts';

export class Fm000000DashboardFacilitiesController {
    public gridTitle: String = '';

    public gridSubtitle: String = '';

    public gridRoute: String = '';

    public gridErrorSlot: String = '';

    public gridColStatusName: String = '';

    public gridColStatusLabel: Boolean = false;

    public gridTarefas!: Modal;

    public modalFiltro!: Modal;

    public modalVisibilidade!: Modal;

    public filtroRangeInicio: string = '';

    public filtroRangeFinal: string = '';

    public queryStringFiltro: string = `?DTRANGEINICIO=${this.filtroRangeInicio}&DTRANGEFINAL=${this.filtroRangeFinal}`;

    public textFooterRange: string = 'Filtro de Tempo: Tempo todo';

    public titleGrid: string = 'Facilities | Tempo Todo';

    public async criarModais() {
        this.modalFiltro = await ModalService.createFromJsonPost('', { jsonPath: 'fm_0000_000_dashboard_facilities/modal/fm_0000_000_filtro' });
        this.modalVisibilidade = await ModalService.createFromJsonPost('', { jsonPath: 'fm_0000_000_dashboard_facilities/modal/fm_0000_000_visibilidade' });
    }

    public async abrirModalVisibilidade() {
        await this.modalVisibilidade.show();
        await this.loadHiddenCards();
    }

    public fecharModalVisibilidade() {
        this.modalVisibilidade.hide();
    }

    public montarFooterRange() {
        if (this.filtroRangeFinal === '' || this.filtroRangeInicio === '') {
            this.textFooterRange = 'Filtro de Tempo: Tempo todo';
            this.titleGrid = 'Facilities | Tempo Todo';
            return;
        }
        this.textFooterRange = `Filtro de Tempo: Mostrando de ${this.filtroRangeInicio} a ${this.filtroRangeFinal}`;
        this.titleGrid = `Facilities | ${this.filtroRangeInicio}     -     ${this.filtroRangeFinal}`;
    }

    public async abrirModalFiltro() {
        this.modalFiltro.show();
    }

    public fecharModalFiltro() {
        this.modalFiltro.hide();
    }

    public async abrirGridTarefas() {
        this.gridTarefas = await ModalService.createFromJsonPost('', { jsonPath: 'fm_0000_000_dashboard_facilities/modal/fm_0000_000_grid' });
        await this.gridTarefas.show();
        const grid = await Metadata.getInstance<TekGrid>('grid-Tarefas');
        grid.datasource.get();
    }

    public fecharGridTarefas() {
        this.gridTarefas.destroy();
    }

    public getTarefasGrafico() {
        Promise.all([
            Http.get(`/getDadosTarefaConcluidaF${this.queryStringFiltro}`),
            Http.get(`/getDadosTarefaAtrasadaF${this.queryStringFiltro}`),
            Http.get(`/getDadosLimpezaFinalizadaF${this.queryStringFiltro}`),
            Http.get(`/getDadosTarefaEmAndamento${this.queryStringFiltro}`),
            Http.get(`/getDadosTarefaPendente${this.queryStringFiltro}`),
            Http.get(`/getDadosTarefaCancelado${this.queryStringFiltro}`),
            Http.get(`/getDadosTarefaPendenteAprovacao${this.queryStringFiltro}`),
        ]).then((responses) => {
            const [concluidas, atrasadas, finalizadas, emAndamento, pendente, cancelado, pendenteAprovacao] = responses.map((response) => response.data.data.length);

            const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_tarefa');
            const NewOptions = { ...grafico.options };

            NewOptions.series[0].data.push(
                { name: 'Finalizadas', value: finalizadas, itemStyle: { color: 'rgba(0, 119, 182, 1)' } },
                { name: 'Atrasadas', value: atrasadas, itemStyle: { color: 'rgba(255, 0, 0, 1)' } },
                { name: 'Concluidas', value: concluidas, itemStyle: { color: 'rgba(145, 204, 117, 1)' } },
                { name: 'Em Andamento', value: emAndamento },
                { name: 'Pendente', value: pendente, itemStyle: { color: 'rgba(234, 124, 204, 1)' } },
                { name: 'Cancelado', value: cancelado, itemStyle: { color: 'rgba(250, 200, 88, 1)' } },
                { name: 'Pendentes de Aprovação', value: pendenteAprovacao, itemStyle: { color: 'rgba(180, 149, 96, 1)' } },
            );
            this.disableLegenda('tarefas', '4');
            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public rowClick({ param, component }: IEventParam<ECharts>) {
        let nome;
        let filtroTipo = '';
        let index;
        let nomeGrid = '';
        if (component.options.xAxis.nr) {
            nome = param.seriesName;
            index = param.dataIndex;
            filtroTipo = `&${component.options.xAxis.nr[index]}`;
            nomeGrid = `- ${param.name}` || '';
        } else {
            nome = param.name;
        }
        switch (nome) {
            case 'Finalizadas':
                this.gridTitle = `Tarefas Finalizadas ${nomeGrid}`;
                this.gridRoute = `/getDadosLimpezaFinalizadaF${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas finalizadas';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            case 'Finalizadas (aguardando vistoria)':
                this.gridTitle = `Tarefas Finalizadas ${nomeGrid}`;
                this.gridRoute = `/getDadosLimpezaFinalizadaF${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas finalizadas';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            case 'Atrasadas':
                this.gridTitle = `Tarefas Atrasadas ${nomeGrid}`;
                this.gridRoute = `/getDadosTarefaAtrasadaF${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas Atrasadas';
                this.gridColStatusLabel = true;
                this.abrirGridTarefas();
                break;
            case 'Concluidas':
                this.gridTitle = `Tarefas Concluidas ${nomeGrid}`;
                this.gridRoute = `/getDadosTarefaConcluidaF${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas Concluidas';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            case 'Em Andamento':
                this.gridTitle = `Tarefas Em Andamento ${nomeGrid}`;
                this.gridRoute = `/getDadosTarefaEmAndamento${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas Em Andamento';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            case 'Pendente':
                this.gridTitle = `Tarefas Pendentes ${nomeGrid}`;
                this.gridRoute = `/getDadosTarefaPendente${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas Pendentes';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            case 'Cancelado':
                this.gridTitle = `Tarefas Canceladas ${nomeGrid}`;
                this.gridRoute = `/getDadosTarefaCancelado${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas Canceladas';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            case 'Pendentes de Aprovação':
                this.gridTitle = `Tarefas Pendentes de Aprovação ${nomeGrid}`;
                this.gridRoute = `/getDadosTarefaPendenteAprovacao${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlot = 'Não há tarefas Pendentes de Aprovação';
                this.gridColStatusLabel = false;
                this.abrirGridTarefas();
                break;
            default:
                Utilities.showErrorMessage('Erro ao abrir grid de tarefas');
                break;
        }
    }

    public disableLegendaLista() {
        this.disableLegenda('vistorias', '7');
        this.disablePrint('vistorias', '7');
        this.disableLegenda('tarefas', '9');
        this.disablePrint('tarefas', '9');
    }

    // Card tarefas por tipo local
    public getTarefasPorLocal() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_tarefa_local');
        const NewOptions = { ...grafico.options };
        Http.get(`/getTarefasPorLocal${this.queryStringFiltro}`).then((response: any) => {
            const locais = response.data.data;
            locais.forEach((local: any) => {
                NewOptions.xAxis.data.push(local.NMLOCALCLIENTE);
                NewOptions.xAxis.nr.push(`NRLOCALCLIENTE=${local.NRLOCALCLIENTE}`);

                NewOptions.series[0].data.push(local.TOTAL_CONCLUIDAS);
                NewOptions.series[1].data.push(local.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(local.TOTAL_FINALIZADAS);
                NewOptions.series[3].data.push(local.TOTAL_ANDAMENTO);
                NewOptions.series[4].data.push(local.TOTAL_PENDENTES);
                NewOptions.series[5].data.push(local.TOTAL_CANCELADOS);
                NewOptions.series[6].data.push(local.TOTAL_PENDENTESAPROVACAO);
            });
            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    // Card tarefas por Checklist
    public getTarefasPorChecklist() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_tarefa_checklist');
        const NewOptions = { ...grafico.options };
        Http.get(`/getTarefasPorChecklist${this.queryStringFiltro}`).then((response: any) => {
            const Checklists = response.data.data;

            Checklists.forEach((Checklist: any) => {
                NewOptions.xAxis.data.push(Checklist.NMCHECKLIST);
                NewOptions.xAxis.nr.push(`NRCHECKLIST=${Checklist.NRCHECKLIST}`);

                NewOptions.series[0].data.push(Checklist.TOTAL_CONCLUIDAS);
                NewOptions.series[1].data.push(Checklist.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(Checklist.TOTAL_FINALIZADAS);
                NewOptions.series[3].data.push(Checklist.TOTAL_ANDAMENTO);
                NewOptions.series[4].data.push(Checklist.TOTAL_PENDENTES);
                NewOptions.series[5].data.push(Checklist.TOTAL_CANCELADOS);
                NewOptions.series[6].data.push(Checklist.TOTAL_PENDENTESAPROVACAO);
            });
            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    // card tarefas por cliente

    public getTarefasPorCliente() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_tarefa_clientes');
        const NewOptions = { ...grafico.options };
        Http.get(`/getTarefasPorCliente${this.queryStringFiltro}`).then((response: any) => {
            const clientes = response.data.data;

            clientes.forEach((cliente: any) => {
                NewOptions.xAxis.data.push(cliente.NMFANTCLIE);
                NewOptions.xAxis.nr.push(`CDCLIENTE=${cliente.CDCLIENTE}`);

                NewOptions.series[0].data.push(cliente.TOTAL_CONCLUIDAS);
                NewOptions.series[1].data.push(cliente.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(cliente.TOTAL_FINALIZADAS);
                NewOptions.series[3].data.push(cliente.TOTAL_ANDAMENTO);
                NewOptions.series[4].data.push(cliente.TOTAL_PENDENTES);
                NewOptions.series[5].data.push(cliente.TOTAL_CANCELADOS);
                NewOptions.series[6].data.push(cliente.TOTAL_PENDENTESAPROVACAO);
            });

            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public aplicarFiltro() {
        const dateRange = Metadata.getInstance<DateRange>('periodo-busca');
        const [filtroRangeInicio, filtroRangeFinal] = dateRange.value;

        if (this.filtroRangeFinal === filtroRangeFinal && this.filtroRangeInicio === filtroRangeInicio) {
            this.fecharModalFiltro();
            return;
        }

        this.filtroRangeInicio = filtroRangeInicio || '';
        this.filtroRangeFinal = filtroRangeFinal || '';

        this.queryStringFiltro = `?DTRANGEINICIO=${this.filtroRangeInicio}&DTRANGEFINAL=${this.filtroRangeFinal}`;

        this.montarFooterRange();
        this.fecharModalFiltro();
        this.reloadAllDashs();
    }

    public reloadAllDashs() {
        const dashTarefas = Metadata.getInstance<Frame>('frame-dash-tarefas');
        dashTarefas.reload();

        const dashVistorias = Metadata.getInstance<Frame>('frame-dash-vistorias');
        dashVistorias.reload();

        const dashAuditorias = Metadata.getInstance<Frame>('frame-dash-auditorias');
        dashAuditorias.reload();
    }

    // DASHBOARD VISTORIAS

    public gridTitleVistorias: string = '';

    public gridSubtitleVistorias: string = '';

    public gridRouteVistorias: string = '';

    public gridErrorSlotVistorias: string = '';

    public gridVistorias!: Modal;

    public gridVistoriasAtraso: Boolean = false;

    public getVistoriasGrafico() {
        Promise.all([
            Http.get(`/getVistoriasPendentes${this.queryStringFiltro}`),
            Http.get(`/getVistoriasAtrasadas${this.queryStringFiltro}`),
            Http.get(`/getVistoriasAvaliadas${this.queryStringFiltro}`),
            Http.get(`/getVistoriasEmAndamento${this.queryStringFiltro}`),
            Http.get(`/getVistoriasCancelado${this.queryStringFiltro}`),
        ]).then((responses) => {
            const [pendentes, atrasadas, avaliadas, emAndamento, cancelado] = responses.map((response) => response.data.data.length);

            const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_vistoria');
            const NewOptions = { ...grafico.options };

            NewOptions.series[0].data.push(
                { name: 'Pendentes', value: pendentes, itemStyle: { color: 'rgba(234, 124, 204, 1)' } },
                { name: 'Atrasadas', value: atrasadas, itemStyle: { color: 'rgba(255, 0, 0, 1)' } },
                { name: 'Avaliadas', value: avaliadas, itemStyle: { color: 'rgba(145, 204, 117, 1)' } },
                { name: 'Em Andamento', value: emAndamento, itemStyle: { color: 'rgba(0, 119, 182, 1)' } },
                { name: 'Canceladas', value: cancelado, itemStyle: { color: 'rgba(250, 200, 88, 1)' } },
            );
            this.disableLegenda('vistorias', '1');
            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public chartClick({ param, component }: IEventParam<ECharts>) {
        let nome;
        let filtroTipo = '';
        let index;
        let nomeGrid = '';
        if (component.options.xAxis.nr) {
            nome = param.seriesName;
            index = param.dataIndex;
            filtroTipo = `&${component.options.xAxis.nr[index]}`;
            nomeGrid = `- ${param.name}` || '';
        } else {
            nome = param.name;
        }
        switch (nome) {
            case 'Pendentes':
                this.gridTitleVistorias = `Vistorias Pendentes ${nomeGrid}`;
                this.gridRouteVistorias = `/getVistoriasPendentes${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotVistorias = 'Não há Vistorias Pendentes';
                this.gridVistoriasAtraso = false;
                this.abrirGridVistorias();
                break;
            case 'Atrasadas':
                this.gridTitleVistorias = `Vistorias Atrasadas ${nomeGrid}`;
                this.gridRouteVistorias = `/getVistoriasAtrasadas${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotVistorias = 'Não há Vistorias Atrasadas';
                this.gridVistoriasAtraso = true;
                this.abrirGridVistorias();
                break;
            case 'Avaliadas':
                this.gridTitleVistorias = `Vistorias Avaliadas ${nomeGrid}`;
                this.gridRouteVistorias = `/getVistoriasAvaliadas${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotVistorias = 'Não há Vistorias Avaliadas';
                this.gridVistoriasAtraso = false;
                this.abrirGridVistorias();
                break;
            case 'Em Andamento':
                this.gridTitleVistorias = `Vistorias Em Andamento ${nomeGrid}`;
                this.gridRouteVistorias = `/getVistoriasEmAndamento${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotVistorias = 'Não há Vistorias Em Andamento';
                this.gridVistoriasAtraso = false;
                this.abrirGridVistorias();
                break;
            case 'Canceladas':
                this.gridTitleVistorias = `Vistorias Canceladas ${nomeGrid}`;
                this.gridRouteVistorias = `/getVistoriasCancelado${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotVistorias = 'Não há Vistorias Canceladas';
                this.gridVistoriasAtraso = false;
                this.abrirGridVistorias();
                break;
            default:
                Utilities.showErrorMessage('Erro ao abrir grid vistorias');
                break;
        }
    }

    public async abrirGridVistorias() {
        this.gridVistorias = await ModalService.createFromJsonPost('', { jsonPath: 'fm_0000_000_dashboard_facilities/modal/fm_0000_000_grid_vistorias' });
        this.gridVistorias.show();
    }

    public fecharGridVistorias() {
        this.gridVistorias.destroy();
    }

    // card vistorias por cliente
    public getVistoriasPorCliente() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_vistoria_cliente');
        const NewOptions = { ...grafico.options };
        Http.get(`/getVistoriasPorCliente${this.queryStringFiltro}`).then((response: any) => {
            const clientes = response.data.data;

            clientes.forEach((cliente: any) => {
                NewOptions.xAxis.data.push(cliente.NMFANTCLIE);
                NewOptions.xAxis.nr.push(`CDCLIENTE=${cliente.CDCLIENTE}`);

                NewOptions.series[0].data.push(cliente.TOTAL_PENDENTES);
                NewOptions.series[1].data.push(cliente.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(cliente.TOTAL_AVALIADAS);
                NewOptions.series[3].data.push(cliente.TOTAL_ANDAMENTO);
                NewOptions.series[4].data.push(cliente.TOTAL_CANCELADO);
            });

            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    // Card vistorias por local
    public getVistoriasPorLocal() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_vistoria_local');
        const NewOptions = { ...grafico.options };
        Http.get(`/getVistoriasPorLocal${this.queryStringFiltro}`).then((response: any) => {
            const locais = response.data.data;

            locais.forEach((local: any) => {
                NewOptions.xAxis.data.push(local.NMLOCALCLIENTE);
                NewOptions.xAxis.nr.push(`NRLOCALCLIENTE=${local.NRLOCALCLIENTE}`);

                NewOptions.series[0].data.push(local.TOTAL_PENDENTES);
                NewOptions.series[1].data.push(local.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(local.TOTAL_AVALIADAS);
                NewOptions.series[3].data.push(local.TOTAL_ANDAMENTO);
                NewOptions.series[4].data.push(local.TOTAL_CANCELADO);
            });
            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    // Card vistorias por Checklist
    public getVistoriasPorChecklist() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_vistoria_checklist');
        const NewOptions = { ...grafico.options };
        Http.get(`/getVistoriasPorChecklist${this.queryStringFiltro}`).then((response: any) => {
            const Checklists = response.data.data;

            Checklists.forEach((Checklist: any) => {
                NewOptions.xAxis.data.push(Checklist.NMCHECKLIST);
                NewOptions.xAxis.nr.push(`NRCHECKLIST=${Checklist.NRCHECKLIST}`);

                NewOptions.series[0].data.push(Checklist.TOTAL_PENDENTES);
                NewOptions.series[1].data.push(Checklist.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(Checklist.TOTAL_AVALIADAS);
                NewOptions.series[3].data.push(Checklist.TOTAL_ANDAMENTO);
                NewOptions.series[4].data.push(Checklist.TOTAL_CANCELADO);
            });

            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    // AUDITORIAS

    public gridTitleAuditorias: string = '';

    public gridSubtitleAuditorias: string = '';

    public gridRouteAuditorias: string = '';

    public gridErrorSlotAuditorias: string = '';

    public gridAuditorias!: Modal;

    public gridAuditoriasAtraso = false;

    public async abrirGridAuditorias() {
        this.gridAuditorias = await ModalService.createFromJsonPost('', { jsonPath: 'fm_0000_000_dashboard_facilities/modal/fm_0000_000_grid_auditorias' });
        this.gridAuditorias.show();
    }

    public fecharGridAuditorias() {
        this.gridAuditorias.destroy();
    }

    // Card Nao Conformidades por tipo local

    public getNaoConformidadesTipoLoc() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_auditoria_nao_conformidades_local');
        const NewOptions = { ...grafico.options };

        Http.get(`/getNaoConformidadesPorLocal${this.queryStringFiltro}`).then((response: any) => {
            const naoConformidades = response.data.data;

            naoConformidades.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOLOCALCLI,
                    value: element.QTD,
                });
            });
            grafico.updateChart(NewOptions);
            this.disableLegenda('vistorias', '5');
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    // Card nao conformidades totais

    public getNaoConformidades() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_auditoria_nao_conformidades');
        const NewOptions = { ...grafico.options };

        Http.get(`/getNaoConformidades${this.queryStringFiltro}`).then((response: any) => {
            const naoConformidades = response.data.data;

            naoConformidades.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMCRITERIOTARGERCHK,
                    value: element.QTDADE,
                });
            });
            grafico.updateChart(NewOptions);
            this.disableLegenda('vistorias', '6');
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getAuditoriasGrafico() {
        Promise.all([
            Http.get(`/getAuditoriasConcluidas${this.queryStringFiltro}`),
            Http.get(`/getAuditoriasAtrasadas${this.queryStringFiltro}`),
            Http.get(`/getAuditoriasEmAndamento${this.queryStringFiltro}`),
            Http.get(`/getAuditoriasPendentes${this.queryStringFiltro}`),
        ]).then((responses) => {
            const [concluidas, atrasadas, emAndamento, pendentes] = responses.map((response) => (response.data.data && response.data.data.length) || 0);

            const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_auditoria');
            const NewOptions = { ...grafico.options };

            NewOptions.series[0].data.push(
                { name: 'Em Andamento', value: emAndamento, itemStyle: { color: 'rgba(0, 119, 182, 1)' } },
                { name: 'Atrasadas', value: atrasadas, itemStyle: { color: 'rgba(255, 0, 0, 1)' } },
                { name: 'Concluidas', value: concluidas, itemStyle: { color: 'rgba(145, 204, 117, 1)' } },
                { name: 'Pendentes', value: pendentes, itemStyle: { color: 'rgba(234, 124, 204, 1)' } },
            );
            this.disableLegenda('auditorias', '4');
            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getAuditoriasPorLocal() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_auditoria_local');
        const NewOptions = { ...grafico.options };
        Http.get(`/getAuditoriasPorLocal${this.queryStringFiltro}`).then((response: any) => {
            const locais = response.data.data;

            locais.forEach((Local: any) => {
                NewOptions.xAxis.data.push(Local.NMLOCALCLIENTE);
                NewOptions.xAxis.nr.push(`NRLOCALCLIENTE=${Local.NRLOCALCLIENTE}`);

                NewOptions.series[0].data.push(Local.TOTAL_CONCLUIDAS);
                NewOptions.series[1].data.push(Local.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(Local.TOTAL_ANDAMENTO);
                NewOptions.series[3].data.push(Local.TOTAL_PENDENTES);
            });

            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getAuditoriasPorCliente() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_auditoria_cliente');
        const NewOptions = { ...grafico.options };
        Http.get(`/getAuditoriasPorCliente${this.queryStringFiltro}`).then((response: any) => {
            const clientes = response.data.data;

            clientes.forEach((cliente: any) => {
                NewOptions.xAxis.data.push(cliente.NMFANTCLIE);
                NewOptions.xAxis.nr.push(`CDCLIENTE=${cliente.CDCLIENTE}`);

                NewOptions.series[0].data.push(cliente.TOTAL_CONCLUIDAS);
                NewOptions.series[1].data.push(cliente.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(cliente.TOTAL_ANDAMENTO);
                NewOptions.series[3].data.push(cliente.TOTAL_PENDENTES);
            });

            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public chartClickAudit({ param, component }: IEventParam<ECharts>) {
        let nome;
        let filtroTipo = '';
        let index;
        let nomeGrid = '';
        if (component.options.xAxis.nr) {
            nome = param.seriesName;
            index = param.dataIndex;
            filtroTipo = `&${component.options.xAxis.nr[index]}`;
            nomeGrid = `- ${param.name}` || '';
        } else {
            nome = param.name;
        }
        switch (nome) {
            case 'Em Andamento':
                this.gridTitleAuditorias = `Auditorias Em Andamento ${nomeGrid}`;
                this.gridRouteAuditorias = `/getAuditoriasEmAndamento${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotAuditorias = 'Não há Auditorias Em Andamento';
                this.gridAuditoriasAtraso = false;
                this.abrirGridAuditorias();
                break;
            case 'Atrasadas':
                this.gridTitleAuditorias = `Auditorias Atrasadas ${nomeGrid}`;
                this.gridRouteAuditorias = `/getAuditoriasAtrasadas${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotAuditorias = 'Não há Auditorias Atrasadas';
                this.gridAuditoriasAtraso = true;
                this.abrirGridAuditorias();
                break;
            case 'Concluidas':
                this.gridTitleAuditorias = `Auditorias Concluidas ${nomeGrid}`;
                this.gridRouteAuditorias = `/getAuditoriasConcluidas${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotAuditorias = 'Não há Auditorias Concluidas';
                this.gridAuditoriasAtraso = false;
                this.abrirGridAuditorias();
                break;
            case 'Pendentes':
                this.gridTitleAuditorias = `Auditorias Pendentes ${nomeGrid}`;
                this.gridRouteAuditorias = `/getAuditoriasPendentes${this.queryStringFiltro}${filtroTipo}`;
                this.gridErrorSlotAuditorias = 'Não há Auditorias Pendentes';
                this.gridAuditoriasAtraso = false;
                this.abrirGridAuditorias();
                break;
            default:
                Utilities.showErrorMessage('Erro ao abrir Grid com Auditorias');
                break;
        }
    }

    public getAuditoriasPorModelo() {
        const grafico = Metadata.getInstance<ECharts>('fm_card_burndown_auditoria_modelo');
        const NewOptions = { ...grafico.options };
        Http.get(`/getAuditoriasPorModelo${this.queryStringFiltro}`).then((response: any) => {
            const modelos = response.data.data;

            modelos.forEach((modelo: any) => {
                NewOptions.xAxis.data.push(modelo.NMMODELOAUDIT);
                NewOptions.xAxis.nr.push(`NRMODELOAUDIT=${modelo.NRMODELOAUDIT}`);

                NewOptions.series[0].data.push(modelo.TOTAL_CONCLUIDAS);
                NewOptions.series[1].data.push(modelo.TOTAL_ATRASADAS);
                NewOptions.series[2].data.push(modelo.TOTAL_ANDAMENTO);
                NewOptions.series[3].data.push(modelo.TOTAL_PENDENTES);
            });

            grafico.updateChart(NewOptions);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    // GERAL

    public async esconderCard({ component }: any): Promise<void> {
        const nome = component.name.split('-');
        const dashboardInstance = this.getDashboard(nome.length > 1 ? nome[nome.length - 2].trim() : '');

        if (!dashboardInstance) {
            Utilities.showErrorMessage('Erro ao Recuperar a instancia do dashboard');
            return;
        }

        const { cards } = dashboardInstance;
        cards.forEach((card: IDashboardGridItem): void => {
            if (card.i === dashboardInstance.currentCardId) {
                dashboardInstance.showToggleVisibilityCardDialog(card);
            }
        });
    }

    public getDashboard(nome: string) {
        switch (nome) {
            case 'vistorias':
                return Metadata.getInstance<DashboardGrid>('dashboard-vistorias');
            case 'tarefas':
                return Metadata.getInstance<DashboardGrid>('dashboard-tarefas');
            case 'auditorias':
                return Metadata.getInstance<DashboardGrid>('dashboard-auditorias');
            default:
                return null;
        }
    }

    public loadHiddenCards() {
        const hiddenCards = Metadata.getInstance<TekGrid>('grid-cards-escondidos');
        const cardsOcultosTarefas = localStorage.getItem('@ZdDashboardGrid/hideCards/dashboard-tarefas');
        const cardsOcultosVistorias = localStorage.getItem('@ZdDashboardGrid/hideCards/dashboard-vistorias');
        const cardsOcultosAuditorias = localStorage.getItem('@ZdDashboardGrid/hideCards/dashboard-auditorias');

        const tarefasData = cardsOcultosTarefas ? JSON.parse(cardsOcultosTarefas) : [];
        const vistoriasData = cardsOcultosVistorias ? JSON.parse(cardsOcultosVistorias) : [];
        const auditoriasData = cardsOcultosAuditorias ? JSON.parse(cardsOcultosAuditorias) : [];

        const cardsOcultos = [...tarefasData, ...vistoriasData, ...auditoriasData];

        hiddenCards.datasource.updateData(cardsOcultos);
    }

    public reloadDash() {
        const tabs = Metadata.getInstance<Tabs>('tabs-dashboard');

        if (tabs.activeTab === 0) {
            const frame = Metadata.getInstance<Frame>('frame-dash-tarefas');
            frame.reload();
        }

        if (tabs.activeTab === 1) {
            const frame = Metadata.getInstance<Frame>('frame-dash-vistorias');
            frame.reload();
        }

        if (tabs.activeTab === 2) {
            const frame = Metadata.getInstance<Frame>('frame-dash-auditorias');
            frame.reload();
        }
    }

    public restaurarCards() {
        const tabs = Metadata.getInstance<Tabs>('tabs-dashboard');

        if (tabs.activeTab === 0) {
            localStorage.removeItem('@ZdDashboardGrid/dashboard-tarefas');
            localStorage.removeItem('@ZdDashboardGrid/hideCards/dashboard-tarefas');
            this.reloadDash();
        }

        if (tabs.activeTab === 1) {
            localStorage.removeItem('@ZdDashboardGrid/dashboard-vistorias');
            localStorage.removeItem('@ZdDashboardGrid/hideCards/dashboard-vistorias');
            this.reloadDash();
        }

        if (tabs.activeTab === 2) {
            localStorage.removeItem('@ZdDashboardGrid/dashboard-auditorias');
            localStorage.removeItem('@ZdDashboardGrid/hideCards/dashboard-auditorias');
            this.reloadDash();
        }
    }

    public visibilidadeCard({ row }: any) {
        const nome: string = `${row.name.split('-').pop()?.trim().toLowerCase() ?? ''}s`;

        const dashboard = this.getDashboard(nome);
        if (!dashboard) {
            Utilities.showErrorMessage('Erro ao esconder Card');
            return;
        }

        dashboard.toggleVisibility(row);
        this.loadHiddenCards();
    }

    public posicaoLegenda({ component }: any) {
        const nome = component.name.split('-');
        const dashboardInstance = this.getDashboard(nome.length > 1 ? nome[nome.length - 2].trim() : '');

        if (dashboardInstance) {
            const pathComponent = dashboardInstance.getFrameProps(dashboardInstance.currentCardId!);
            const nameComponent = pathComponent.path?.substring(pathComponent.path.lastIndexOf('/') + 1);
            const chart = Metadata.getInstance<ECharts>(`${nameComponent}`);
            const NewOptions = { ...chart.options };

            if (component.name.split('-').slice(0, 2).join('-') === 'superior-esquerdo') {
                NewOptions.legend.right = '80%';
                NewOptions.legend.top = '5%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'superior-direito') {
                NewOptions.legend.right = '10%';
                NewOptions.legend.top = '5%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'inferior-esquerdo') {
                NewOptions.legend.right = '10%';
                NewOptions.legend.top = '80%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'inferior-direito') {
                NewOptions.legend.right = '10%';
                NewOptions.legend.top = '80%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'esconder-legendas') {
                NewOptions.legend.show = !(NewOptions.legend.show);
            }

            chart.updateChart(NewOptions);
        }
    }

    public fullscreen({ component }: any) {
        const nome = component.name.split('-');
        const nomeFormatado = nome.length > 1 ? nome[nome.length - 2].trim() : '';
        const dashboardInstance = this.getDashboard(nomeFormatado);

        if (dashboardInstance) {
            const botaoFullScreen = Metadata.getInstance<Button>(`fullScreen-card-${nomeFormatado}-${dashboardInstance.currentCardId}`);
            const botaoEsconder = Metadata.getInstance<Button>(`esconder-card-${nomeFormatado}-${dashboardInstance.currentCardId}`);
            const card: string = dashboardInstance.currentCardId!;

            dashboardInstance.toggleFullScreen(card);
            if (botaoFullScreen.iconName === 'mdi-fullscreen') {
            botaoFullScreen.iconName = 'mdi-fullscreen-exit';
            botaoEsconder.disabled = true;
            } else {
                botaoFullScreen.iconName = 'mdi-fullscreen';
                botaoEsconder.disabled = false;
            }
        }
    }

    public async printCard({ component }:any): Promise<void> {
        const nome = component.name.split('-');
        const nomeFormatado = nome.length > 1 ? nome[nome.length - 2].trim() : '';
        const dashboardInstance = this.getDashboard(nomeFormatado);

        if (!dashboardInstance) {
            Utilities.showErrorMessage('Erro ao recuperar dashboardInstance');
            return;
        }

        const cardId = `DashboardGridFrame-${dashboardInstance.currentCardId}-dashboard-${nomeFormatado}`;
        const cardElement = document.getElementById(cardId);

        if (!cardElement) {
            Utilities.showErrorMessage('Erro ao recuperar cardElement');
            return;
        }

        const elementCanva = cardElement.querySelector('canvas');

        if (elementCanva) {
            const originalWidth = elementCanva.width;
            const originalHeight = elementCanva.height;

            const tempCanvas = document.createElement('canvas');
            tempCanvas.width = originalWidth;
            tempCanvas.height = originalHeight;
            const tempCtx = tempCanvas.getContext('2d');

            if (tempCtx) {
                tempCtx.fillStyle = 'white';
                tempCtx.fillRect(0, 0, originalWidth, originalHeight);

                tempCtx.drawImage(elementCanva, 0, 0);

                const canvas2 = tempCanvas.toDataURL();
                this.downloadImage(canvas2, 'chartPrint.png');
            }
        }
    }

    private downloadImage(imageData: string, filename: string): void {
        const xhr = new XMLHttpRequest();
        xhr.responseType = 'blob';

        xhr.onload = () => {
            const blob = xhr.response;
            const url = window.URL.createObjectURL(blob);
            const a = document.createElement('a');

            a.href = url;
            a.download = filename;
            a.style.display = 'none';
            document.body.appendChild(a);
            a.click();
            a.remove();

            window.URL.revokeObjectURL(url);
        };

        xhr.open('GET', imageData);
        xhr.send();
    }

    public disableLegenda(nomeTab: string, cardId: string) {
        const legenda = Metadata.getInstance<Button>(`legend-button-${nomeTab}-${cardId}`);
        legenda.disabled = true;
    }

    public disablePrint(nomeTab: string, cardId: string) {
        const print = Metadata.getInstance<Button>(`print-card-${nomeTab}-${cardId}`);
        print.disabled = true;
    }
}
