import {
    Modal, ModalService, TextInput, LoadingService, DialogService, Form, IGridEvent,
    FileInput,
    Select,
} from '@zeedhi/common';
import {
    IEventParam, Metadata, Http,
    MemoryDatasource,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import { RichText } from '@zeedhi/zd-richtext';

export class FmModeloPopController {
    private exibirInformacoesModeloModal!: Modal;

    public currentRow: any;

    private tituloModal: string = '';

    public resposta: string = '';

    public nrRowAtual: any;

    public rowDeleteAnexo: any = {};

    public habilitarBotao = true;

    public modalErroExcluir!: Modal;

    public modalErroInatividade!: Modal;

    public async createModalPop() {
        this.exibirInformacoesModeloModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_modelopop/fm_modelopop_modal' });
        this.modalErroExcluir = await ModalService.createFromJson('fm_modelopop/fm_modelopop_modal_exclusao', true);
        this.modalErroInatividade = await ModalService.createFromJson('fm_modelopop/fm_modelopop_modal_inatividade', true);
    }

    public async abrirModalErro() {
        this.modalErroExcluir.show();
    }

    public async abrirModalErroInatividade() {
        this.modalErroInatividade.show();
    }

    public fecharModalErro() {
        this.modalErroExcluir.hide();
    }

    public fecharModalInatividade() {
        this.modalErroInatividade.hide();
    }

    public async destruirModais() {
        this.modalErroExcluir.destroy();
        this.modalErroInatividade.destroy();
    }

    public async limparCampos() {
        Metadata.getInstance<Form>('formCadastroModelo').clearForm();
        Metadata.getInstance<Form>('formArquivoCadastroModelo').clearForm();
    }

    public async exibirModalNovoCadastro() {
        await this.exibirInformacoesModeloModal.show();
        this.tituloModal = 'Cadastro Modelo';
        this.limparCampos();
    }

    public async exibirModalCadastroEdicao({ row }: IGridEvent) {
        this.tituloModal = 'Editar Modelo';
        await this.exibirInformacoesModeloModal.show();

        Metadata.getInstance<TextInput>('NMMODELOPOP').value = row.NMMODELOPOP;
        Metadata.getInstance<RichText>('DESCMODELOPOP').value = row.DESCMODELOPOP;
        Metadata.getInstance<TextInput>('LINKMODELOPOP').value = row.LINKMODELOPOP;
        Metadata.getInstance<TextInput>('NRMODELOPOP').value = row.NRMODELOPOP;
        Metadata.getInstance<Select>('IDATIVO').value = row.IDATIVO;
        Metadata.getInstance<TextInput>('NRANEXOMODELOPOP').value = row.NRANEXOMODELOPOP;

        if (row.NMARQUIVO) {
            try {
                const response = await fetch(`/api/getFile?fileName=${encodeURIComponent(row.NMARQUIVO)}`);
                const fileBlob = await response.blob();
                Metadata.getInstance<FileInput>('ARQUIVO').value = new File([fileBlob], row.NMARQUIVO);
            } catch (error) {
                console.error('Error fetching the file:', error);
                Metadata.getInstance<FileInput>('ARQUIVO').value = null;
            }
        } else {
            Metadata.getInstance<FileInput>('ARQUIVO').value = null;
        }
    }

    public destruirModal() {
        this.exibirInformacoesModeloModal?.destroy();
    }

    public async fecharModalPop() {
        this.exibirInformacoesModeloModal.hide();
    }

    public async salvarModeloPop() {
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-modelopop');
        const DESCMODELOPOP = Metadata.getInstance<RichText>('DESCMODELOPOP').value;
        const LINKMODELOPOP = Metadata.getInstance<TextInput>('LINKMODELOPOP').value;
        const NMMODELOPOP = Metadata.getInstance<TextInput>('NMMODELOPOP').value;
        const IDATIVO = Metadata.getInstance<Select>('IDATIVO').value;
        const NRMODELOPOP = Metadata.getInstance<TextInput>('NRMODELOPOP').value;
        const NRANEXOMODELOPOP = Metadata.getInstance<TextInput>('NRANEXOMODELOPOP').value;
        const file = (await Utilities.buildArrayToUpload('ARQUIVO'));
        const camposValidos = (DESCMODELOPOP === '<p></p>' && LINKMODELOPOP === null && file.length <= 0);

        if (camposValidos) {
            Utilities.alertMessage('Preencha pelo menos um modelo de procedimento padrão.', 'error');
            return;
        }

        const postData = {
            NMMODELOPOP,
            NRMODELOPOP,
            NRANEXOMODELOPOP,
            IDATIVO,
            DESCMODELOPOP,
            LINKMODELOPOP,
            ARQUIVO: file,
        };

        LoadingService.show();
        try {
            await Http.post('/salvarModeloPop', postData).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data);
                    this.fecharModalPop();
                } else {
                    Utilities.showSuccessMessage();
                    grid.datasource.get();
                    this.fecharModalPop();
                }
            });
        } catch (error) {
            Utilities.showErrorMessage(error);
        } finally {
            LoadingService.hide();
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmModeloPopController.aposConfirmarDelete', 'Excluir Modelo', 'Deseja realmente excluir os registros selecionados?');
    }

    public async aposConfirmarDelete({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('TekGrid_template_grid-modelopop');
        const selectedRows = grid.selectedRows.map((element: any) => element.NRMODELOPOP);
        LoadingService.show();

        try {
            await Http.post('/deletarAnexoPop', { row: this.rowDeleteAnexo });

            const response = await Http.post('/excluirModeloPop', { ROW: selectedRows });
            DialogService.hide();

            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.tarefas);
            }
        } catch (error) {
            Utilities.showErrorMessage(error);
        } finally {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        }
    }

    public async montarModalErro(tarefas: any) {
        await this.abrirModalErro();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-modelopop');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRCHECKLISTAGEND: element.NRCHECKLISTAGEND,
                NMCHECKLISTAGEND: element.NMCHECKLISTAGEND,
                NMFANTCLIE: element.NMFANTCLIE,
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                IDSITUACAOAGEND: element.IDSITUACAOAGEND,
                DTHORAAGEND: element.DTHORAAGEND,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
